﻿using System;
using System.Collections.Generic;
using System.Text;
using Butterfly.Core;
using Butterfly.HabboHotel;
using Butterfly.Messages;
using Butterfly.Net;
using Butterfly.Storage;

namespace Butterfly
{
    static class ButterflyEnvironment
    {
        private static ConfigurationData Configuration;
        private static DatabaseManager DatabaseManager;
        private static Encoding DefaultEncoding;
        private static TcpConnectionManager ConnectionManager;
        //private static MusSocket MusSocket;
        private static Game Game;
        internal static DateTime ServerStarted;

        internal static string PrettyVersion
        {
            get
            {
                return "ButterFly Emulator v2.0";
            }
        }

        internal static void Initialize()
        {
            ServerStarted = DateTime.Now;
            Console.Title = "Iniciando ButterFly Emulator";
            DefaultEncoding = Encoding.Default;
            Logging.WriteLine("ButterFly v2.0 (Build 2000)");

            Console.WriteLine();
            Console.WriteLine();
            Console.WriteLine();

            try
            {
                Configuration = new ConfigurationData(@"Settings/configuration.ini");

                if (ButterflyEnvironment.GetConfig().data["db.password"].Length == 0)
                {
                    throw new ArgumentException("For security reasons, your MySQL password cannot be left blank. Change your password to start the server.");
                }

                if (ButterflyEnvironment.GetConfig().data["db.password"] == "changeme")
                {
                    throw new ArgumentException("Your MySQL password may not be 'changeme'.\nChange your password to start the server.");
                }

                DateTime Start = DateTime.Now;

                DatabaseServer dbServer = new DatabaseServer(
                    ButterflyEnvironment.GetConfig().data["db.hostname"],
                    uint.Parse(ButterflyEnvironment.GetConfig().data["db.port"]),
                    ButterflyEnvironment.GetConfig().data["db.username"],
                    ButterflyEnvironment.GetConfig().data["db.password"]);

                Database db = new Database(
                    ButterflyEnvironment.GetConfig().data["db.name"],
                    uint.Parse(ButterflyEnvironment.GetConfig().data["db.pool.minsize"]),
                    uint.Parse(ButterflyEnvironment.GetConfig().data["db.pool.maxsize"]));

                DatabaseManager = new DatabaseManager(dbServer, db);
                //MusSocket = new MusSocket(UberEnvironment.GetConfig().data["mus.tcp.bindip"],
                //    int.Parse(UberEnvironment.GetConfig().data["mus.tcp.port"]),
                //    UberEnvironment.GetConfig().data["mus.tcp.allowedaddr"].Split(';'), 20);

                Game = new Game(int.Parse(ButterflyEnvironment.GetConfig().data["game.tcp.conlimit"]));
                Game.ContinueLoading();
                //UberEnvironment.GetGame().GetCatalog().InitCache();

                ConnectionManager = new TcpConnectionManager(
                    int.Parse(ButterflyEnvironment.GetConfig().data["game.tcp.port"]),
                    int.Parse(ButterflyEnvironment.GetConfig().data["game.tcp.conlimit"]));
                ConnectionManager.GetListener().Start();

                TimeSpan TimeUsed = DateTime.Now - Start;
                Logging.WriteLine("Servidor -> ¡Listo! (" + TimeUsed.Seconds + " s, " + TimeUsed.Milliseconds + " ms)");
                Console.Beep();

                //Logging.LogCriticalException("FATAIL FAILURE");

            }
            catch (KeyNotFoundException)
            {
                Logging.WriteLine("Please check your configuration file - some values appear to be missing.");
                Logging.WriteLine("Press any key to shut down ...");

                Console.ReadKey(true);
                ButterflyEnvironment.Destroy();

                return;
            }
            catch (InvalidOperationException e)
            {
                Logging.WriteLine("Failed to initialize ButterflyEmulator: " + e.Message);
                Logging.WriteLine("Press any key to shut down ...");

                Console.ReadKey(true);
                ButterflyEnvironment.Destroy();

                return;
            }
        }

        internal static bool EnumToBool(string Enum)
        {
            return (Enum == "1");
        }

        internal static string BoolToEnum(bool Bool)
        {
            if (Bool)
            {
                return "1";
            }

            return "0";
        }

        internal static int GetRandomNumber(int Min, int Max)
        {
            RandomBase Quick = new Quick();
            return Quick.Next(Min, Max);
        }

        internal static Double GetUnixTimestamp()
        {
            DateTime epoch = new DateTime(1970, 1, 1, 0, 0, 0, 0).AddHours(2);
            DateTime LOL = DateTime.Now;
            TimeSpan sQLOL = LOL - epoch;
            return sQLOL.TotalSeconds;
        }

        internal static string FilterInjectionChars(string Input)
        {
            return FilterInjectionChars(Input, false);
        }

        internal static string FilterInjectionChars(string Input, bool AllowLinebreaks)
        {
            Input = Input.Replace(Convert.ToChar(1), ' ');
            Input = Input.Replace(Convert.ToChar(2), ' ');
            //Input = Input.Replace(Convert.ToChar(3), ' ');
            Input = Input.Replace(Convert.ToChar(9), ' ');

            if (!AllowLinebreaks)
            {
                Input = Input.Replace(Convert.ToChar(13), ' ');
            }

            return Input;
        }

        internal static bool IsValidAlphaNumeric(string inputStr)
        {
            if (string.IsNullOrEmpty(inputStr))
            {
                return false;
            }

            for (int i = 0; i < inputStr.Length; i++)
            {
                if (!(char.IsLetter(inputStr[i])) && (!(char.IsNumber(inputStr[i]))))
                {
                    return false;
                }
            }

            return true;
        }

        internal static ConfigurationData GetConfig()
        {
            return Configuration;
        }

        internal static DatabaseManager GetDatabase()
        {
            return DatabaseManager;
        }

        internal static Encoding GetDefaultEncoding()
        {
            return DefaultEncoding;
        }

        internal static TcpConnectionManager GetConnectionManager()
        {
            return ConnectionManager;
        }

        internal static Game GetGame()
        {
            return Game;
        }

        //internal static Game GameInstance
        //{
        //    get
        //    {
        //        return Game;
        //    }
        //    set
        //    {
        //        Game = value;
        //    }
        //}

        internal static void Destroy()
        {
            Logging.WriteLine("Destruyendo Butterfly...");

            if (GetGame() != null)
            {
                GetGame().Destroy();
                Game = null;
            }

            if (GetConnectionManager() != null)
            {
                Logging.WriteLine("Destruyendo Conexiones");
                GetConnectionManager().GetListener().Destroy();
                GetConnectionManager().DestroyManager();
                //ConnectionManager = null;
            }

            if (GetDatabase() != null)
            {
                try
                {
                    Logging.WriteLine("Destruyendo Conexión con la Base de Datos");
                    //GetDatabase().StopClientMonitor();
                    GetDatabase().DestroyClients();
                    //GetDatabase().DestroyDatabaseManager();
                    //DatabaseManager = null;
                }
                catch { }
            }

            Logging.WriteLine("Uninitialized successfully. Closing.");

            //Environment.Exit(0); Cba :P
        }

        private static bool ShutdownInitiated = false;

        internal static void SendMassMessage(string Message)
        {
            try
            {
                ServerMessage HotelAlert = new ServerMessage(139);
                HotelAlert.AppendStringWithBreak(Message);
                ButterflyEnvironment.GetGame().GetClientManager().BroadcastMessage(HotelAlert);
            }
            catch { }
        }

        internal static void PreformShutDown()
        {
            PreformShutDown(true);
        }

        internal static void PreformShutDown(bool ExitWhenDone)
        {
            if (ShutdownInitiated)
                return;

            ShutdownInitiated = true;

            Logging.DisablePrimaryWriting();
            SendMassMessage("ATENCIÓN:\r\nEl servidor se está apagando. Todos los Furnis de las salas/cambiados/comprados están siendo guardados.");

            Console.WriteLine("Apagando Servidor...");
            GetConnectionManager().GetListener().Stop();
            GetGame().GetClientManager().CloseAll();
            GetGame().GetCatalog().Save();
            Game.GetRoomManager().RemoveAllRooms();

            using (DatabaseClient dbClient = GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE users SET online = '0'");
                dbClient.ExecuteQuery("UPDATE rooms SET users_now = '0'");
            }

            ConnectionManager.Shutdown();

            Game.Destroy();
            try
            {
                Console.WriteLine("Destruyendo Conexión con la Base de Datos");

                GetDatabase().StopMonitor();
                GetDatabase().DestroyClients();
                GetDatabase().DestroyManager();

                //DatabaseManager = null;
            }
            catch { }

            Console.WriteLine("¡Sistema Apagado, Adiós!");
            if (ExitWhenDone)
                Environment.Exit(Environment.ExitCode);
        }
    }
}
