﻿using System;
using System.Collections;
using System.IO;
using System.Text;
using Butterfly_Emulator.ServerManager;

namespace Butterfly.Core
{
    internal static class Logging
    {
        private static bool mDisabled = false;

        internal static void WriteLine(string Line)
        {
            if (!mDisabled)
                Console.WriteLine(Line);
        }

        internal static void LogException(string logText)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\exceptions.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(logText + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
            }
            catch (Exception) 
            { 
                WriteLine(logText); 
            }

            WriteLine("Exception has been saved");
        }

        internal static void LogCriticalException(string logText)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\criticalexceptions.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(logText + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();

                WriteLine("CRITICAL ERROR LOGGED");
            }
            catch (Exception) 
            {
                WriteLine(logText); 
            }
        }

        internal static void LogCacheError(string logText)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\cacheerror.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(logText + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
            }
            catch (Exception) 
            { 
                WriteLine(logText); 
            }

            WriteLine("Critical error saved");
        }

        internal static void LogMessage(string logText)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\logg.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(logText + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
            }
            catch { }

            WriteLine(logText);
        }

        internal static void LogDDOS(string logText)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\ddos.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(logText + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
            }
            catch { }

            WriteLine(logText);
        }

        internal static void LogThreadException(string Exception, string Threadname)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(@"Logs\threaderror.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes("Error in thread " + Threadname + ": \r\n" + Exception + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
                Logging.WriteLine("Error in " + Threadname + " caught");
            }
            catch (Exception) 
            { 
                WriteLine(Exception); 
            }
        }

        //internal static void LogCodeWhichTakesLotsOfTimeToRun(string Data)
        //{
        //    try
        //    {
        //        FileStream errWriter = new System.IO.FileStream(@"Logs\fail.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
        //        byte[] Msg = ASCIIEncoding.ASCII.GetBytes(Data + "\r\n\r\n");
        //        errWriter.Write(Msg, 0, Msg.Length);
        //        errWriter.Dispose();
        //        Logging.WriteLine("Long time logged!");
        //    }
        //    catch (Exception)
        //    {
        //        WriteLine(Data);
        //    }
        //}

        internal static void LogPacketException(string Packet, string Exception)
        {
            try
            {
                SessionManagement.IncreaseError();
                FileStream errWriter = new System.IO.FileStream(@"Logs\packeterror.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes("Error in packet " + Packet + ": \r\n" + Exception + "\r\n\r\n");
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
                Logging.WriteLine("User disconnection logged");
            }
            catch (Exception) 
            { 
                WriteLine(Exception); 
            }
        }

        internal static void HandleException(Exception pException, string pLocation)
        {
            StringBuilder ExceptionData = new StringBuilder();
            ExceptionData.AppendLine("Exception logged " + DateTime.Now.ToString() + " in " + pLocation + ":");
            ExceptionData.AppendLine(pException.ToString());
            if (pException.InnerException != null)
            {
                ExceptionData.AppendLine("Inner exception:");
                ExceptionData.AppendLine(pException.InnerException.ToString());
            }
            if (pException.HelpLink != null)
            {
                ExceptionData.AppendLine("Help link:");
                ExceptionData.AppendLine(pException.HelpLink);
            }
            if (pException.Source != null)
            {
                ExceptionData.AppendLine("Source:");
                ExceptionData.AppendLine(pException.Source);
            }
            if (pException.Data != null)
            {
                ExceptionData.AppendLine("Data:");
                foreach (DictionaryEntry Entry in pException.Data)
                {
                    ExceptionData.AppendLine("  Key: " + Entry.Key + "Value: " + Entry.Value);
                }
            }
            if (pException.Message != null)
            {
                ExceptionData.AppendLine("Message:");
                ExceptionData.AppendLine(pException.Message);
            }
            if (pException.StackTrace != null)
            {
                ExceptionData.AppendLine("Stack trace:");
                ExceptionData.AppendLine(pException.StackTrace);
            }
            ExceptionData.AppendLine();
            ExceptionData.AppendLine();
            LogException(ExceptionData.ToString());
        }

        internal static void DisablePrimaryWriting()
        {
            mDisabled = true;
            Console.Clear();
        }

        internal static void LogShutdownError(string p)
        {
            ///throw new NotImplementedException();
        }
    }
}
