﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Threading;
using System.Threading.Tasks;
using Butterfly.Core;
using Butterfly.HabboHotel.Users.Subscriptions;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pets;
using Butterfly.Messages;
using Butterfly.Storage;
using Butterfly_Emulator.HabboHotel.Items;

namespace Butterfly.HabboHotel.Catalogs
{
    class Catalog
    {
        internal Dictionary<int, CatalogPage> Pages;
        internal List<EcotronReward> EcotronRewards;

        private Marketplace Marketplace;

        private ServerMessage[] mCataIndexCache;
        private Task mFurniIDCYcler;

        internal Catalog()
        {
            Marketplace = new Marketplace();
        }

        private void UpdateID()
        {
            uint dbval = mCacheID;

            while (true)
            {
                try
                {
                    if (dbval != mCacheID)
                    {
                        using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                            dbClient.ExecuteQuery("UPDATE item_id_generator SET id_generator = '" + mCacheID + "' LIMIT 1");
                        dbval = mCacheID;
                    }
                }
                catch (Exception e)
                {
                    Logging.LogThreadException(e.ToString(), "item_id_generator updater task");
                }
                Thread.Sleep(10000);
            }
        }

        internal void Save()
        {
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                dbClient.ExecuteQuery("UPDATE item_id_generator SET id_generator = '" + mCacheID + "' LIMIT 1");
        }

        internal void Initialize(DatabaseClient dbClient)
        {
            Pages = new Dictionary<int, CatalogPage>();
            EcotronRewards = new List<EcotronReward>();

            DataTable Data = dbClient.ReadDataTable("SELECT * FROM catalog_pages ORDER BY order_num ASC");
            DataTable EcoData = dbClient.ReadDataTable("SELECT * FROM ecotron_rewards ORDER BY item_id");
            mCacheID = (UInt32)dbClient.ReadDataRow("SELECT id_generator FROM item_id_generator LIMIT 1")[0];

            Hashtable CataItems = new Hashtable();
            DataTable CatalogueItems = dbClient.ReadDataTable("SELECT id,item_ids,catalog_name,cost_credits,cost_pixels,amount,page_id,cost_snow,hc_state FROM catalog_items");

            if (CatalogueItems != null)
            {
                foreach (DataRow Row in CatalogueItems.Rows)
                {
                    if (string.IsNullOrEmpty(Row["item_ids"].ToString()) || (int)Row["amount"] <= 0)
                    {
                        continue;
                    }
                    CataItems.Add((uint)Row["id"], new CatalogItem((uint)Row["id"], (string)Row["catalog_name"], (string)Row["item_ids"], (int)Row["cost_credits"], (int)Row["cost_pixels"], (int)Row["amount"], (int)Row["page_id"], (int)Row["cost_snow"], int.Parse(Row["hc_state"].ToString())));
                    //Items.Add(new CatalogItem((uint)Row["id"], (string)Row["catalog_name"], (string)Row["item_ids"], (int)Row["cost_credits"], (int)Row["cost_pixels"], (int)Row["amount"]));
                }
            }


            mFurniIDCYcler = new Task(UpdateID);
            mFurniIDCYcler.Start();

            if (Data != null)
            {
                foreach (DataRow Row in Data.Rows)
                {
                    Boolean Visible = false;
                    Boolean Enabled = false;
                    Boolean ComingSoon = false;

                    if (Row["visible"].ToString() == "1")
                    {
                        Visible = true;
                    }

                    if (Row["enabled"].ToString() == "1")
                    {
                        Enabled = true;
                    }

                    if (Row["coming_soon"].ToString() == "1")
                    {
                        ComingSoon = true;
                    }

                    Pages.Add((int)Row["id"], new CatalogPage((int)Row["id"], (int)Row["parent_id"],
                        (string)Row["caption"], Visible, Enabled, ComingSoon, (uint)Row["min_rank"],
                        ButterflyEnvironment.EnumToBool(Row["club_only"].ToString()), (int)Row["icon_color"],
                        (int)Row["icon_image"], (string)Row["page_layout"], (string)Row["page_headline"],
                        (string)Row["page_teaser"], (string)Row["page_special"], (string)Row["page_text1"],
                        (string)Row["page_text2"], (string)Row["page_text_details"], (string)Row["page_text_teaser"], ref CataItems));
                }
            }

            if (EcoData != null)
            {
                foreach (DataRow Row in EcoData.Rows)
                {
                    EcotronRewards.Add(new EcotronReward((uint)Row["display_id"], (uint)Row["item_id"], (uint)Row["reward_level"]));
                }
            }
        }

        internal void InitCache()
        {
            mCataIndexCache = new ServerMessage[8]; //Max 7 ranks

            for (int i = 1; i < 8; i++)
            {
                mCataIndexCache[i] = SerializeIndexForCache(i);
            }

            foreach (CatalogPage Page in Pages.Values)
            {
                Page.InitMsg();
            }
        }

        internal CatalogItem FindItem(uint ItemId)
        {
            foreach (CatalogPage Page in Pages.Values)
            {
                foreach (CatalogItem Item in Page.Items)
                {
                    if (Item.Id == ItemId)
                    {
                        return Item;
                    }
                }
            }

            return null;
        }

        //internal Boolean IsItemInCatalog(uint BaseId)
        //{
        //    DataRow Row = null;

        //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //    {
        //        Row = dbClient.ReadDataRow("SELECT id FROM catalog_items WHERE item_ids = '" + BaseId + "' LIMIT 1");
        //    }

        //    if (Row != null)
        //    {
        //        return true;
        //    }

        //    return false;
        //}

        internal int GetTreeSize(int rank, int TreeId)
        {
            int i = 0;

            foreach (CatalogPage Page in Pages.Values)
            {
                if (Page.MinRank > rank)
                {
                    continue;
                }

                if (Page.ParentId == TreeId)
                {
                    i++;
                }
            }


            return i;
        }

        internal CatalogPage GetPage(int Page)
        {
            if (!Pages.ContainsKey(Page))
            {
                return null;
            }

            return Pages[Page];
        }

        internal void HandlePurchase(GameClient Session, int PageId, uint ItemId, string ExtraData, Boolean IsGift, string GiftUser, string GiftMessage)
        {
            CatalogPage Page = GetPage(PageId);

            if (Page == null || Page.ComingSoon || !Page.Enabled || !Page.Visible)
            {
                return;
            }

            if (Page.ClubOnly && !Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_club"))
            {
                return;
            }

            CatalogItem Item = Page.GetItem(ItemId);
            uint GiftUserId = 0;
            CatalogItem uItems;

            if (Item != null)
            {
                uItems = Item;
            }
            else
            {
                using (DatabaseClient MySQL = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    DataRow Row = MySQL.ReadDataRow("SELECT * FROM catalog_items WHERE id = " + ItemId);
                    uItems = new CatalogItem((uint)Row["id"], (string)Row["catalog_name"], (string)Row["item_ids"], (int)Row["cost_credits"], (int)Row["cost_pixels"], (int)Row["amount"], (int)Row["page_id"], (int)Row["cost_snow"], int.Parse(Row["hc_state"].ToString()));
                }
            }

            if (uItems.Name.Contains("HABBO_CLUB"))
            {
                #region Purchase Club!
                // PAYmQHABBO_CLUB_BASIC_1_MONTHSCHHISGZvGSBSE[mQHABBO_CLUB_VIP_1_MONTHQFHIISGZvGSBSEXnQHABBO_CLUB_VIP_3_MONTHSPOHIKQW[vGIPFZmQHABBO_CLUB_BASIC_3_MONTHSQKHHKQW[vGIPF
                int TypeOfClub = 0;
                Subscription Sub;
                if (Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_vip"))
                    Sub = Session.GetHabbo().GetSubscriptionManager().GetSubscription("habbo_vip");
                else if (Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_club"))
                    Sub = Session.GetHabbo().GetSubscriptionManager().GetSubscription("habbo_club");
                else
                    Sub = null;
                if (uItems.Name == "HABBO_CLUB_BASIC_1_MONTH")
                {
                    TypeOfClub = 1;
                }
                else if (uItems.Name.Contains("HABBO_CLUB_BASIC_3"))
                {
                    TypeOfClub = 2;
                }
                else if (uItems.Name == "HABBO_CLUB_VIP_1_MONTH")
                {
                    TypeOfClub = 3;
                }
                else if (uItems.Name.Contains("HABBO_CLUB_VIP_3"))
                {
                    TypeOfClub = 4;
                }
                else if (uItems.Name == "HABBO_CLUB_UPGRADE_1")
                {
                    TypeOfClub = 5;
                }
                else if (uItems.Name == "HABBO_CLUB_UPGRADE_3")
                {
                    TypeOfClub = 6;
                }

                if (TypeOfClub == 1)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_club", (60 * 60 * 24 * 31));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(1);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }
                else if (TypeOfClub == 2)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_club", (60 * 60 * 24 * 31 * 3));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(1);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }
                else if (TypeOfClub == 3)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_vip", (60 * 60 * 24 * 31));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(2);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }
                else if (TypeOfClub == 4)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_vip", (60 * 60 * 24 * 31 * 3));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(2);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }
                else if (TypeOfClub == 5)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_vip", (60 * 60 * 24 * 31));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(2);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }
                else if (TypeOfClub == 6)
                {
                    Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_vip", (60 * 60 * 24 * 31 * 3));
                    ServerMessage FuseRight = new ServerMessage(2);
                    FuseRight.AppendInt32(2);
                    Session.SendMessage(FuseRight);
                    Session.GetMessageHandler().GetSubscriptionData();
                }

                bool CreditsFail = false;
                bool PixelsFail = false;

                if (Session.GetHabbo().Credits < uItems.CreditsCost)
                {
                    CreditsFail = true;
                }

                if (CreditsFail || PixelsFail)
                {
                    ServerMessage Failed = new ServerMessage(68);
                    Failed.AppendBoolean(CreditsFail);
                    Failed.AppendBoolean(PixelsFail);
                    Session.SendMessage(Failed);
                    return;
                }

                if (uItems.CreditsCost > 0)
                {
                    int NewCredits = Session.GetHabbo().Credits - uItems.CreditsCost;
                    Session.GetHabbo().UpdateCreditsBalance();
                }

                ServerMessage PurchaseClub = new ServerMessage(67);
                //AC[mQHABBO_CLUB_VIP_1_MONTH{2}QFHHH{1}
                PurchaseClub.AppendUInt(uItems.Id);
                PurchaseClub.AppendStringWithBreak(uItems.Name);
                PurchaseClub.AppendInt32(uItems.CreditsCost);
                PurchaseClub.AppendBoolean(true);
                PurchaseClub.AppendBoolean(false);
                PurchaseClub.AppendBoolean(false);
                Session.SendMessage(PurchaseClub);
                #endregion
                return;
            }

            if (Item == null)
            {
                return;
            }

            if (IsGift)
            {
                if (!Item.GetBaseItem().AllowGift)
                {
                    return;
                }

                Logging.LogMessage(Session.GetHabbo().Username + " sent " + GiftUser + " a gift!");

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.AddParamWithValue("gift_user", GiftUser);

                    try
                    {
                        GiftUserId = (uint)dbClient.ReadDataRow("SELECT id FROM users WHERE username = @gift_user LIMIT 1")[0];
                    }
                    catch (Exception) { }
                }

                if (GiftUserId == 0)
                {
                    Session.GetMessageHandler().GetResponse().Init(76);
                    Session.GetMessageHandler().GetResponse().AppendBoolean(true);
                    Session.GetMessageHandler().GetResponse().AppendStringWithBreak(GiftUser);
                    Session.GetMessageHandler().SendResponse();

                    return;
                }
            }

            Boolean CreditsError = false;
            Boolean PixelError = false;

            if (Session.GetHabbo().Credits < Item.CreditsCost)
            {
                CreditsError = true;
            }

            if (Session.GetHabbo().ActivityPoints < Item.PixelsCost)
            {
                PixelError = true;
            }

            if (CreditsError || PixelError)
            {
                Session.GetMessageHandler().GetResponse().Init(68);
                Session.GetMessageHandler().GetResponse().AppendBoolean(CreditsError);
                Session.GetMessageHandler().GetResponse().AppendBoolean(PixelError);
                Session.GetMessageHandler().SendResponse();

                return;
            }

            if (IsGift && Item.GetBaseItem().Type == 'e')
            {
                Session.SendNotif("You can not send this item as a gift.");
                return;
            }
            //Console.WriteLine(Item.GetBaseItem().ItemId);
            //Console.WriteLine(Item.GetBaseItem().InteractionType.ToLower());
            // Extra Data is _NOT_ filtered at this point and MUST BE VERIFIED BELOW:
            switch (Item.GetBaseItem().InteractionType)
            {
                case InteractionType.none:
                    ExtraData = "";
                    break;

                #region Pet handling
                case InteractionType.pet0:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }
                        

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception e) { Logging.WriteLine(e.ToString()); return; }

                    break;

                case InteractionType.pet1:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet2:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet3:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet4:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet5:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }
                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet6:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }
                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;

                case InteractionType.pet7:

                    try
                    {
                        int Belcredits;
                        using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            Belcredits = dbclient.ReadInt32("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id + ";");
                        }
                        if (Belcredits < 20)
                        {
                            Session.SendNotif("You don't have enough belcredits to buy a pet");
                            return;
                        }
                        else
                        {
                            using (DatabaseClient dbclient = ButterflyEnvironment.GetDatabase().GetClient())
                            {
                                dbclient.ExecuteQuery("UPDATE users SET belcredits = belcredits - 20 WHERE id = " + Session.GetHabbo().Id + ";");
                            }
                        }
                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                        {
                            return;
                        }

                        if (Race.Length != 3)
                        {
                            return;
                        }

                        if (Color.Length != 6)
                        {
                            return;
                        }
                    }
                    catch (Exception) { return; }

                    break;
                #endregion

                case InteractionType.roomeffect:

                    Double Number = 0;

                    try
                    {
                        Number = Double.Parse(ExtraData);
                    }
                    catch (Exception) { }

                    ExtraData = Number.ToString().Replace(',', '.');
                    break; // maintain extra data // todo: validate

                case InteractionType.postit:
                    ExtraData = "FFFF33";
                    break;

                case InteractionType.dimmer:
                    ExtraData = "1,1,1,#000000,255";
                    break;

                case InteractionType.trophy:
                    ExtraData = Session.GetHabbo().Username + Convert.ToChar(9) + DateTime.Now.Day + "-" + DateTime.Now.Month + "-" + DateTime.Now.Year + Convert.ToChar(9) + ButterflyEnvironment.FilterInjectionChars(ExtraData, true);
                    break;

                default:
                    ExtraData = "";
                    break;
            }

            if (Item.CreditsCost > 0)
            {
                Session.GetHabbo().Credits -= Item.CreditsCost;
                Session.GetHabbo().UpdateCreditsBalance();
            }

            if (Item.PixelsCost > 0)
            {
                Session.GetHabbo().ActivityPoints -= Item.PixelsCost;
                Session.GetHabbo().UpdateActivityPointsBalance(true);
            }

            Session.GetMessageHandler().GetResponse().Init(67);
            Session.GetMessageHandler().GetResponse().AppendUInt(Item.GetBaseItem().ItemId);
            Session.GetMessageHandler().GetResponse().AppendStringWithBreak(Item.GetBaseItem().Name);
            Session.GetMessageHandler().GetResponse().AppendInt32(Item.CreditsCost);
            Session.GetMessageHandler().GetResponse().AppendInt32(Item.PixelsCost);
            Session.GetMessageHandler().GetResponse().AppendInt32(0);
            Session.GetMessageHandler().GetResponse().AppendInt32(1);
            Session.GetMessageHandler().GetResponse().AppendStringWithBreak(Item.GetBaseItem().Type.ToString());
            Session.GetMessageHandler().GetResponse().AppendInt32(Item.GetBaseItem().SpriteId);
            Session.GetMessageHandler().GetResponse().AppendStringWithBreak("");
            Session.GetMessageHandler().GetResponse().AppendInt32(1);
            Session.GetMessageHandler().GetResponse().AppendInt32(-1);
            Session.GetMessageHandler().GetResponse().AppendStringWithBreak("");
            Session.GetMessageHandler().SendResponse();

            if (IsGift)
            {
                uint GenId = GenerateItemId();
                Item Present = GeneratePresent();

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.AddParamWithValue("gift_message", "!" + GiftMessage);
                    dbClient.AddParamWithValue("extra_data", ExtraData);

                    dbClient.ExecuteQuery("INSERT INTO user_items (id,user_id,base_item,extra_data) VALUES ('" + GenId + "','" + GiftUserId + "','" + Present.ItemId + "',@gift_message)");
                    dbClient.ExecuteQuery("INSERT INTO user_presents (item_id,base_id,amount,extra_data) VALUES ('" + GenId + "','" + Item.GetBaseItem().ItemId + "','" + Item.Amount + "',@extra_data)");
                }

                GameClient Receiver = ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(GiftUserId);

                if (Receiver != null)
                {
                    Receiver.SendNotif("You have received a gift! Check your inventory.");
                    Receiver.GetHabbo().GetInventoryComponent().UpdateItems(true);
                }

                Session.SendNotif("Gift sent successfully!");
            }
            else
            {
                DeliverItems(Session, Item.GetBaseItem(), Item.Amount, ExtraData);
            }
        }

        internal static bool CheckPetName(string PetName)
        {
            if (PetName.Length < 1 || PetName.Length > 16)
            {
                return false;
            }

            if (!ButterflyEnvironment.IsValidAlphaNumeric(PetName))
            {
                return false;
            }

            return true;
        }

        internal void DeliverItems(GameClient Session, Item Item, int Amount, String ExtraData)
        {
            switch (Item.Type.ToString())
            {
                case "i":
                case "s":

                    for (int i = 0; i < Amount; i++)
                    {
                        uint GeneratedId = GenerateItemId();
                        switch (Item.InteractionType)
                        {
                            case InteractionType.pet0:
                            

                                string[] PetData = ExtraData.Split('\n');

                                Pet GeneratedPet = CreatePet(Session.GetHabbo().Id, PetData[0], 0, PetData[1], PetData[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;



                            case InteractionType.pet1:

                                string[] PetData1 = ExtraData.Split('\n');

                                Pet GeneratedPet1 = CreatePet(Session.GetHabbo().Id, PetData1[0], 1, PetData1[1], PetData1[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet1);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;

                            case InteractionType.pet2:
                                
                                string[] PetData5 = ExtraData.Split('\n');

                                Pet GeneratedPet5 = CreatePet(Session.GetHabbo().Id, PetData5[0], 5, PetData5[1], PetData5[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet5);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;



                            case InteractionType.pet3:

                                string[] PetData2 = ExtraData.Split('\n');

                                Pet GeneratedPet2 = CreatePet(Session.GetHabbo().Id, PetData2[0], 2, PetData2[1], PetData2[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet2);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;

                            case InteractionType.pet4:

                                string[] PetData3 = ExtraData.Split('\n');

                                Pet GeneratedPet3 = CreatePet(Session.GetHabbo().Id, PetData3[0], 3, PetData3[1], PetData3[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet3);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;

                            case InteractionType.pet5:
                                
                                string[] PetData7 = ExtraData.Split('\n');

                                Pet GeneratedPet7 = CreatePet(Session.GetHabbo().Id, PetData7[0], 7, PetData7[1], PetData7[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet7);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;

                            case InteractionType.pet6:
                                string[] PetData4 = ExtraData.Split('\n');

                                Pet GeneratedPet4 = CreatePet(Session.GetHabbo().Id, PetData4[0], 4, PetData4[1], PetData4[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet4);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");


                                break;


                            case InteractionType.pet7:

                                string[] PetData6 = ExtraData.Split('\n');

                                Pet GeneratedPet6 = CreatePet(Session.GetHabbo().Id, PetData6[0], 6, PetData6[1], PetData6[2]);

                                Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet6);
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                                break;

                            //case InteractionType.pet8:

                            //    string[] PetData7 = ExtraData.Split('\n');

                            //    Pet GeneratedPet7 = CreatePet(Session.GetHabbo().Id, PetData7[0], 7, PetData7[1], PetData7[2]);

                            //    Session.GetHabbo().GetInventoryComponent().AddPet(GeneratedPet7);
                            //    Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, 320, "0");

                            //    break;

                            case InteractionType.teleport:

                                uint TeleTwo = GenerateItemId();

                                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                                {
                                    dbClient.ExecuteQuery("INSERT INTO tele_links (tele_one_id,tele_two_id) VALUES ('" + GeneratedId + "','" + TeleTwo + "')");
                                    dbClient.ExecuteQuery("INSERT INTO tele_links (tele_one_id,tele_two_id) VALUES ('" + TeleTwo + "','" + GeneratedId + "')");
                                }

                                Session.GetHabbo().GetInventoryComponent().AddItem(TeleTwo, Item.ItemId, "0");
                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, Item.ItemId, "0");
                                break;

                            case InteractionType.dimmer:

                                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                                {
                                    dbClient.ExecuteQuery("INSERT INTO room_items_moodlight (item_id,enabled,current_preset,preset_one,preset_two,preset_three) VALUES ('" + GeneratedId + "','0','1','#000000,255,0','#000000,255,0','#000000,255,0')");
                                }

                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, Item.ItemId, ExtraData);
                                break;

                            default:

                                Session.GetHabbo().GetInventoryComponent().AddItem(GeneratedId, Item.ItemId, ExtraData);
                                break;
                        }
                    }

                    Session.GetHabbo().GetInventoryComponent().UpdateItems(false);
                    break;

                case "e":

                    for (int i = 0; i < Amount; i++)
                    {
                        Session.GetHabbo().GetAvatarEffectsInventoryComponent().AddEffect(Item.SpriteId, 3600);
                    }

                    break;

                case "h":

                    for (int i = 0; i < Amount; i++)
                    {
                        Session.GetHabbo().GetSubscriptionManager().AddOrExtendSubscription("habbo_club", 2678400);
                    }

                    if (!Session.GetHabbo().GetBadgeComponent().HasBadge("HC1"))
                    {
                        Session.GetHabbo().GetBadgeComponent().GiveBadge("HC1", true);
                    }

                    Session.GetMessageHandler().GetResponse().Init(7);
                    Session.GetMessageHandler().GetResponse().AppendStringWithBreak("habbo_club");

                    if (Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_club"))
                    {
                        Double Expire = Session.GetHabbo().GetSubscriptionManager().GetSubscription("habbo_club").ExpireTime;
                        Double TimeLeft = Expire - ButterflyEnvironment.GetUnixTimestamp();
                        int TotalDaysLeft = (int)Math.Ceiling(TimeLeft / 86400);
                        int MonthsLeft = TotalDaysLeft / 31;

                        if (MonthsLeft >= 1) MonthsLeft--;

                        Session.GetMessageHandler().GetResponse().AppendInt32(TotalDaysLeft - (MonthsLeft * 31));
                        Session.GetMessageHandler().GetResponse().AppendBoolean(true);
                        Session.GetMessageHandler().GetResponse().AppendInt32(MonthsLeft);
                    }
                    else
                    {
                        for (int i = 0; i < 3; i++)
                        {
                            Session.GetMessageHandler().GetResponse().AppendInt32(0);
                        }
                    }

                    Session.GetMessageHandler().SendResponse();

                    List<string> Rights = ButterflyEnvironment.GetGame().GetRoleManager().GetRightsForHabbo(Session.GetHabbo());

                    Session.GetMessageHandler().GetResponse().Init(2);
                    if (Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_vip"))
                        Session.GetMessageHandler().GetResponse().AppendInt32(2);
                    else if (Session.GetHabbo().GetSubscriptionManager().HasSubscription("habbo_club"))
                        Session.GetMessageHandler().GetResponse().AppendInt32(1);
                    else
                        Session.GetMessageHandler().GetResponse().AppendInt32(0);
                    /*
                    Session.GetMessageHandler().GetResponse().AppendInt32(Rights.Count);

                    foreach (string Right in Rights)
                    {
                        Session.GetMessageHandler().GetResponse().AppendStringWithBreak(Right);
                    }
                    */
                    Session.GetMessageHandler().SendResponse();

                    break;

                default:

                    Session.SendNotif("Something went wrong! The item type could not be processed. Please do not try to buy this item anymore, instead inform support as soon as possible.");
                    break;
            }
        }

        internal static Item GeneratePresent()
        {
            int Random = ButterflyEnvironment.GetRandomNumber(0, 6);

            switch (Random)
            {
                case 1:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(165); // present_gen1

                case 2:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(166); // present_gen2

                case 3:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(167); // present_gen3

                case 4:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(168); // present_gen4

                case 5:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(169); // present_gen5

                case 6:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(170); // present_gen6

                case 0:
                default:

                    return ButterflyEnvironment.GetGame().GetItemManager().GetItem(164); // present_gen
            }
        }

        internal Pet CreatePet(uint UserId, string Name, int Type, string Race, string Color)
        {

            Pet NewPet = new Pet(GenerateItemId(), UserId, 0, Name, (uint)Type, Race, Color, 0, 100, 100, 0, ButterflyEnvironment.GetUnixTimestamp(), 0, 0, 0.0);
            NewPet.DBState = DatabaseUpdateState.NeedsInsert;
            return NewPet;
            //DataRow Row = null;

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.AddParamWithValue("userid", UserId);
            //    dbClient.AddParamWithValue("name", Name);
            //    dbClient.AddParamWithValue("type", Type);
            //    dbClient.AddParamWithValue("race", Race);
            //    dbClient.AddParamWithValue("color", Color);
            //    dbClient.AddParamWithValue("createstamp", UberEnvironment.GetUnixTimestamp());
            //    dbClient.ReadDataRow("INSERT INTO user_pets (user_id,name,type,race,color,expirience,energy,createstamp) VALUES (@userid,@name,@type,@race,@color,0,100,@createstamp)");
            //}

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.AddParamWithValue("userid", UserId);
            //    dbClient.AddParamWithValue("name", Name);
            //    Row = dbClient.ReadDataRow("SELECT id, user_id, room_id, name, type, race, color, expirience, energy, nutrition, respect, createstamp, x, y, z FROM user_pets WHERE user_id = @userid AND name = @name LIMIT 1");
            //}

            //return this.GeneratePetFromRow(Row, GenerateItemId());
        }

        internal static Pet GeneratePetFromRow(DataRow Row)
        {
            if (Row == null)
            {
                return null;
            }

            return new Pet((uint)Row["id"], (uint)Row["user_id"], (uint)Row["room_id"], (string)Row["name"], (uint)Row["type"], (string)Row["race"], (string)Row["color"], (int)Row["expirience"], (int)Row["energy"], (int)Row["nutrition"], (int)Row["respect"], (double)Row["createstamp"], (int)Row["x"], (int)Row["y"], (double)Row["z"]);
        }

        //internal Pet GeneratePetFromRow(DataRow Row, uint PetID)
        //{
        //    if (Row == null)
        //        return null;

        //    return new Pet(PetID, (uint)Row["user_id"], (uint)Row["room_id"], (string)Row["name"], (uint)Row["type"], (string)Row["race"], (string)Row["color"], (int)Row["expirience"], (int)Row["energy"], (int)Row["nutrition"], (int)Row["respect"], (double)Row["createstamp"], (int)Row["x"], (int)Row["y"], (double)Row["z"]);
        //}

        private uint mCacheID = 0;
        internal uint GenerateItemId()
        {
            //uint i = 0;

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    i = mCacheID++;
            //    dbClient.ExecuteQuery("UPDATE item_id_generator SET id_generator = '" + mCacheID + "' LIMIT 1");
            //}

            return mCacheID++;
        }

        internal EcotronReward GetRandomEcotronReward()
        {
            uint Level = 1;

            if (ButterflyEnvironment.GetRandomNumber(1, 2000) == 2000)
            {
                Level = 5;
            }
            else if (ButterflyEnvironment.GetRandomNumber(1, 200) == 200)
            {
                Level = 4;
            }
            else if (ButterflyEnvironment.GetRandomNumber(1, 40) == 40)
            {
                Level = 3;
            }
            else if (ButterflyEnvironment.GetRandomNumber(1, 4) == 4)
            {
                Level = 2;
            }

            List<EcotronReward> PossibleRewards = GetEcotronRewardsForLevel(Level);

            if (PossibleRewards != null && PossibleRewards.Count >= 1)
            {
                return PossibleRewards[ButterflyEnvironment.GetRandomNumber(0, (PossibleRewards.Count - 1))];
            }
            else
            {
                return new EcotronReward(0, 1479, 0); // eco lamp two :D
            }
        }

        internal List<EcotronReward> GetEcotronRewardsForLevel(uint Level)
        {
            List<EcotronReward> Rewards = new List<EcotronReward>();

            foreach (EcotronReward R in EcotronRewards)
            {
                if (R.RewardLevel == Level)
                {
                    Rewards.Add(R);
                }
            }


            return Rewards;
        }

        internal ServerMessage SerializeIndexForCache(int rank)
        {
            ServerMessage Index = new ServerMessage(126);
            Index.AppendBoolean(true);
            Index.AppendInt32(0);
            Index.AppendInt32(0);
            Index.AppendInt32(-1);
            Index.AppendStringWithBreak("");
            Index.AppendInt32(GetTreeSize(rank, -1));

            foreach (CatalogPage Page in Pages.Values)
            {
                if (Page.ParentId != -1)
                {
                    continue;
                }

                Page.Serialize(rank, Index);

                foreach (CatalogPage _Page in Pages.Values)
                {
                    if (_Page.ParentId != Page.PageId)
                    {
                        continue;
                    }

                    _Page.Serialize(rank, Index);
                }
            }


            return Index;
        }

        internal ServerMessage GetIndexMessageForRank(uint Rank)
        {
            if (Rank < 1)
                Rank = 1;
            if (Rank > 7)
                Rank = 7;

            return mCataIndexCache[Rank];
        }

        internal ServerMessage SerializePage(CatalogPage Page)
        {
            ServerMessage PageData = new ServerMessage(127);
            PageData.AppendInt32(Page.PageId);

            switch (Page.Layout)
            {
                case "frontpage":

                    PageData.AppendStringWithBreak("frontpage3");
                    PageData.AppendInt32(3);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendStringWithBreak(Page.LayoutTeaser);
                    PageData.AppendStringWithBreak("");
                    PageData.AppendInt32(10);
                    PageData.AppendStringWithBreak(Page.Text1 + Convert.ToChar('\n') + Page.Text2);
                    PageData.AppendStringWithBreak("Más información >>");
                    PageData.AppendStringWithBreak("¿Cómo conseguir créditos?" + Convert.ToChar(2) + "1. No te hace falta conseguirlos, el hotel es gratis.\n2. ¡No hay problema, si quieres puedes canjear créditos por códigos aquí debajo!.\n3. ¿A qué esperas para unirte ya a la comunidad y participar?.\n\n4. Para más info, vuelve a la home e investiga." + Convert.ToChar(2) + "Canjea tu código aquí:");
                    PageData.AppendStringWithBreak("");
                    PageData.AppendStringWithBreak("#FAF8CC");
                    PageData.AppendStringWithBreak("#FAF8CC");
                    PageData.AppendStringWithBreak("Echa un vistazo >>");
                    PageData.AppendStringWithBreak("magic.credits");

                    break;

                case "club_buy":
                    // There is a interessant page :D
                    PageData.AppendStringWithBreak("club_buy");
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak("hc2_clubtitle");
                    break;

                case "club_gifts":
                    // AcgaCclub_gifts{2}Ihc2_clubtitle{2}IEcha un vistazo a los regalos HC y VIP. Si eres HC, podrÃ¡s elegir entre una selecciÃ³n de regalos HC. Si eres VIP, entre una selecciÃ³n de regalos VIP.{2}HM{1}
                    PageData.AppendStringWithBreak("club_gifts");
                    PageData.AppendBoolean(true);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendBoolean(true);
                    PageData.AppendStringWithBreak(Page.Text1);
                    PageData.AppendBoolean(false);
                    PageData.AppendString("M");
                    break;

                case "recycler_info":

                    PageData.AppendStringWithBreak(Page.Layout);
                    PageData.AppendInt32(2);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendStringWithBreak(Page.LayoutTeaser);
                    PageData.AppendInt32(3);
                    PageData.AppendStringWithBreak(Page.Text1);
                    PageData.AppendStringWithBreak(Page.Text2);
                    PageData.AppendStringWithBreak(Page.TextDetails);

                    break;

                case "recycler_prizes":

                    // Ac@aArecycler_prizesIcatalog_recycler_headline3IDe Ecotron geeft altijd een van deze beloningen:H
                    PageData.AppendStringWithBreak("recycler_prizes");
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak("catalog_recycler_headline3");
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak(Page.Text1);

                    break;

                case "spaces":

                    PageData.AppendStringWithBreak("spaces_new");
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak(Page.Text1);

                    break;

                case "recycler":

                    PageData.AppendStringWithBreak(Page.Layout);
                    PageData.AppendInt32(2);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendStringWithBreak(Page.LayoutTeaser);
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak(Page.Text1, 10);
                    PageData.AppendStringWithBreak(Page.Text2);
                    PageData.AppendStringWithBreak(Page.TextDetails);

                    break;

                case "trophies":

                    PageData.AppendStringWithBreak("trophies");
                    PageData.AppendInt32(1);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendInt32(2);
                    PageData.AppendStringWithBreak(Page.Text1);
                    PageData.AppendStringWithBreak(Page.TextDetails);

                    break;

                case "pets":

                    PageData.AppendStringWithBreak("pets");
                    PageData.AppendInt32(2);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendStringWithBreak(Page.LayoutTeaser);
                    PageData.AppendInt32(4);
                    PageData.AppendStringWithBreak(Page.Text1);
                    PageData.AppendStringWithBreak("Give a name:");
                    PageData.AppendStringWithBreak("Pick a color:");
                    PageData.AppendStringWithBreak("Pick a race:");

                    break;

                default:

                    PageData.AppendStringWithBreak(Page.Layout);
                    PageData.AppendInt32(3);
                    PageData.AppendStringWithBreak(Page.LayoutHeadline);
                    PageData.AppendStringWithBreak(Page.LayoutTeaser);
                    PageData.AppendStringWithBreak(Page.LayoutSpecial);
                    PageData.AppendInt32(3);
                    PageData.AppendStringWithBreak(Page.Text1);
                    PageData.AppendStringWithBreak(Page.TextDetails);
                    PageData.AppendStringWithBreak(Page.TextTeaser);


                    // A`jeAdefault_3x3Kctlg_windows_headline1_enctlg_windows_teaser1_encatalog_special_txtbg2KLet some sunshine in! Our windows come in many styles to give an unique look to Your room. Who said Your room does not have a view?Click on an item for more information.Ooh, new view!SC[}HDEV window_tripleQAHIiZpOIMY@IDEV window_basicPAHIiZrOIMX~HDEV window_squareJHIi[pOIMZuIDEV window_skyscraperQAHIi[uOIM[uIdeal_skyscraper1PCHIi[uOQAMXvIdeal_skyscraper2PEHIi[uORBMX@IDEV window_chinese_narrowKHIiYrOIM[HDEV window_chinese_wideQAHIiXrOIMYHDEV window_goldenKHIi[qOIMXHDEV window_grungeQAHIiZqOIMXpIDEV window_holeQAHIiZtOIM[~HDEV window_romantic_narrowKHIiYqOIMZ~HDEV window_romantic_widePAHIiXqOIM[@IDEV window_70s_narrowKHIiXsOIMZ@IDEV window_70s_widePAHIi[rOIM

                    break;
            }

            PageData.AppendInt32(Page.Items.Count);

            foreach (CatalogItem Item in Page.Items)
            {
                Item.Serialize(PageData);
            }

            return PageData;
        }

        //internal ServerMessage SerializeTestIndex()
        //{
        //    ServerMessage Message = new ServerMessage(126);

        //    Message.AppendInt32(0);
        //    Message.AppendInt32(0);
        //    Message.AppendInt32(0);
        //    Message.AppendInt32(-1);
        //    Message.AppendStringWithBreak("");
        //    Message.AppendInt32(0);
        //    Message.AppendInt32(100);

        //    for (int i = 1; i <= 150; i++)
        //    {
        //        Message.AppendInt32(1);
        //        Message.AppendInt32(i);
        //        Message.AppendInt32(i);
        //        Message.AppendInt32(i);
        //        Message.AppendStringWithBreak("#" + i);
        //        Message.AppendInt32(0);
        //        Message.AppendInt32(0);
        //    }

        //    return Message;

        //    // A~HHHMHSBIRAI`ieAFront PageHHHJKaieACollectablesHHIKJMFurni ShopHPJIHSMcieASpacesHHIHSO`jeAWindowsHHIHSEajeAMood LightsHHIHRJbjeATwilightHHIHRNcjeATeleportsHHIHSI`keAModeHHIHSDakeACandyHHIHPLbkeAPuraHHIHRCckeAAreaHHIHQE`leACountryHHIHQIaleALodgeHHIHRKbleAPlasticHHIHPPcleAKitchenHHIHQD`meABathroomHHIHQKameAPlantsHHIHPMbmeARugsHHIHRJcmeARollersHHIHSK`neAPosters and FlagsHHIHPOaneATrophiesHHIHSBbneAExtrasHHHHHcneACameraHHHHRB`oeALimited EditionHHHHPHaoeAUrban Crime SceneHHHHSCboeAOrientalHHHHPNcoeASportHHHQBQL`peARelaxHHHHRLapeARomantique WeddingHHHSBQMbpeASCI FIHHHQBHcpeAHabbo County USHHHHRF`qeAWacky WeekendsHHHHSFaqeAExecutiveHHHHRDbqeABensalemHHHHQCcqeAArcticHHHHPC`reAAlhambraHHHHSNareATiki !HHHHPPbreAChristmasHHHHRMcreAShalimarHHHHRO`seAValentine's LoveHHHHRHaseAHabboweenHHHHQHbseAMoviesHHISAPBMPets ShopHRAHHH`teAPetsHHIHPEateACatsHHIHPFbteADogsHHIHREcteACrocodilesHHIHSJ`ueAPets' AccessoriesHHIHRJaueAPets InfoHHIQAQAMPixel ShopHRAIHPGbweAPixel CollectableHHIHSLcweARentalsHHIHQO`xeASpecial EffectsHHHHPKaxeAPixel DiscountsHHIHSHbxeAHello FurniHHIHPDcxeAAutomobileHHISBRA`yeAHabbo ExchangeHHISAQBayeAHabbo ClubHIIHQBcyeABuy Habbo ClubHHIKSAMEcotronHKIHSAazeAEcotronHHIHRFbzeARewardsHHIHRJczeAInstructionsHHISARB`{eASpecial OffersHHHHHa{eAmagic.creditsHHHHHb{eAmagic.pixelsHH
        //}

        //internal VoucherHandler GetVoucherHandler()
        //{
        //    return VoucherHandler;
        //}

        internal Marketplace GetMarketplace()
        {
            return Marketplace;
        }
    }
}
