﻿using System;
using System.Collections.Generic;
using Butterfly.Core;
using Butterfly.HabboHotel.Misc;
using Butterfly.HabboHotel.Support;
using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Users.Subscriptions;
using Butterfly.HabboHotel.Users.Authenticator;
using Butterfly.Messages;
using Butterfly.Net;
using Butterfly.Util;

namespace Butterfly.HabboHotel.GameClients
{
    class GameClient
    {
        private uint Id;

        private TcpConnection Connection;
        private GameClientMessageHandler MessageHandler;

        private Habbo Habbo;

        internal Boolean PongOK;

        //internal uint ClientId
        //{
        //    get
        //    {
        //        return Id;
        //    }
        //}

        //internal bool WaitingForDisconnect = false;

        //internal Boolean LoggedIn
        //{
        //    get
        //    {
        //        if (Habbo == null)
        //        {
        //            return false;
        //        }

        //        return true;
        //    }
        //}

        internal GameClient(uint ClientId, ref TcpConnection pConnection)
        {
            Id = ClientId;
            Connection = pConnection;

        }

        internal TcpConnection GetConnection()
        {
            return Connection;
        }

        internal GameClientMessageHandler GetMessageHandler()
        {
            return MessageHandler;
        }

        internal Habbo GetHabbo()
        {
            return Habbo;
        }

        internal void StartConnection()
        {
            if (Connection == null)
            {
                return;
            }

            PongOK = true;


            TcpConnection.RouteReceivedDataCallback DataRouter = new TcpConnection.RouteReceivedDataCallback(HandleConnectionData);
            Connection.Start(DataRouter);
        }

        internal void InitHandler()
        {
            MessageHandler = new GameClientMessageHandler(this);

            MessageHandler.RegisterGlobal();
            MessageHandler.RegisterHandshake();
            MessageHandler.RegisterHelp();
        }

        //internal ServerMessage GenerateUsersRoomNaviPacket()
        //{
        //    return UberEnvironment.GetGame().GetNavigator().GetDynamicNavigatorPacket(this, -3);
        //}

        internal void Login(string AuthTicket)
        {
            try
            {
                HabboData UserData = new HabboData(AuthTicket, GetConnection().ipAddress, true);

                if (!UserData.UserFound) //No user found for this SSO ticket
                {
                    Disconnect();
                    return;
                }

                Habbo NewHabbo = Authenticator.TryLoginHabbo(this, UserData, UserData);

                ButterflyEnvironment.GetGame().GetClientManager().LogClonesOut(NewHabbo.Id);

                this.Habbo = NewHabbo;
                this.Habbo.LoadData(UserData);

                UserData.Clear();
            }
            catch (Exception e)
            {
                SendNotif("Login error: " + e.Message);
                Logging.HandleException(e, "login");
                Disconnect();

                return;
            }

            try
            {
                ButterflyEnvironment.GetGame().GetBanManager().CheckForBanConflicts(this);
            }
            catch (ModerationBanException e)
            {
                SendBanMessage(e.Message);
                Disconnect();

                return;
            }

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{

            //}

            List<string> Rights = ButterflyEnvironment.GetGame().GetRoleManager().GetRightsForHabbo(GetHabbo());

            GetMessageHandler().GetResponse().Init(2);
            GetMessageHandler().GetResponse().Init(2);
            if (GetHabbo().GetSubscriptionManager().HasSubscription("habbo_vip"))
               GetMessageHandler().GetResponse().AppendInt32(2);
            else if (GetHabbo().GetSubscriptionManager().HasSubscription("habbo_club"))
                GetMessageHandler().GetResponse().AppendInt32(1);
            else
                GetMessageHandler().GetResponse().AppendInt32(0);

            GetMessageHandler().SendResponse();

            if (GetHabbo().HasFuse("fuse_mod"))
            {
                SendMessage(ButterflyEnvironment.GetGame().GetModerationTool().SerializeTool());
                ButterflyEnvironment.GetGame().GetModerationTool().SendOpenTickets(this);
            }

            SendMessage(GetHabbo().GetAvatarEffectsInventoryComponent().Serialize());

            MessageHandler.GetResponse().Init(290);
            MessageHandler.GetResponse().AppendBoolean(true);
            MessageHandler.GetResponse().AppendBoolean(false);
            MessageHandler.SendResponse();

            MessageHandler.GetResponse().Init(3);
            MessageHandler.SendResponse();

            MessageHandler.GetResponse().Init(517);
            MessageHandler.GetResponse().AppendBoolean(true);
            MessageHandler.SendResponse();

            if (PixelManager.NeedsUpdate(this))
                PixelManager.GivePixels(this);

            MessageHandler.GetResponse().Init(455);
            MessageHandler.GetResponse().AppendUInt(GetHabbo().HomeRoom);
            MessageHandler.SendResponse();

            MessageHandler.GetResponse().Init(458);
            MessageHandler.GetResponse().AppendInt32(30);
            MessageHandler.GetResponse().AppendInt32(GetHabbo().FavoriteRooms.Count);

            foreach (uint Id in GetHabbo().FavoriteRooms)
            {
                MessageHandler.GetResponse().AppendUInt(Id);
            }

            MessageHandler.SendResponse();

            if (GetHabbo().HasFuse("fuse_use_club_badge") && !GetHabbo().GetBadgeComponent().HasBadge("ACH_BasicClub1"))
            {
                GetHabbo().GetBadgeComponent().GiveBadge("ACH_BasicClub1", true);
            }
            else if (!GetHabbo().HasFuse("fuse_use_club_badge") && GetHabbo().GetBadgeComponent().HasBadge("ACH_BasicClub1"))
            {
                GetHabbo().GetBadgeComponent().RemoveBadge("ACH_BasicClub1");
            }

            ButterflyEnvironment.GetGame().GetAchievementManager().UnlockAchievement(this, 11, 1);

            if (!GetHabbo().GetBadgeComponent().HasBadge("Z63"))
                GetHabbo().GetBadgeComponent().GiveBadge("Z63", true);

                MessageHandler.RegisterUsers();
                MessageHandler.RegisterMessenger();
                MessageHandler.RegisterCatalog();
                MessageHandler.RegisterNavigator();
                MessageHandler.RegisterRooms();
        }

        internal void SendBanMessage(string Message)
        {
            ServerMessage BanMessage = new ServerMessage(35);
            BanMessage.AppendStringWithBreak("A moderator has kicked you from the hotel:", 13);
            BanMessage.AppendStringWithBreak(Message);
            GetConnection().SendMessage(BanMessage);
        }

        internal void SendNotif(string Message)
        {
            SendNotif(Message, false);
        }

        internal void SendNotif(string Message, Boolean FromHotelManager)
        {
            ServerMessage nMessage = new ServerMessage();

            if (FromHotelManager)
            {
                nMessage.Init(139);
            }
            else
            {
                nMessage.Init(161);
            }

            nMessage.AppendStringWithBreak(Message);
            GetConnection().SendMessage(nMessage);
        }

        //internal void SendNotif(string Message, string Url)
        //{
        //    ServerMessage nMessage = new ServerMessage(161);
        //    nMessage.AppendStringWithBreak(Message);
        //    nMessage.AppendStringWithBreak(Url);
        //    GetConnection().SendMessage(nMessage);
        //}

        internal void Stop()
        {
            if (GetMessageHandler() != null)
                MessageHandler.Destroy();

            if (Connection != null)
                Connection.Dispose();

            if (GetHabbo() != null)
                Habbo.OnDisconnect();
        }

        private bool Disconnected = false;

        internal void Disconnect()
        {
            if (!Disconnected)
            {
                ButterflyEnvironment.GetGame().GetClientManager().StopClient(Id);
                Disconnected = true;
            }
        }

        internal void HandleConnectionData(ref byte[] data)
        {
            if (data[0] == 64)
            {
                int pos = 0;

                while (pos < data.Length)
                {
                    try
                    {
                        int MessageLength = Base64Encoding.DecodeInt32(new byte[] { data[pos++], data[pos++], data[pos++] });
                        uint MessageId = Base64Encoding.DecodeUInt32(new byte[] { data[pos++], data[pos++] });

                        byte[] Content = new byte[MessageLength - 2];

                        for (int i = 0; i < Content.Length; i++)
                        {
                            Content[i] = data[pos++];
                        }

                        ClientMessage Message = new ClientMessage(MessageId, Content);

                        if (MessageHandler == null)
                        {
                            InitHandler(); //Never ever register the packets BEFORE you receive any data.
                        }

                        //DateTime PacketMsgStart = DateTime.Now;
                        try
                        {
                            MessageHandler.HandleRequest(Message);
                        }
                        catch (Exception e) { Logging.LogPacketException(Message.ToString(), e.ToString()); }
                        //TimeSpan TimeUsed = DateTime.Now - PacketMsgStart;

                        //if (TimeUsed.Milliseconds > 50 || TimeUsed.Seconds > 0 || TimeUsed.Minutes > 0)
                        //{
                        //    Logging.LogPacketException(Message.ToString(), "Too much time on packet: " + TimeUsed.Milliseconds + "ms, " + TimeUsed.Seconds + " sec, " + TimeUsed.Minutes + " min");
                        //}

                        //string DAta = null;
                        //if (DAta.Length > 0) { }
                    }
                    catch (Exception e)
                    {
                        //Logging.LogException("User DC: " + e.ToString());
                        Logging.HandleException(e, "packet handling");
                        Disconnect();
                    }
                }
            }
            else
            {
                Connection.SendData(CrossdomainPolicy.GetXmlPolicy());
                Connection.Dispose();
            }
        }

        internal void SendMessage(ServerMessage Message)
        {
            if (Message == null)
                return;
            if (GetConnection() == null)
                return;
            GetConnection().SendMessage(Message);
        }
    }
}
