﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Butterfly.Core;
using Butterfly.HabboHotel.Misc;
using Butterfly.Messages;
using Butterfly.Net;
using Butterfly.Storage;


namespace Butterfly.HabboHotel.GameClients
{
    class GameClientManager
    {
        private Task ConnectionChecker;
        private GameClient[] Clients;

        private Task mConnectionKiller;
        private List<TcpConnection> mToDisconnect;

        internal int ClientCount
        {
            get
            {
                if (Clients == null)
                    return 0;
                int Count = 0;
                for (int i = 0; i < Clients.Length; i++)
                {
                    if (Clients[i] != null)
                        if (Clients[i].GetHabbo() != null)
                            if (!string.IsNullOrEmpty(Clients[i].GetHabbo().Username))
                                Count++;
                }
                return Count;
            }
        }


        internal GameClientManager(int Count)
        {
            Clients = new GameClient[Count];
            mToDisconnect = new List<TcpConnection>();
            mConnectionKiller = new Task(DisconnectorLoop);
            mConnectionKiller.Start();
        }

        private void DisconnectorLoop()
        {
            while (true)
            {
                try
                {
                    List<TcpConnection> List = mToDisconnect;
                    mToDisconnect = new List<TcpConnection>();

                    if (List != null)
                        foreach (TcpConnection Client in List)
                        {
                            if (Client != null)
                                Client.ConnectionDead();
                            //Console.WriteLine("Client disconnected in disconnector loop");
                        }
                }
                catch (Exception e) { Logging.LogThreadException(e.ToString(), "Disconnector task"); }

                Thread.Sleep(500);
            }
        }

        internal void AddGameClient(TcpConnection Client)
        {
            if (!mToDisconnect.Contains(Client))
                mToDisconnect.Add(Client);
        }

        //internal void Clear()
        //{
        //    //Array.Clear(Clients, 0, Clients.Length);
        //}

        internal GameClient GetClient(uint ClientId)
        {
            try
            {
                return Clients[ClientId];
            }
            catch
            {
                return null;
            }
        }

        //internal bool RemoveClient(uint ClientId)
        //{
        //    Clients[ClientId] = null;
        //    //try
        //    //{
        //    //    Clients.Remove(ClientId);
        //    //    return true;
        //    //}
        //    //catch
        //    //{
        //    //    return false;
        //    //}
        //}

        internal void CreateAndStartClient(uint ClientId, ref TcpConnection Connection)
        {
            Clients[ClientId] = new GameClient(ClientId, ref Connection);
            Clients[ClientId].StartConnection();
        }

        internal void StopClient(uint ClientId)
        {
            GameClient Client = GetClient(ClientId);

            if (Client == null)
                return;

            ButterflyEnvironment.GetConnectionManager().DropConnection(ClientId);

            Client.Stop();
            Clients[ClientId] = null;
        }

        internal void StartConnectionChecker()
        {
            if (ConnectionChecker != null)
            {
                return;
            }

            ConnectionChecker = new Task(TestClientConnections);
            //ConnectionChecker.Name = "Connection Checker";
            //ConnectionChecker.Priority = ThreadPriority.Lowest;
            ConnectionChecker.Start();
        }

        internal void StopConnectionChecker()
        {
            if (ConnectionChecker == null)
            {
                return;
            }

            //ConnectionChecker = null;
        }

        //internal static DateTime LastExecuted;

        private void TestClientConnections()
        {
            int interval = int.Parse(ButterflyEnvironment.GetConfig().data["client.ping.interval"]);

            if (interval <= 100)
                throw new ArgumentException("Invalid configuration value for ping interval! Must be above 100 miliseconds.");

            while (true)
            {
                try
                {
                    ServerMessage PingMessage = new ServerMessage(50);
                
                    List<GameClient> TimedOutClients = new List<GameClient>();
                    List<GameClient> ToPing = new List<GameClient>();

                    for (int i = 0; i < Clients.Length; i++)
                    {
                        GameClient Client = Clients[i];

                        if (Client == null)
                            continue;
                        if (Client.PongOK)
                        {
                            Client.PongOK = false;
                            ToPing.Add(Client);
                        }
                        else
                            TimedOutClients.Add(Client);
                    }

                    //using (ClonedTable Table = Clients.GetThreadSafeTable)
                    //{
                    //    foreach (GameClient Client in Clients.Values)
                    //    {
                    //        if (Client.PongOK)
                    //        {
                    //            Client.PongOK = false;
                    //            ToPing.Add(Client);
                    //        }
                    //        else
                    //        {
                    //            TimedOutClients.Add(Client.ClientId);
                    //        }
                    //    }
                    //}

                    foreach (GameClient Client in TimedOutClients)
                    {
                        try
                        {
                            Client.Disconnect();
                        }
                        catch { }
                    }

                    byte[] PingMessageBytes = PingMessage.GetBytes();
                    foreach (GameClient Client in ToPing)
                    {
                        try
                        {
                            Client.GetConnection().SendData(PingMessageBytes);
                        }
                       catch
                        {
                        }
                    }

                }
                catch (Exception e) { Logging.LogThreadException(e.ToString(), "Connection checker task"); }

                Thread.Sleep(interval);
            }
        }

        internal GameClient GetClientByHabbo(uint HabboId)
        {
            if (Clients == null)
                return null;
            for (int i = 0; i < Clients.Length; i++)
            {
                try
                {
                    GameClient Client = Clients[i];
                    if (Client != null && Client.GetHabbo() != null)
                    {
                        if (Client.GetHabbo().Id == HabboId)
                            return Client;
                    }
                }
                catch { }
            }
            return null;
        }

        internal GameClient GetClientByHabbo(string Name)
        {
            try
            {
                for (int i = 0; i < Clients.Length; i++)
                {
                    GameClient Client = Clients[i];
                    if (Client == null)
                        continue;
                    if (Client.GetHabbo() == null)
                        continue;

                    if (Client.GetHabbo().Username.ToLower() == Name.ToLower())
                        return Client;
                }
            }
            catch { }
            return null;
        }

        internal void BroadcastMessage(ServerMessage Message)
        {
            byte[] PacketData = Message.GetBytes();
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client == null)
                    continue;
                try
                {
                    Client.GetConnection().SendData(PacketData);
                }
                catch
                {
                }
            }
        }

        internal void GiveAllOnlineCredits(int Amount)
        {
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client == null)
                    continue;
                if (Client.GetHabbo() == null)
                    continue;
                try
                {
                    Client.GetHabbo().Credits += Amount;
                    Client.GetHabbo().UpdateCreditsBalance();
                    Client.SendNotif("You just received " + Amount + " from staff!");
                }
                catch { }
            }
        }

        internal void GiveAllOnlineBadge(string pBadge)
        {
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client == null)
                    continue;
                if (Client.GetHabbo() == null)
                    continue;
                try
                {
                    Client.GetHabbo().GetBadgeComponent().GiveBadge(pBadge, true);
                    Client.SendNotif("You just received a badge from staff!");
                }
                catch { }
            }
        }

        internal void BroadcastMessage(ServerMessage Message, String FuseRequirement)
        {
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client == null)
                    continue;
                try
                {
                    if (FuseRequirement.Length > 0)
                        if (Client.GetHabbo() == null || !Client.GetHabbo().HasFuse(FuseRequirement))
                            continue;

                    Client.SendMessage(Message);
                }
                catch { }
            }
        }

        internal void CheckEffects()
        {
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client == null)
                    continue;
                if (Client.GetHabbo() == null || Client.GetHabbo().GetAvatarEffectsInventoryComponent() == null)
                    continue;

                Client.GetHabbo().GetAvatarEffectsInventoryComponent().CheckExpired();
            }
        }

        internal void CloseAll()
        {
            StringBuilder QueryBuilder = new StringBuilder();
            bool RunUpdate = false;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                for (int i = 0; i < Clients.Length; i++)
                {
                    GameClient Client = Clients[i];
                    if (Client != null && Client.GetHabbo() != null)
                    {
                        try
                        {
                            Client.GetHabbo().GetInventoryComponent().RunDBUpdate();
                            QueryBuilder.Append(Client.GetHabbo().GetQueryString);
                            RunUpdate = true;
                        }
                        catch { }
                    }
                }
                if (RunUpdate)
                {
                    try
                    {
                        dbClient.ExecuteQuery(QueryBuilder.ToString()); //DONE!
                    }
                    catch (Exception e)
                    {
                        //Logging.LogShutdownError(e.ToString());
                        Logging.HandleException(e, "GameClientManager.CloseAll()");
                    }
                }
            }

            Console.WriteLine("Done saving users inventory!");
            Console.WriteLine("Closing server connections...");

            try
            {
                for (int i = 0; i < Clients.Length; i++)
                {
                    GameClient Client = Clients[i];
                    if (Client != null && Client.GetConnection() != null)
                    {
                        try
                        {
                            Client.GetConnection().Dispose();
                        }
                        catch { }
                    }
                }
            }
            catch (Exception e)
            {
                Logging.LogShutdownError(e.ToString());
            }
            Array.Clear(Clients, 0, Clients.Length);
            Console.WriteLine("Connections closed!");
        }

        internal void LogClonesOut(uint UserID)
        {
            for (int i = 0; i < Clients.Length; i++)
            {
                GameClient Client = Clients[i];
                if (Client != null && Client.GetHabbo() != null && Client.GetHabbo().Id == UserID)
                    Client.Disconnect();
            }
        }

        internal string GetNameById(uint Id)
        {
            GameClient Cl = GetClientByHabbo(Id);

            if (Cl != null)
            {
                return Cl.GetHabbo().Username;
            }

            DataRow Row;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                Row = dbClient.ReadDataRow("SELECT username FROM users WHERE id = '" + Id + "' LIMIT 1");
            }

            if (Row == null)
            {
                return "Unknown User";
            }

            return (string)Row[0];
        }

        //internal void DeployHotelCreditsUpdate()
        //{
        //    for (int i = 0; i < Clients.Length; i++)
        //    {
        //        GameClient Client = Clients[i];
        //        if (Client.GetHabbo() == null)
        //        {
        //            continue;
        //        }

        //        int newCredits = 0;

        //        using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //        {
        //            newCredits = (int)dbClient.ReadDataRow("SELECT credits FROM users WHERE id = '" + Client.GetHabbo().Id + "' LIMIT 1")[0];
        //        }

        //        int oldBalance = Client.GetHabbo().Credits;

        //        Client.GetHabbo().Credits = newCredits;

        //        if (oldBalance < 3000)
        //        {
        //            Client.GetHabbo().UpdateCreditsBalance(false);
        //            //Client.SendNotif("Uber Credits Update" + Convert.ToChar(13) + "-----------------------------------" + Convert.ToChar(13) + "We have refilled your credits up to 3000 - enjoy! The next credits update will occur in 3 hours.", "http://uber.meth0d.org/credits");
        //        }
        //        else if (oldBalance >= 3000)
        //        {
        //            //Client.SendNotif("Uber Credits Update" + Convert.ToChar(13) + "-----------------------------------" + Convert.ToChar(13) + "Sorry! Because your credit balance is 3000 or higher, we have not refilled your credits. The next credits update will occur in 3 hours.", "http://uber.meth0d.org/credits");
        //        }
        //    }
        //}

        //internal void CheckForAllBanConflicts()
        //{
        //    Dictionary<GameClient, ModerationBanException> ConflictsFound = new Dictionary<GameClient, ModerationBanException>();
        //    for (int i = 0; i < this.Clients.Length; i++)
        //    {
        //        GameClient Client = this.Clients[i];
        //        if (Client == null)
        //            continue;
        //        try
        //        {
        //            UberEnvironment.GetGame().GetBanManager().CheckForBanConflicts(Client);
        //        }

        //        catch (ModerationBanException e)
        //        {
        //            ConflictsFound.Add(Client, e);
        //        }

        //    }


        //    foreach (KeyValuePair<GameClient, ModerationBanException> Data in ConflictsFound)
        //    {
        //        Data.Key.SendBanMessage(Data.Value.Message);
        //        Data.Key.Disconnect();
        //    }

        //}

        internal void CheckPixelUpdates()
        {
            try
            {
                if (Clients == null)
                    return;

                for (int i = 0; i < Clients.Length; i++)
                {
                    GameClient Client = Clients[i];
                    if (Client == null)
                        continue;
                    if (Client.GetHabbo() == null || !PixelManager.NeedsUpdate(Client))
                        continue;

                    PixelManager.GivePixels(Client);
                }
            }

            catch (Exception e)
            {
                //Logging.WriteLine("[GCMExt.CheckPixelUpdates]: " + e.Message);
                Logging.LogThreadException(e.ToString(), "GCMExt.CheckPixelUpdates task");
            }
        }

        internal List<ServerMessage> GenerateUsersOnlineList()
        {
            List<ServerMessage> Messages = new List<ServerMessage>();
            int CurrentMsgCount = 0;
            ServerMessage CurrentWorking = new ServerMessage();
            CurrentWorking.Init(161);
            CurrentWorking.AppendStringWithBreak("Users online:\r");

            for (int i = 0; i < this.Clients.Length; i++)
            {
                GameClient Client = this.Clients[i];
                if (Client == null)
                    continue;
                if (Client.GetHabbo() == null)
                    continue;

                if (CurrentMsgCount > 20)
                {
                    Messages.Add(CurrentWorking);
                    CurrentMsgCount = 0;
                    CurrentWorking = new ServerMessage();
                    CurrentWorking.Init(161);
                }

                CurrentMsgCount++;
                CurrentWorking.AppendStringWithBreak(Client.GetHabbo().Username + " {" + Client.GetHabbo().Rank + "}\r");
            }
            Messages.Add(CurrentWorking);

            return Messages;
        }
    }
}
