﻿using System;
using System.Collections.Generic;
using Butterfly.Core;
using Butterfly.HabboHotel.Items.Interactors;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;
using Butterfly_Emulator.HabboHotel.Items;

namespace Butterfly.HabboHotel.Items
{
    class RoomItem
    {
        internal UInt32 Id;
        internal UInt32 RoomId;
        internal UInt32 BaseItem;
        internal string ExtraData;
        internal string Figure;
        internal string Gender;

        private Dictionary<int, AffectedTile> mAffectedPoints;

        internal Dictionary<int, AffectedTile> GetAffectedTiles
        {
            get
            {
                return mAffectedPoints;
            }
        }

        private int mX;
        internal int GetX
        {
            get
            {
                return mX;
            }
        }

        private int mY;

        internal int GetY
        {
            get
            {
                return mY;
            }
        }

        private Double mZ;

        internal Double GetZ
        {
            get
            {
                return mZ;
            }
        }

        internal void SetState(int pX, int pY, Double pZ)
        {
            this.mX = pX;
            this.mY = pY;
            this.mZ = pZ;
            mAffectedPoints = Room.GetAffectedTiles(GetBaseItem().Length, GetBaseItem().Width, mX, mY, Rot);
        }

        internal int Rot;

        internal string WallPos;

        internal bool UpdateNeeded;
        internal int UpdateCounter;

        internal UInt32 InteractingUser;
        internal UInt32 InteractingUser2;

        private Item mBaseItem;
        private Room mRoom;
        private bool mIsWallItem;
        private bool mIsFloorItem;

        private bool mIsRoller;
        internal bool IsTrans;
        
        internal bool IsRoller
        {
            get
            {
                return mIsRoller;
            }
        }

        internal Coord Coordinate
        {
            get
            {
                return new Coord(mX, mY);
            }
        }

        internal double TotalHeight
        {
            get
            {
                return mZ + GetBaseItem().Height;
            }
        }

        internal bool IsWallItem
        {
            get
            {
                return mIsWallItem;
            }
        }

        internal bool IsFloorItem
        {
            get
            {
                return mIsFloorItem;
            }
        }

        internal Coord SquareInFront
        {
            get
            {
                Coord Sq = new Coord(mX, mY);

                if (Rot == 0)
                {
                    Sq.Y--;
                }
                else if (Rot == 2)
                {
                    Sq.X++;
                }
                else if (Rot == 4)
                {
                    Sq.Y++;
                }
                else if (Rot == 6)
                {
                    Sq.X--;
                }

                return Sq;
            }
        }

        internal Coord SquareBehind
        {
            get
            {
                Coord Sq = new Coord(mX, mY);

                if (Rot == 0)
                {
                    Sq.Y++;
                }
                else if (Rot == 2)
                {
                    Sq.X--;
                }
                else if (Rot == 4)
                {
                    Sq.Y--;
                }
                else if (Rot == 6)
                {
                    Sq.X++;
                }

                return Sq;
            }
        }

        internal FurniInteractor Interactor
        {
            get
            {
                switch (GetBaseItem().InteractionType)
                {
                    case InteractionType.none:
                    default:
                        return new InteractorGenericSwitch(GetBaseItem().Modes);
                    case InteractionType.teleport:
                        return new InteractorTeleport();
                    case InteractionType.bottle:
                        return new InteractorSpinningBottle();
                    case InteractionType.dice:
                        return new InteractorDice();
                    case InteractionType.habbowheel:
                        return new InteractorHabboWheel();
                    case InteractionType.loveshuffler:
                        return new InteractorLoveShuffler();
                    case InteractionType.onewaygate:
                        return new InteractorOneWayGate();
                    case InteractionType.alert:
                        return new InteractorAlert();
                    case InteractionType.vendingmachine:
                        return new InteractorVendor();
                    case InteractionType.gate:
                        return new InteractorGate(GetBaseItem().Modes);
                    case InteractionType.scoreboard:
                        return new InteractorScoreboard();
                }
            }
        }

        internal RoomItem(UInt32 Id, UInt32 RoomId, UInt32 BaseItem, string ExtraData, int X, int Y, Double Z, int Rot, string WallPos, Room pRoom)
        {
            this.Id = Id;
            this.RoomId = RoomId;
            this.BaseItem = BaseItem;
            this.ExtraData = ExtraData;
            this.mX = X;
            this.mY = Y;
            this.mZ = Z;
            this.Rot = Rot;
            this.WallPos = WallPos;
            this.UpdateNeeded = false;
            this.UpdateCounter = 0;
            this.InteractingUser = 0;
            this.InteractingUser2 = 0;
            this.IsTrans = false;

            mBaseItem = ButterflyEnvironment.GetGame().GetItemManager().GetItem(BaseItem);
            mRoom = pRoom; //Todo: rub my penis

            if (GetBaseItem() == null)
                Logging.LogException("Unknown baseID: " + BaseItem);

            switch (GetBaseItem().InteractionType)
            {
                case InteractionType.teleport:

                    ReqUpdate(0);
                    IsTrans = true;
                    break;
                case InteractionType.roller:
                    mIsRoller = true;
                    pRoom.GotRollers = true;
                    break;

                case InteractionType.fbgate:
                    IsTrans = true;
                    break;
            }

            //THIS is called cache, BITCH!
            //mCoord = new Coord(X, Y);
            //mTotalheight = Z + GetBaseItem().Height;
            mIsWallItem = (GetBaseItem().Type == 'i');
            mIsFloorItem = (GetBaseItem().Type == 's');
            mAffectedPoints = Room.GetAffectedTiles(GetBaseItem().Length, GetBaseItem().Width, mX, mY, Rot);
        }

        //internal RoomItem(int Id, uint RoomId, int BaseItem, string ExtraData, Int16 X, Int16 Y, Double Z, Int16 Rot, string WallPos, Room pRoom, bool dbexist)
        //{
        //    this.Id = (uint)Id;
        //    this.RoomId = RoomId;
        //    this.BaseItem = (uint)BaseItem;
        //    this.ExtraData = ExtraData;
        //    this.mX = X;
        //    this.mY = Y;
        //    this.mZ = Z;
        //    this.Rot = Rot;
        //    this.WallPos = WallPos;
        //    this.UpdateNeeded = false;
        //    this.UpdateCounter = 0;
        //    this.InteractingUser = 0;
        //    this.InteractingUser2 = 0;
        //    this.IsTrans = false;

        //    mBaseItem = UberEnvironment.GetGame().GetItemManager().GetItem(this.BaseItem);
        //    mRoom = pRoom; //Todo: rub my penis

        //    if (GetBaseItem() == null)
        //        Logging.LogException("Unknown baseID: " + BaseItem);

        //    switch (GetBaseItem().InteractionType.ToLower())
        //    {
        //        case "teleport":

        //            ReqUpdate(0);

        //            break;
        //        case "roller":
        //            mIsRoller = true;
        //            pRoom.GotRollers = true;
        //            break;

        //        case "fbgate":
        //            IsTrans = true;
        //            break;
        //    }

        //    //THIS is called cache, BITCH!
        //    //mCoord = new Coord(X, Y);
        //    //mTotalheight = Z + GetBaseItem().Height;
        //    mIsWallItem = (GetBaseItem().Type == 'i');
        //    mIsFloorItem = (GetBaseItem().Type == 's');
        //    mAffectedPoints = GetRoom().GetAffectedTiles(GetBaseItem().Length, GetBaseItem().Width, mX, mY, Rot);
        //}

        internal void ProcessUpdates()
        {
            this.UpdateCounter--;

            if (this.UpdateCounter <= 0 || IsTrans)
            {
                this.UpdateNeeded = false;
                this.UpdateCounter = 0;

                RoomUser User = null;
                RoomUser User2 = null;

                switch (GetBaseItem().InteractionType)
                {
                    case InteractionType.onewaygate:

                        User = null;

                        if (InteractingUser > 0)
                        {
                            User = GetRoom().GetRoomUserByHabbo(InteractingUser);
                            //GetRoom().FreeSqareForUsers(mX, mY);
                        }

                        if (User != null && User.X == mX && User.Y == mY)
                        {
                            ExtraData = "1";

                            User.MoveTo(SquareBehind);

                            ReqUpdate(0);
                            UpdateState(false, true);
                        }
                        else if (User != null && User.Coordinate == SquareBehind)
                        {
                            User.UnlockWalking();

                            ExtraData = "0";
                            InteractingUser = 0;

                            UpdateState(false, true);
                        }
                        else if (ExtraData == "1")
                        {
                            ExtraData = "0";
                            UpdateState(false, true);
                        }

                        if (User == null)
                        {
                            InteractingUser = 0;
                        }

                        break;

                    case InteractionType.teleport:

                        User = null;
                        User2 = null;

                        bool keepDoorOpen = false;
                        bool showTeleEffect = false;

                        // Do we have a primary user that wants to go somewhere?
                        if (InteractingUser > 0)
                        {
                            User = GetRoom().GetRoomUserByHabbo(InteractingUser);

                            // Is this user okay?
                            if (User != null)
                            {
                                // Is he in the tele?
                                if (User.Coordinate == Coordinate)
                                {
                                    //GetRoom().CloseSqare(X, Y);
                                    //Remove the user from the square
                                    User.AllowOverride = false;
                                    //GetRoom().FreeSqareForUsers(mX, mY);

                                    if (User.TeleDelay == -1)
                                    {
                                        User.TeleDelay = 1;
                                    }

                                    if (TeleHandler.IsTeleLinked(Id, mRoom))
                                    {
                                        showTeleEffect = true;

                                        if (User.TeleDelay == 0)
                                        {
                                            // Woop! No more delay.
                                            uint TeleId = TeleHandler.GetLinkedTele(Id, mRoom);
                                            uint RoomId = TeleHandler.GetTeleRoomId(TeleId, mRoom);

                                            // Do we need to tele to the same room or gtf to another?
                                            if (RoomId == this.RoomId)
                                            {
                                                RoomItem Item = GetRoom().GetItem(TeleId);

                                                if (Item == null)
                                                {
                                                    User.UnlockWalking();
                                                }
                                                else
                                                {


                                                    // Set pos
                                                    User.SetPos(Item.GetX, Item.GetY, Item.GetZ);
                                                    User.SetRot(Item.Rot);

                                                    // Force tele effect update (dirty)
                                                    Item.ExtraData = "2";
                                                    Item.UpdateState(false, true);

                                                    // Set secondary interacting user
                                                    Item.InteractingUser2 = InteractingUser;
                                                }
                                            }
                                            else
                                            {
                                                // Let's run the teleport delegate to take futher care of this..
                                                if (!User.IsBot)
                                                    ButterflyEnvironment.GetGame().GetRoomManager().AddTeleAction(new TeleUserData(User.GetClient().GetMessageHandler(), User.GetClient().GetHabbo(), RoomId, TeleId));
                                            }

                                            // We're done with this tele. We have another one to bother.
                                            InteractingUser = 0;
                                        }
                                        else
                                        {
                                            // We're linked, but there's a delay, so decrease the delay and wait it out.
                                            User.TeleDelay--;
                                        }
                                    }
                                    else
                                    {
                                        // This tele is not linked, so let's gtfo.
                                        User.UnlockWalking();
                                        InteractingUser = 0;
                                    }
                                }
                                // Is he in front of the tele?
                                else if (User.Coordinate == SquareInFront)
                                {
                                    User.AllowOverride = true;
                                    // Open the door
                                    keepDoorOpen = true;

                                    // Lock his walking. We're taking control over him. Allow overriding so he can get in the tele.
                                    if (User.IsWalking && (User.GoalX != mX || User.GoalY != mY))
                                    {
                                        User.ClearMovement(true);
                                    }

                                    User.CanWalk = false;
                                    User.AllowOverride = true;

                                    // Move into the tele
                                    User.MoveTo(Coordinate);
                                }
                                // Not even near, do nothing and move on for the next user.
                                else
                                {
                                    InteractingUser = 0;
                                }
                            }
                            else
                            {
                                // Invalid user, do nothing and move on for the next user. 
                                InteractingUser = 0;
                            }
                        }

                        // Do we have a secondary user that wants to get out of the tele?
                        if (InteractingUser2 > 0)
                        {
                            User2 = GetRoom().GetRoomUserByHabbo(InteractingUser2);

                            // Is this user okay?
                            if (User2 != null)
                            {
                                // If so, open the door, unlock the user's walking, and try to push him out in the right direction. We're done with him!
                                keepDoorOpen = true;
                                User2.UnlockWalking();
                                User2.MoveTo(SquareInFront);
                            }

                            // This is a one time thing, whether the user's valid or not.
                            InteractingUser2 = 0;
                        }

                        // Set the new item state, by priority
                        if (keepDoorOpen)
                        {
                            if (ExtraData != "1")
                            {
                                ExtraData = "1";
                                UpdateState(false, true);
                            }
                        }
                        else if (showTeleEffect)
                        {
                            if (ExtraData != "2")
                            {
                                ExtraData = "2";
                                UpdateState(false, true);
                            }
                        }
                        else
                        {
                            if (ExtraData != "0")
                            {
                                ExtraData = "0";
                                UpdateState(false, true);
                            }
                        }

                        // We're constantly going!
                        ReqUpdate(1);

                        break;

                    case InteractionType.bottle:

                        ExtraData = ButterflyEnvironment.GetRandomNumber(0, 7).ToString();
                        UpdateState();
                        break;

                    case InteractionType.dice:

                        ExtraData = ButterflyEnvironment.GetRandomNumber(1, 6).ToString();
                        UpdateState();
                        break;

                    case InteractionType.habbowheel:

                        ExtraData = ButterflyEnvironment.GetRandomNumber(1, 10).ToString();
                        UpdateState();
                        break;

                    case InteractionType.loveshuffler:

                        if (ExtraData == "0")
                        {
                            ExtraData = ButterflyEnvironment.GetRandomNumber(1, 4).ToString();
                            ReqUpdate(20);
                        }
                        else if (ExtraData != "-1")
                        {
                            ExtraData = "-1";
                        }

                        UpdateState(false, true);
                        break;

                    case InteractionType.alert:

                        if (this.ExtraData == "1")
                        {
                            this.ExtraData = "0";
                            this.UpdateState(false, true);
                        }

                        break;

                    case InteractionType.vendingmachine:

                        if (this.ExtraData == "1")
                        {
                            User = GetRoom().GetRoomUserByHabbo(InteractingUser);

                            if (User != null)
                            {
                                User.UnlockWalking();

                                int randomDrink = GetBaseItem().VendingIds[ButterflyEnvironment.GetRandomNumber(0, (GetBaseItem().VendingIds.Count - 1))];
                                User.CarryItem(randomDrink);
                            }

                            this.InteractingUser = 0;
                            this.ExtraData = "0";

                            UpdateState(false, true);
                        }

                        break;

                    case InteractionType.fbgate:
                        {
                                if (!string.IsNullOrEmpty(Figure))
                                {
                                    User = GetRoom().GetUserForSquare(Coordinate.X, Coordinate.Y);
                                    if (User != null)
                                    {
                                        if (User.Coordinate == this.Coordinate && (User.GetClient().GetHabbo().Gender != Gender || User.GetClient().GetHabbo().Look != Figure))
                                        {
                                            User.GetClient().GetHabbo().Gender = Gender;
                                            User.GetClient().GetHabbo().Look = Figure;

                                            ServerMessage RoomUpdate = new ServerMessage(266);
                                            RoomUpdate.AppendInt32(User.VirtualId);
                                            RoomUpdate.AppendStringWithBreak(User.GetClient().GetHabbo().Look);
                                            RoomUpdate.AppendStringWithBreak(User.GetClient().GetHabbo().Gender.ToLower());
                                            RoomUpdate.AppendStringWithBreak(User.GetClient().GetHabbo().Motto);
                                            this.mRoom.SendMessage(RoomUpdate);
                                        }
                                    }
                                }

                            break;
                        }
                }
            }
        }

        internal void ReqUpdate(int Cycles)
        {
            this.UpdateCounter = Cycles;
            this.UpdateNeeded = true;
        }

        internal void UpdateState()
        {
            UpdateState(true, true);
        }

        internal void UpdateState(bool inDb, bool inRoom)
        {
            if (inDb)
            {
                GetRoom().AddChangedItem(this);
                //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                //{
                //    dbClient.AddParamWithValue("extra_data", this.ExtraData);
                //    dbClient.ExecuteQuery("UPDATE room_items SET extra_data = @extra_data WHERE id = '" + Id + "' LIMIT 1");
                //}
            }

            if (inRoom)
            {
                ServerMessage Message = new ServerMessage();

                if (IsFloorItem)
                {
                    Message.Init(88);
                    Message.AppendStringWithBreak(Id.ToString());
                    Message.AppendStringWithBreak(ExtraData);
                }
                else
                {
                    Message.Init(85);
                    Serialize(Message);
                }

                GetRoom().SendMessage(Message);
            }
        }

        internal void Serialize(ServerMessage Message)
        {
            // AU33614959XEP:w=3,11 l=31,83 l0
            // A]r|ebuAQsPAQAJ0.0q|ebuA0MC[!`A

            if (IsFloorItem)
            {
                Message.AppendUInt(Id);
                Message.AppendInt32(GetBaseItem().SpriteId);
                Message.AppendInt32(mX);
                Message.AppendInt32(mY);
                Message.AppendInt32(Rot);
                Message.AppendStringWithBreak(mZ.ToString().Replace(',', '.'));
                Message.AppendInt32(0);
                Message.AppendStringWithBreak(ExtraData);
                Message.AppendInt32(-1);
                if (GetBaseItem().InteractionType.ToString().ToLower() == "default")
                    Message.AppendBoolean(false);
                else
                    Message.AppendBoolean(true);
            }
            else if (IsWallItem)
            {
                Message.AppendStringWithBreak(Id + "");
                Message.AppendInt32(GetBaseItem().SpriteId);
                Message.AppendStringWithBreak(WallPos);

                switch (GetBaseItem().InteractionType)
                {
                    case InteractionType.postit:
                        Message.AppendStringWithBreak(ExtraData.Split(' ')[0]);
                        break;

                    default:

                        Message.AppendStringWithBreak(ExtraData);
                        break;
                }
                if (GetBaseItem().InteractionType.ToString().ToLower() == "default")
                    Message.AppendBoolean(false);
                else
                    Message.AppendBoolean(true);
            }
        }

        internal Item GetBaseItem()
        {
            if (mBaseItem == null)
                mBaseItem = ButterflyEnvironment.GetGame().GetItemManager().GetItem(BaseItem);

            return mBaseItem;
        }

        internal Room GetRoom()
        {
            if (mRoom == null)
                mRoom = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(RoomId); //Todo: rub my penis

            return mRoom;
        }
    }
}
