﻿using System;
using System.Threading;
using Butterfly.Core;
using Butterfly.Storage;
using Butterfly.Net;
using System.Text;

namespace Butterfly.HabboHotel.Misc
{
    internal class LowPriorityWorker
    {
        private static int UserPeak;

        private static string FormatToBytes(int Num)
        {
            if (Num < 1024)
                return Num + "B/s";
            else if (Num < 1048576) //KB/s
            {
                double Traffic = Num / 1024;
                return String.Format("{0:0.##}", Traffic) + "KB/s";
            }
            else //MB/s
            {
                double Traffic = Num / 1048576;
                return String.Format("{0:0.##}", Traffic) + "MB/s";
            }
        }

        private static string mColdTitle;
        internal static void ConsoleTitleWorker()
        {
            while (true)
            {
                try
                {
                    int TotalBytesSent = TcpConnection.GetNumberOfSentBytes();
                    int TotalBytesReceived = TcpConnection.GetNumberOfReceivedBytes();

                    Console.Title = mColdTitle + "In: " + FormatToBytes(TotalBytesReceived) + " Out: " + FormatToBytes(TotalBytesSent);
                }
                catch (Exception Exception)
                {
                    Logging.LogThreadException(Exception.ToString(), "ConsoleTitleWorker");
                }

                Thread.Sleep(1000);
            }
        }

        internal static void Process()
        {
            mColdTitle = string.Empty;
            // allow 10 seconds for Uber to finish anything it may be doing, then enter regular loop
            //Thread.Sleep(10000); MY SERVER LOADS FASTER THAN 1 SECOND WHY THE FUCK SHOULD I LET IT SLEEP FOR OVER 10X THE LOADING TIME?!

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                UserPeak = dbClient.ReadInt32("SELECT userpeak FROM server_status");
            }

            while (true)
            {
                try
                {
                    TimeSpan Uptime = DateTime.Now - ButterflyEnvironment.ServerStarted;
                    mColdTitle = "Butterfly | Uptime: " + Uptime.Minutes + " minutes, " + Uptime.Hours + " hours and " + Uptime.Days + " day | " +
                        "Online users: " + ButterflyEnvironment.GetGame().GetClientManager().ClientCount + " | ";

                    #region Garbage Collection
                    GC.Collect();
                    GC.WaitForPendingFinalizers();
                    #endregion

                    #region Statistics
                    int Status = 1;
                    int UsersOnline = ButterflyEnvironment.GetGame().GetClientManager().ClientCount;

                    if (UsersOnline > UserPeak)
                        UserPeak = UsersOnline;

                    int RoomsLoaded = ButterflyEnvironment.GetGame().GetRoomManager().LoadedRoomsCount;

                    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                    {
                        dbClient.ExecuteQuery("UPDATE server_status SET stamp = '" + ButterflyEnvironment.GetUnixTimestamp() + "', status = '" + Status + "', users_online = '" + UsersOnline + "', rooms_loaded = '" + RoomsLoaded + "', server_ver = '" + ButterflyEnvironment.PrettyVersion + "', userpeak = '" + UserPeak + "' LIMIT 1;");
                    }
                    #endregion

                    #region Effects
                    ButterflyEnvironment.GetGame().GetClientManager().CheckEffects();
                    #endregion
                }
                catch (Exception e) { Logging.LogThreadException(e.ToString(), "Server status update task"); }
                Thread.Sleep(30000);
            }
        }

        //private static void UpdateMaxUserCount(int UserCount)
        //{


        //}
    }
}