﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Users.Messenger;
using Butterfly.Messages;
using Butterfly.Storage;

namespace Butterfly.HabboHotel.Navigators
{
    class Navigator
    {
        private Dictionary<int, string> PublicCategories;
        private List<FlatCat> PrivateCategories;

        private Dictionary<Int32, PublicItem> PublicItems;

        internal Navigator()
        {
            PublicCategories = new Dictionary<int, string>();
            PrivateCategories = new List<FlatCat>();
            PublicItems = new Dictionary<int, PublicItem>();
        }

        internal void Initialize(DatabaseClient dbClient)
        {
            DataTable dPubCats = dbClient.ReadDataTable("SELECT id,caption FROM navigator_pubcats WHERE enabled = '1'");
            DataTable dPrivCats = dbClient.ReadDataTable("SELECT id,caption,min_rank FROM navigator_flatcats WHERE enabled = '1'");
            DataTable dPubItems = dbClient.ReadDataTable("SELECT id,bannertype,caption,image,image_type,room_id,category_id,category_parent_id FROM navigator_publics ORDER BY ordernum ASC;");

            if (dPubCats != null)
            {
                foreach (DataRow Row in dPubCats.Rows)
                {
                    PublicCategories.Add((int)Row["id"], (string)Row["caption"]);
                }
            }

            if (dPrivCats != null)
            {
                foreach (DataRow Row in dPrivCats.Rows)
                {
                    PrivateCategories.Add(new FlatCat((int)Row["id"], (string)Row["caption"], (int)Row["min_rank"]));
                }
            }

            if (dPubItems != null)
            {
                foreach (DataRow Row in dPubItems.Rows)
                {
                    PublicItems.Add((int)Row["id"], new PublicItem((int)Row["id"], int.Parse(Row["bannertype"].ToString()), (string)Row["caption"],
                        (string)Row["image"], ((Row["image_type"].ToString().ToLower() == "internal") ? PublicImageType.INTERNAL : PublicImageType.EXTERNAL),
                        (uint)Row["room_id"], (int)Row["category_id"], (int)Row["category_parent_id"]));
                }
            }
        }

        internal Int32 GetCountForParent(Int32 ParentId)
        {
            int i = 0;

            foreach (PublicItem Item in PublicItems.Values)
            {
                if (Item.ParentId == ParentId || ParentId == -1)
                {
                    i++;
                }
            }

            return i;
        }

        internal FlatCat GetFlatCat(Int32 Id)
        {
            using (TimedLock.Lock(PrivateCategories))
            {
                foreach (FlatCat FlatCat in PrivateCategories)
                {
                    if (FlatCat.Id == Id)
                    {
                        return FlatCat;
                    }
                }
            }

            return null;
        }

        internal ServerMessage SerializeFlatCategories()
        {
            ServerMessage Cats = new ServerMessage(221);
            Cats.AppendInt32(PrivateCategories.Count);

            foreach (FlatCat FlatCat in PrivateCategories)
            {
                if (FlatCat.Id > 0)
                {
                    Cats.AppendBoolean(true);
                }

                Cats.AppendInt32(FlatCat.Id);
                Cats.AppendStringWithBreak(FlatCat.Caption);
            }

            Cats.AppendStringWithBreak("");

            return Cats;
        }

        internal ServerMessage SerializePublicRooms()
        {
            ServerMessage Frontpage = new ServerMessage(450);
            Frontpage.AppendInt32(GetCountForParent(-1));

            foreach (PublicItem Pub in PublicItems.Values)
            {
                Pub.Serialize(Frontpage);
            }

            return Frontpage;
        }

        internal static ServerMessage SerializeFavoriteRooms(GameClient Session)
        {
            ServerMessage Rooms = new ServerMessage(451);
            Rooms.AppendInt32(0);
            Rooms.AppendInt32(6);
            Rooms.AppendStringWithBreak("");
            Rooms.AppendInt32(Session.GetHabbo().FavoriteRooms.Count);

            using (TimedLock.Lock(Session.GetHabbo().FavoriteRooms))
            {
                foreach (uint Id in Session.GetHabbo().FavoriteRooms)
                {
                    ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Id).Serialize(Rooms, false);
                }
            }

            return Rooms;
        }

        internal static ServerMessage SerializeRecentRooms(GameClient Session)
        {
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                DataTable Data = dbClient.ReadDataTable("SELECT rooms.id, rooms.caption, rooms.description, rooms.roomtype, rooms.owner, rooms.state, rooms.category, rooms.users_now, rooms.users_max, rooms.model_name, rooms.public_ccts, rooms.score, rooms.allow_pets, rooms.allow_pets_eat, rooms.allow_walkthrough, rooms.allow_hidewall, rooms.password, rooms.wallpaper, rooms.floor, rooms.landscape, rooms.icon_items, rooms.icon_bg, rooms.icon_fg, rooms.tags, rooms.allow_rightsoverride FROM rooms JOIN user_roomvisits ON rooms.id = user_roomvisits.room_id WHERE user_roomvisits.user_id = " + Session.GetHabbo().Id + " ORDER BY entry_timestamp DESC LIMIT 50;");

                List<RoomData> ValidRecentRooms = new List<RoomData>();
                List<uint> RoomsListed = new List<uint>();

                if (Data != null)
                {
                    foreach (DataRow Row in Data.Rows)
                    {

                        RoomData tData = ButterflyEnvironment.GetGame().GetRoomManager().FetchRoomData((uint)Row["id"], Row);
                        tData.Fill(Row);
                        ValidRecentRooms.Add(tData);
                        RoomsListed.Add(tData.Id);
                    }
                }

                ServerMessage Rooms = new ServerMessage(451);
                Rooms.AppendInt32(0);
                Rooms.AppendInt32(7);
                Rooms.AppendStringWithBreak("");
                Rooms.AppendInt32(ValidRecentRooms.Count);

                foreach (RoomData _Data in ValidRecentRooms)
                {
                    _Data.Serialize(Rooms, false);
                }

                return Rooms;
            }
        }

        internal static ServerMessage SerializeEventListing(int Category)
        {
            ServerMessage Message = new ServerMessage(451);
            Message.AppendInt32(Category);
            Message.AppendInt32(12);
            Message.AppendStringWithBreak("");

            List<Room> EventRooms = ButterflyEnvironment.GetGame().GetRoomManager().GetEventRoomsForCategory(Category);
            Message.AppendInt32(EventRooms.Count);

            foreach (Room Room in EventRooms)
            {
                RoomData Data = Room.GetRoomData;
                Data.Serialize(Message, true);
            }

            return Message;
        }

        internal static ServerMessage SerializePopularRoomTags()
        {
            Dictionary<string, int> Tags = new Dictionary<string, int>();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                DataTable Data = dbClient.ReadDataTable("SELECT tags,users_now FROM rooms WHERE roomtype = 'private' AND users_now > 0 ORDER BY users_now DESC LIMIT 50;");


                if (Data != null)
                {
                    foreach (DataRow Row in Data.Rows)
                    {
                        List<string> RoomTags = new List<string>();

                        foreach (string Tag in Row["tags"].ToString().Split(','))
                        {
                            RoomTags.Add(Tag);
                        }

                        foreach (string Tag in RoomTags)
                        {
                            if (Tags.ContainsKey(Tag))
                            {
                                Tags[Tag] += (int)Row["users_now"];
                            }
                            else
                            {
                                Tags.Add(Tag, (int)Row["users_now"]);
                            }
                        }
                    }
                }

                List<KeyValuePair<string, int>> SortedTags = new List<KeyValuePair<string, int>>(Tags);

                SortedTags.Sort(

                    delegate(KeyValuePair<string, int> firstPair,

                    KeyValuePair<string, int> nextPair)
                    {
                        return firstPair.Value.CompareTo(nextPair.Value);
                    }

                );

                ServerMessage Message = new ServerMessage(452);
                Message.AppendInt32(SortedTags.Count);

                foreach (KeyValuePair<string, int> TagData in SortedTags)
                {
                    Message.AppendStringWithBreak(TagData.Key);
                    Message.AppendInt32(TagData.Value);
                }

                return Message;
            }
        }

        internal static ServerMessage SerializeSearchResults(string SearchQuery)
        {
            DataTable Data = new DataTable();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                SearchQuery = ButterflyEnvironment.FilterInjectionChars(SearchQuery.ToLower()).Trim();

                if (SearchQuery.Length > 0)
                {
                    dbClient.AddParamWithValue("query", SearchQuery + "%");
                    dbClient.AddParamWithValue("tags_query", "%" + SearchQuery + "%");
                    Data = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags, allow_rightsoverride FROM rooms WHERE caption LIKE @query AND roomtype = 'private' OR tags LIKE @tags_query AND roomtype = 'private' OR owner LIKE @query AND roomtype = 'private' ORDER BY users_now DESC");
                }
            }

            List<RoomData> Results = new List<RoomData>();

            if (Data != null)
            {
                foreach (DataRow Row in Data.Rows)
                {
                    RoomData RData = ButterflyEnvironment.GetGame().GetRoomManager().FetchRoomData((uint)Row["id"], Row);
                    Results.Add(RData);
                }
            }

            ServerMessage Message = new ServerMessage(451);
            Message.AppendInt32(1);
            Message.AppendInt32(9);
            Message.AppendStringWithBreak(SearchQuery);
            Message.AppendInt32(Results.Count);

            foreach (RoomData Room in Results)
            {
                Room.Serialize(Message, false);
            }

            return Message;
        }

        internal static byte[] SerializeRoomListing(GameClient Session, int Mode)
        {
            if (Mode != -2) 
                return GetDynamicNavigatorPacket(Session, Mode).GetBytes();
            else
            {
                byte[] DataToreturn = ButterflyEnvironment.GetGame().GetNaviCache().GetPacket(Mode);
                if (DataToreturn != null)
                    return DataToreturn;
                else return GetDynamicNavigatorPacket(null, Mode).GetBytes();
            }
        }

        internal static ServerMessage GetDynamicNavigatorPacket(GameClient Session, int Mode)
        {
            ServerMessage Rooms = new ServerMessage(451);

            if (Mode >= -1)
            {
                Rooms.AppendInt32(Mode);
                Rooms.AppendInt32(1);
            }
            else if (Mode == -2)
            {
                Rooms.AppendInt32(0);
                Rooms.AppendInt32(2);
            }
            else if (Mode == -3)
            {
                Rooms.AppendInt32(0);
                Rooms.AppendInt32(5);
            }
            else if (Mode == -4)
            {
                Rooms.AppendInt32(0);
                Rooms.AppendInt32(3);
            }
            else if (Mode == -5)
            {
                Rooms.AppendInt32(0);
                Rooms.AppendInt32(4);
            }

            Rooms.AppendStringWithBreak("");

            List<RoomData> Data = new List<RoomData>();

            switch (Mode)
            {
                case -5:
                case -4:
                    //Rooms.AppendInt32(0);
                    //Rooms.AppendInt32(Mode * (-1) - 1);
                    //Rooms.AppendStringWithBreak("");
                    List<string> FriendsNames = new List<string>();
                    Dictionary<int, Room> FriendsRooms = new Dictionary<int, Room>();
                    Hashtable UserFriends = Session.GetHabbo().GetMessenger().GetBuddies().Clone() as Hashtable;
                    Dictionary<RoomData, int> RoomData = new Dictionary<RoomData, int>();

                    foreach (MessengerBuddy Homie in UserFriends.Values)
                    {
                        if (!Homie.IsOnline)
                            continue;
                        if (!Homie.InRoom)
                            continue;

                        GameClient User = ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(Homie.Id);
                        if (User == null)
                            continue;
                        if (User.GetHabbo() == null)
                            continue;
                        if (User.GetHabbo().CurrentRoom == null)
                            continue;
                        RoomData HomiesRoom = User.GetHabbo().CurrentRoom.GetRoomData;

                        if (!RoomData.ContainsKey(HomiesRoom))
                        RoomData.Add(HomiesRoom, HomiesRoom.UsersNow);
                    }
                    //Rooms.AppendInt32(RoomData.Count);

                    var OrderedRooms = from k in RoomData.Keys
                                       orderby k.UsersNow descending
                                       select k;

                    FriendsNames.Clear();
                    FriendsRooms.Clear();
                    UserFriends.Clear();
                    RoomData.Clear();

                    foreach (RoomData roomdata in OrderedRooms)
                        Data.Add(roomdata);
                    //roomdata.Serialize(Rooms, false);

                    //return Rooms;

                    break;

                case -3:
                    // Rooms.AppendInt32(0);
                    /// Rooms.AppendInt32(5);
                    // Rooms.AppendStringWithBreak("");
                    //dbClient.AddParamWithValue("name", Session.GetHabbo().Username);
                    //Data = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags FROM rooms WHERE owner = @name ORDER BY id ASC");
                    //Data = Session.GetHabbo().GetUserData.GetUsersRooms;
                    // Rooms.AppendInt32(Session.GetHabbo().UsersRooms.Count);
                    foreach (RoomData roomdata in Session.GetHabbo().UsersRooms)
                        Data.Add(roomdata);
                    //roomdata.Serialize(Rooms, false);

                    //return Rooms;
                    break;

                case -2:
                    //Rooms.AppendInt32(0);
                    //Rooms.AppendInt32(2);
                    //Rooms.AppendStringWithBreak("");
                    DataTable dTable;
                    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                        dTable = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags, allow_rightsoverride FROM rooms WHERE score > 0 AND roomtype = 'private' ORDER BY score DESC LIMIT 40");

                    foreach (DataRow Row in dTable.Rows)
                    {
                        Data.Add(ButterflyEnvironment.GetGame().GetRoomManager().FetchRoomData((uint)Row["id"], Row));
                    }
                    break;

                case -1:
                    //Rooms.AppendInt32(-1);
                    //Rooms.AppendInt32(1);
                    //Rooms.AppendStringWithBreak("");
                    Dictionary<Room, int> ActiveRooms = ButterflyEnvironment.GetGame().GetRoomManager().GetActiveRoomList();

                    //Rooms.AppendInt32(ActiveRooms.Count);
                    var items = from k in ActiveRooms.Keys
                                orderby k.UserCount descending
                                select k;

                    //LINQ FTW!!!!
                    int max = 0;

                    foreach (Room _room in items)
                    {
                        if (max >= 40)
                            break;
                        max++;
                        Data.Add(_room.GetRoomData);
                    }

                    //return Rooms;
                    break;

                default:
                    // Rooms.AppendInt32(Mode);
                    // Rooms.AppendInt32(1);

                    Dictionary<Room, int> tActiveRooms = ButterflyEnvironment.GetGame().GetRoomManager().GetActiveRoomList();

                    var RoomDataFromDryCache = from k in ButterflyEnvironment.GetGame().GetRoomManager().mDryCache
                                               where k.Category == Mode
                                               select k;

                    var RoomDataFromActiveRooms = from k in tActiveRooms.Keys
                                                  where k.Category == Mode
                                                  orderby k.UserCount descending
                                                  select k;

                    int ListCount = 0;
                    foreach (Room _Data in RoomDataFromActiveRooms)
                    {
                        if (ListCount > 40)
                            break;

                        //DataToLoad.Add(_Data.GetRoomData);
                        Data.Add(_Data.GetRoomData);
                        ListCount++;
                    }

                    foreach (RoomData _Data in RoomDataFromDryCache)
                    {
                        if (ListCount > 40)
                            break;

                        if (Data.Contains(_Data))
                            Data.Remove(_Data);

                        Data.Add(_Data);
                        ListCount++;
                    }

                    // Rooms.AppendInt32(DataToLoad.Count);

                    //foreach (RoomData tData in DataToLoad)
                    //    tData.Serialize(Rooms, false);

                    // return Rooms;
                    //Rooms.AppendInt32(RoomDataFromDryCache.le
                    //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                    //    Data = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags FROM rooms WHERE category = '" + Mode + "' AND roomtype = 'private' ORDER BY users_now DESC LIMIT 40");

                    break;
            }

            Rooms.AppendInt32(Data.Count);

            foreach (RoomData tData in Data)
            {
                tData.Serialize(Rooms, false);
            }

            return Rooms;
        }
    }
}
