﻿using System;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;

namespace Butterfly.HabboHotel.Navigators
{
    enum PublicImageType
    {
        INTERNAL = 0,
        EXTERNAL = 1
    }

    class PublicItem
    {
        private int BannerId;

        internal int Type;

        internal string Caption;
        internal string Image;
        internal PublicImageType ImageType;

        internal UInt32 RoomId;
        internal int CategoryId;

        internal int ParentId;

        internal int Id
        {
            get
            {
                return BannerId;
            }
        }

        internal Boolean IsCategory
        {
            get
            {
                if (CategoryId > 0)
                {
                    return true;
                }

                return false;
            }
        }

        internal RoomData RoomData
        {
            get
            {
                if (IsCategory)
                {
                    return new RoomData();
                }

                
                if (ButterflyEnvironment.GetGame() == null)
                    throw new NullReferenceException();
                if (ButterflyEnvironment.GetGame().GetRoomManager() == null)
                    throw new NullReferenceException();
                if (ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId) == null)
                    throw new NullReferenceException();
                return ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId);
            }
        }

        internal PublicItem(Int32 Id, int Type, string Caption, string Image, PublicImageType ImageType, UInt32 RoomId, int CategoryId, int ParentId)
        {
            this.BannerId = Id;
            this.Type = Type;
            this.Caption = Caption;
            this.Image = Image;
            this.ImageType = ImageType;
            this.RoomId = RoomId;
            this.CategoryId = CategoryId;
            this.ParentId = ParentId;
        }

        internal void Serialize(ServerMessage Message)
        {
            Message.AppendInt32(Id);

            if (IsCategory)
            {
                Message.AppendStringWithBreak(Caption);
            }
            else
            {
                Message.AppendStringWithBreak(RoomData.Name);
            }

            Message.AppendStringWithBreak(RoomData.Description);
            Message.AppendInt32(Type);
            Message.AppendStringWithBreak(Caption);
            Message.AppendStringWithBreak((ImageType == PublicImageType.EXTERNAL) ? Image : "");

            if (!IsCategory)
            {
                Message.AppendUInt(0);
                Message.AppendInt32(RoomData.UsersNow);
                Message.AppendInt32(3);
                Message.AppendStringWithBreak((ImageType == PublicImageType.INTERNAL) ? Image : "");
                Message.AppendUInt(1337);
                Message.AppendInt32(0);
                Message.AppendStringWithBreak(RoomData.CCTs);
                Message.AppendInt32(RoomData.UsersMax);
                Message.AppendUInt(RoomId);
            }
            else
            {
                Message.AppendInt32(0);
                Message.AppendInt32(4);
                Message.AppendInt32(CategoryId);
            }
        }
    }
}
