﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Butterfly.Storage;

namespace Butterfly.HabboHotel.RoomBots
{
    class BotManager
    {
        private List<RoomBot> Bots;

        internal BotManager()
        {
            Bots = new List<RoomBot>();
        }

        internal void LoadBots(DatabaseClient dbClient)
        {
            Bots = new List<RoomBot>();

            DataTable Data = dbClient.ReadDataTable("SELECT id, room_id, ai_type, walk_mode, name, motto, look, x, y, z, rotation, min_x, min_y, max_x, max_y FROM bots;");
            DataTable BotResponses = dbClient.ReadDataTable("SELECT id, bot_id, keywords, response_text, mode, serve_id FROM bots_responses;");
            DataTable BotSpeech = dbClient.ReadDataTable("SELECT text, shout, bot_id FROM bots_speech;");

            List<BotResponse> Responses = new List<BotResponse>();
            List<RandomSpeech> Speeches = new List<RandomSpeech>();

            foreach (DataRow Row in BotResponses.Rows)
            {
                Responses.Add(new BotResponse((UInt32)Row["bot_id"], (string)Row["keywords"], (string)Row["response_text"], Row["mode"].ToString(), (int)Row["serve_id"]));
            }

            foreach (DataRow Row in BotSpeech.Rows)
            {
                Speeches.Add(new RandomSpeech((string)Row["text"], ButterflyEnvironment.EnumToBool(Row["shout"].ToString()), (UInt32)Row["bot_id"]));
            }


            if (Data == null)
            {
                return;
            }

            foreach (DataRow Row in Data.Rows)
            {
                string BotAI = (string)Row["ai_type"];
                AIType BotAIType;
                switch (BotAI)
                {
                    case "generic":
                        BotAIType = AIType.Generic;
                        break;
                    case "guide":
                        BotAIType = AIType.Guide;
                        break;
                    case "pet":
                        BotAIType = AIType.Pet;
                        break;
                    default:
                        BotAIType = AIType.Generic;
                        break;
                }

                Bots.Add(new RoomBot((uint)Row["id"], (uint)Row["room_id"], BotAIType, (string)Row["walk_mode"],
                    (String)Row["name"], (string)Row["motto"], (String)Row["look"], (int)Row["x"], (int)Row["y"], (int)Row["z"],
                    (int)Row["rotation"], (int)Row["min_x"], (int)Row["min_y"], (int)Row["max_x"], (int)Row["max_y"], ref Speeches, ref Responses));
            }
        }

        //internal bool RoomHasBots(UInt32 RoomId)
        //{
        //    return (GetBotsForRoom(RoomId).Count >= 1);
        //}

        internal List<RoomBot> GetBotsForRoom(UInt32 RoomId)
        {
            //List<RoomBot> List = new List<RoomBot>();


            //List.AddRange(from p in Bots where p.BotId == RoomId select p);
            //using (TimedLock.Lock(Bots))
            //{
            //    foreach (RoomBot Bot in Bots)
            //    {
            //        if (Bot.RoomId == RoomId)
            //        {
            //            List.Add(Bot);
            //        }
            //    }
            //}

            //return List;

            return new List<RoomBot>(from p in Bots where p.BotId == RoomId select p);
        }

        internal RoomBot GetBot(UInt32 BotId)
        {
            return (from p in Bots where p.BotId == BotId select p).SingleOrDefault();
            //using (TimedLock.Lock(Bots))
            //{
            //    foreach (RoomBot Bot in Bots)
            //    {
            //        if (Bot.BotId == BotId)
            //        {
            //            return Bot;
            //        }
            //    }
            //}

            //return null;
        }
    }
}
