﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Butterfly.Core;
using Butterfly.HabboHotel.Catalogs;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.Messages;
using Butterfly.Storage;
using Butterfly_Emulator.HabboHotel.Items;

namespace Butterfly.HabboHotel.Rooms
{
    enum MatrixState
    {
        BLOCKED = 0,
        WALKABLE = 1,
        WALKABLE_LASTSTEP = 2
    }

    class Room
    {
        private UInt32 Id;

        internal string Name;
        internal string Description;
        internal string Type;
        internal string Owner;
        internal string Password;
        internal int Category;
        internal int State;
        internal int UsersNow;
        internal int UsersMax;
        internal string ModelName;
        internal string CCTs;
        internal int Score;
        internal List<string> Tags;
        internal bool AllowPets;
        internal bool AllowPetsEating;
        internal bool AllowWalkthrough;
        internal bool Hidewall;
        internal bool RoomMuted;
        internal bool DiagonalEnabled;

        private Thread mCycleThread;
        private bool mCycleEnded;

        //internal List<RoomUser> UserList;
        internal RoomUser[] UserList;
        internal int UserCounter = 0;

        private int IdleTime;

        internal RoomIcon myIcon;

        internal List<uint> UsersWithRights;
        internal bool EveryoneGotRights;
        private Dictionary<UInt32, Double> Bans;

        internal RoomEvent Event;

        internal string Wallpaper;
        internal string Floor;
        internal string Landscape;

        //internal List<RoomItem> Items;
        private Hashtable mFloorItems;

        private Hashtable mRemovedItems;
        private Hashtable mMovedItems;
        private Hashtable mAddedItems;

        private Hashtable mWallItems;
        internal MoodlightData MoodlightData;

        internal List<Trade> ActiveTrades;

        //internal bool KeepAlive;

        //internal MatrixState[,] Matrix;
        ////internal bool[,] UserMatrix;
        ////internal Coord[,] BedMatrix;
        ////internal double[,] HeightMatrix;

        internal double[,] TopStackHeight;
        private byte[,] mGameMap;
        private byte[,] mPoolMap;
        private double[,] mItemHeightMap;
        
        private bool CycleActive = false;
        private RoomModel mModel;

        private bool mGotRollers;

        //private double mRollerNext;
        //private int mRollerDay;

        internal Boolean HasOngoingEvent
        {
            get
            {
                if (Event != null)
                {
                    return true;
                }

                return false;
            }
        }

        internal RoomIcon Icon
        {
            get
            {
                return myIcon;
            }

            set
            {
                myIcon = value;
            }
        }

        internal bool GotRollers
        {
            set
            {
                mGotRollers = value;
            }
        }

        internal Int32 UserCount
        {
            get
            {
                int count = 0;
                if (UserList == null)
                    return 0; 
                for (int i = 0; i < UserList.Length; i++)
                {
                    if (UserList[i] == null)
                        continue;
                    if (UserList[i].IsBot)
                        continue;

                    count++;
                }

                return count;
            }
        }

        internal int TagCount
        {
            get
            {
                return Tags.Count;
            }
        }

        internal RoomModel Model
        {
            get
            {
                return mModel;
            }
        }

        internal uint RoomId
        {
            get
            {
                return Id;
            }
        }

        internal Hashtable FloorItems
        {
            get
            {
                return mFloorItems.Clone() as Hashtable;
                //List<RoomItem> FloorItems = new List<RoomItem>();

                //using (TimedLock.Lock(Items)
                //{
                //    foreach (RoomItem Item in Items)
                //    {
                //        if (!Item.IsFloorItem)
                //        {
                //            continue;
                //        }

                //        FloorItems.Add(Item);
                //    }
                //}

                //return FloorItems;
            }
        }

        internal Hashtable WallItems
        {
            get
            {
                return mWallItems.Clone() as Hashtable;
                //List<RoomItem> WallItems = new List<RoomItem>();

                //using (TimedLock.Lock(Items)
                //{
                //    foreach (RoomItem Item in Items)
                //    {
                //        if (!Item.IsWallItem)
                //        {
                //            continue;
                //        }

                //        WallItems.Add(Item);
                //    }
                //}

                //return WallItems;
            }
        }

        internal Boolean CanTradeInRoom
        {
            get
            {
                if (IsPublic)
                {
                    return false;
                }

                return true;
            }
        }

        internal bool IsPublic
        {
            get
            {
                if (Type == "public")
                {
                    return true;
                }

                return false;
            }
        }

        internal int PetCount
        {
            get
            {
                int c = 0;

                for (int i = 0; i < UserList.Length; i++)
                {
                    RoomUser Current = UserList[i];
                    if (Current != null)
                        if (Current.IsPet)
                            c++;

                }
                return c;
            }
        }

        private int mRollerSpeed;
        private RoomData mRoomData;

        internal RoomData GetRoomData
        {
            get
            {
                mRoomData.Fill(this);
                return mRoomData;
            }
        }

        internal Room(UInt32 Id, string Name, string Description, string Type, string Owner, int Category,
            int State, int UsersMax, string ModelName, string CCTs, int Score, List<string> Tags, bool AllowPets,
            bool AllowPetsEating, bool AllowWalkthrough, bool Hidewall, RoomIcon Icon, string Password, string Wallpaper, string Floor,
            string Landscape, RoomData RoomData, bool RightOverride)
        {
            this.mDisposed = false;
            this.Id = Id;
            this.Name = Name;
            this.Description = Description;
            this.Owner = Owner;
            this.Category = Category;
            this.Type = Type;
            this.State = State;
            this.UsersNow = 0;
            this.UsersMax = UsersMax + 50;
            this.ModelName = ModelName;
            this.CCTs = CCTs;
            this.Score = Score;
            this.Tags = Tags;
            this.AllowPets = AllowPets;
            this.AllowPetsEating = AllowPetsEating;
            this.AllowWalkthrough = AllowWalkthrough;
            this.Hidewall = Hidewall;
            this.UserCounter = 0;
            this.UserList = new RoomUser[UsersMax + 10];
            this.myIcon = Icon;
            this.Password = Password;
            this.Bans = new Dictionary<UInt32, double>();
            //this.Event = null;
            this.Wallpaper = Wallpaper;
            this.Floor = Floor;
            this.Landscape = Landscape;
            this.mWallItems = new Hashtable();
            this.mFloorItems = new Hashtable();
            this.ActiveTrades = new List<Trade>();
            this.mModel = ButterflyEnvironment.GetGame().GetRoomManager().GetModel(this.ModelName, this.Id);
            this.mCycleEnded = false;
            this.DiagonalEnabled = true;
            this.mRoomData = RoomData;
            this.EveryoneGotRights = RightOverride;

            //this.UserMatrix = new bool[Model.MapSizeX, Model.MapSizeY];
            this.mGameMap = new byte[Model.MapSizeX, Model.MapSizeY];
            this.mItemHeightMap = new double[Model.MapSizeX, Model.MapSizeY];

            //this.mCycleThread = new Thread(new ThreadStart(ProcessRoom));
            //ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessRoom), null);

            this.IdleTime = 0;

            this.RoomMuted = false;
            //this.KeepAlive = true;
            this.mGotRollers = false;
            this.mRoolerCycle = 0;
            this.mRollerSpeed = 4;

            mRemovedItems = new Hashtable();
            mMovedItems = new Hashtable();
            mAddedItems= new Hashtable();

            LoadRights();
            LoadFurniture();

            GenerateMaps();

            //mCycleThread.Start();
            //Task.Factory.StartNew(() => ProcessRoom(null), TaskCreationOptions.LongRunning);
            
            CycleActive = true;
        }

        internal void InitBots()
        {
            List<RoomBot> Bots = ButterflyEnvironment.GetGame().GetBotManager().GetBotsForRoom(RoomId);

            foreach (RoomBot Bot in Bots)
            {
                DeployBot(Bot);
            }
        }

        internal void InitPets()
        {
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                DataTable Data = dbClient.ReadDataTable("SELECT id, user_id, room_id, name, type, race, color, expirience, energy, nutrition, respect, createstamp, x, y, z FROM user_pets WHERE room_id = " + RoomId + ";");

                if (Data == null)
                {
                    return;
                }

                foreach (DataRow Row in Data.Rows)
                {
                    Pet Pet = Catalog.GeneratePetFromRow(Row);
                    List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                    List<BotResponse> BotResponse = new List<RoomBots.BotResponse>();
                    DeployBot(new RoomBot(Pet.PetId, RoomId, AIType.Pet, "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                }
            }
        }

        //internal List<Pet> GetPetList()
        //{
        //    List<Pet> ReturnList = new List<Pet>();

        //    for (int i = 0; i < UserList.Length; i++)
        //    {
        //        if (UserList[i] != null)
        //            if (UserList[i].IsPet)
        //                ReturnList.Add(UserList[i].PetData);
        //    }

        //    return ReturnList;
        //}

        internal RoomUser DeployBot(RoomBot Bot)
        {
            return DeployBot(Bot, null);
        }

        internal RoomUser DeployBot(RoomBot Bot, Pet PetData)
        {
            RoomUser BotUser = new RoomUser(0, RoomId, UserCounter++); 
            int PersonalID = GetFreeID();
            BotUser.InternalRoomID = PersonalID;
            this.UserList[PersonalID] = BotUser;

            if ((Bot.X > 0 && Bot.Y > 0) && Bot.X < Model.MapSizeX && Bot.Y < Model.MapSizeY)
            {
                BotUser.SetPos(Bot.X, Bot.Y, Bot.Z);
                BotUser.SetRot(Bot.Rot);
            }
            else
            {
                Bot.X = Model.DoorX;
                Bot.Y = Model.DoorY;

                BotUser.SetPos(Model.DoorX, Model.DoorY, Model.DoorZ);
                BotUser.SetRot(Model.DoorOrientation);
            }

            //UserMatrix[Bot.X, Bot.Y] = true;

            BotUser.BotData = Bot;
            BotUser.BotAI = Bot.GenerateBotAI(BotUser.VirtualId);

            if (BotUser.IsPet)
            {
                BotUser.BotAI.Init((int)Bot.BotId, BotUser.VirtualId, RoomId);
                BotUser.PetData = PetData;
                BotUser.PetData.VirtualId = BotUser.VirtualId;
            }
            else
            {
                BotUser.BotAI.Init(-1, BotUser.VirtualId, RoomId);
            }

            //UserList[GetFreeID()] = BotUser;

            //using (TimedLock.Lock(this.UserList)
            //{
            //    UserList.Add(BotUser);
            //}

            UpdateUserStatus(BotUser);
            BotUser.UpdateNeeded = true;

            ServerMessage EnterMessage = new ServerMessage(28);
            EnterMessage.AppendInt32(1);
            BotUser.Serialize(EnterMessage);
            SendMessage(EnterMessage);

            BotUser.BotAI.OnSelfEnterRoom();

            return BotUser;
        }

        private int GetFreeID()
        {
            return Array.IndexOf(UserList, null);
        }

        internal void RemoveBot(int VirtualId, bool Kicked)
        {
            RoomUser User = GetRoomUserByVirtualId(VirtualId);

            if (User == null || !User.IsBot)
            {
                return;
            }

            User.BotAI.OnSelfLeaveRoom(Kicked);

            ServerMessage LeaveMessage = new ServerMessage(29);
            LeaveMessage.AppendRawInt32(User.VirtualId);
            SendMessage(LeaveMessage);

            //UserMatrix[User.X, User.Y] = false;

            //UserList.Remove(User);

            uint UserID = User.HabboId;

            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser Current = UserList[i];
                if (Current == null)
                    continue;
                if (Current.HabboId == UserID)
                    UserList[i] = null;
            }
        }

        internal void OnUserSay(RoomUser User, string Message, bool Shout)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser Usr = UserList[i];
                if (Usr == null)
                    continue;

                if (!Usr.IsBot)
                {
                    continue;
                }

                if (Shout)
                {
                    Usr.BotAI.OnUserShout(User, Message);
                }
                else
                {
                    Usr.BotAI.OnUserSay(User, Message);
                }
            }
        }

        //internal void RegenerateUserMatrix()
        //{
        //    //this.UserMatrix = new bool[Model.MapSizeX, Model.MapSizeY];
        //    //for (int i = 0; i < UserList.Length; i++)
        //    //{
        //    //    RoomUser Current = UserList[i];
        //    //    if (Current == null)
        //    //        continue;
        //    //    this.UserMatrix[Current.X, Current.Y] = true;
        //    //}
        //}

        internal void GenerateMaps()
        {
            // Create matrix arrays
            //Matrix = new MatrixState[Model.MapSizeX, Model.MapSizeY];
            //BedMatrix = new Coord[Model.MapSizeX, Model.MapSizeY];
            //HeightMatrix = new double[Model.MapSizeX, Model.MapSizeY];
            TopStackHeight = new double[Model.MapSizeX, Model.MapSizeY];
            mPoolMap = new byte[Model.MapSizeX, Model.MapSizeY];
            this.mGameMap = new byte[Model.MapSizeX, Model.MapSizeY];
            this.mItemHeightMap = new double[Model.MapSizeX, Model.MapSizeY];

            // Fill in the basic data based purely on the heightmap
            for (int line = 0; line < Model.MapSizeY; line++)
            {
                for (int chr = 0; chr < Model.MapSizeX; chr++)
                {
                    //Matrix[chr, line] = MatrixState.BLOCKED;
                    //BedMatrix[chr, line] = new Coord(chr, line);
                    //HeightMatrix[chr, line] = 0;
                    TopStackHeight[chr, line] = 0.0;
                    mGameMap[chr, line] = 0;
                    mPoolMap[chr, line] = 0;

                    if (chr == Model.DoorX && line == Model.DoorY)
                    {
                       // Matrix[chr, line] = MatrixState.WALKABLE_LASTSTEP;
                        mGameMap[chr, line] = 3;
                    }
                    else if (Model.SqState[chr, line] == SquareState.OPEN)
                    {
                        //Matrix[chr, line] = MatrixState.WALKABLE;
                        mGameMap[chr, line] = 1;
                    }
                    else if (Model.SqState[chr, line] == SquareState.SEAT)
                    {
                        //Matrix[chr, line] = MatrixState.WALKABLE_LASTSTEP;
                        mGameMap[chr, line] = 2;
                    }
                }
            }

            //if (!this.AllowWalkthrough)
            //{
            //    for (int i = 0; i < UserList.Length; i++)
            //    {
            //        RoomUser User = UserList[i];
            //        if (User == null)
            //            continue;
            //        mGameMap[User.X, User.Y] = 0;
            //    }
            //}

            // Loop through the items in the room
            foreach (RoomItem Item in FloorItems.Values)
            {
                try
                {
                    // If we're dealing with anything other than a floor item, skip
                    if (Item.GetBaseItem().Type != 's')
                    {
                        continue;
                    }

                    if (Item.TotalHeight > mItemHeightMap[Item.GetX, Item.GetY])
                        mItemHeightMap[Item.GetX, Item.GetY] = Item.TotalHeight;

                    // If this is a rug, ignore it.
                    if (Item.GetBaseItem().Height <= 0 && Item.GetBaseItem().InteractionType != Butterfly_Emulator.HabboHotel.Items.InteractionType.pool)
                    {
                        continue;
                    }

                    // Make sure we're the highest item here!
                    if (TopStackHeight[Item.GetX, Item.GetY] <= Item.GetZ)
                    {
                        TopStackHeight[Item.GetX, Item.GetY] = Item.GetZ;

                        // If this item is walkable and on the floor, allow users to walk here.
                        if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.pool)
                        {
                            mPoolMap[Item.GetX, Item.GetY] = 4;
                        }
                        if (Item.GetBaseItem().Walkable)
                        {
                            //Matrix[Item.GetX, Item.GetY] = MatrixState.WALKABLE;
                            //HeightMatrix[Item.GetX, Item.GetY] = Item.GetBaseItem().Height;
                            if (mGameMap[Item.GetX, Item.GetY] != 3)
                                mGameMap[Item.GetX, Item.GetY] = 1;
                        }
                        // If this item is a gate, open, and on the floor, allow users to walk here.
                        else if (Item.GetZ <= (Model.SqFloorHeight[Item.GetX, Item.GetY] + 0.1) && Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.gate && Item.ExtraData == "1")
                        {
                            //Matrix[Item.GetX, Item.GetY] = MatrixState.WALKABLE;
                            if (mGameMap[Item.GetX, Item.GetY] != 3)
                             mGameMap[Item.GetX, Item.GetY] = 1;
                        }
                        // If this item is a set or a bed, make it's square walkable (but only if last step)
                        else if (Item.GetBaseItem().IsSeat || Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.bed)
                        {
                            //Matrix[Item.GetX, Item.GetY] = MatrixState.WALKABLE_LASTSTEP;
                            mGameMap[Item.GetX, Item.GetY] = 3;
                        }
                        // Finally, if it's none of those, block the square.
                        else
                        {
                            //Matrix[Item.GetX, Item.GetY] = MatrixState.BLOCKED;
                            if (mGameMap[Item.GetX, Item.GetY] != 3)
                                mGameMap[Item.GetX, Item.GetY] = 0;
                        }
                    }

                    Dictionary<int, AffectedTile> Points = Item.GetAffectedTiles; //GetAffectedTiles(Item.GetBaseItem().Length, Item.GetBaseItem().Width, Item.GetX, Item.GetY, Item.Rot);

                    if (Points == null)
                    {
                        Points = new Dictionary<int, AffectedTile>();
                    }

                    foreach (AffectedTile Tile in Points.Values)
                    {
                        //mGameMap[Tile.X, Tile.Y] = 0;
                        if (Item.TotalHeight > mItemHeightMap[Tile.X, Tile.Y])
                            mItemHeightMap[Tile.X, Tile.Y] = Item.TotalHeight;

                        // Make sure we're the highest item here!
                        if (TopStackHeight[Tile.X, Tile.Y] <= Item.GetZ)
                        {
                            TopStackHeight[Tile.X, Tile.Y] = Item.GetZ;

                            if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.pool)
                            {
                                mPoolMap[Tile.X, Tile.Y] = 4;
                            }
                            // If this item is walkable and on the floor, allow users to walk here.
                            else if (Item.GetBaseItem().Walkable)
                            {
                                //Matrix[Tile.X, Tile.Y] = MatrixState.WALKABLE;
                                //HeightMatrix[Tile.X, Tile.Y] = Item.GetBaseItem().Height;
                                if (mGameMap[Tile.X, Tile.Y] != 3)
                                    mGameMap[Tile.X, Tile.Y] = 1;

                            }
                            // If this item is a gate, open, and on the floor, allow users to walk here.
                            else if (Item.GetZ <= (Model.SqFloorHeight[Item.GetX, Item.GetY] + 0.1) && Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.gate && Item.ExtraData == "1")
                            {
                                //Matrix[Tile.X, Tile.Y] = MatrixState.WALKABLE;
                                if (mGameMap[Tile.X, Tile.Y] != 3)
                                    mGameMap[Tile.X, Tile.Y] = 1;
                            }
                            // If this item is a set or a bed, make it's square walkable (but only if last step)
                            else if (Item.GetBaseItem().IsSeat || Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.bed)
                            {
                                ///Matrix[Tile.X, Tile.Y] = MatrixState.WALKABLE_LASTSTEP;
                                mGameMap[Tile.X, Tile.Y] = 3;
                            }
                            // Finally, if it's none of those, block the square.
                            else
                            {
                                //Matrix[Tile.X, Tile.Y] = MatrixState.BLOCKED;
                                if (mGameMap[Tile.X, Tile.Y] != 3)
                                    mGameMap[Tile.X, Tile.Y] = 0;
                            }
                        }

                        // Set bad maps
                        if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.bed)
                        {
                            mGameMap[Tile.X, Tile.Y] = 3;
                            //if (Item.Rot == 0 || Item.Rot == 4)
                            //{
                            //    BedMatrix[Tile.X, Tile.Y].Y = Item.GetY;
                            //}

                            //if (Item.Rot == 2 || Item.Rot == 6)
                            //{
                            //    BedMatrix[Tile.X, Tile.Y].X = Item.GetX;
                            //}
                        }

                    }
                }
                catch { }
            }

            if (!AllowWalkthrough)
            {
                for (int i = 0; i < UserList.Length; i++)
                {
                    RoomUser User = UserList[i];
                    if (User == null)
                        continue;
                    mGameMap[User.X, User.Y] = 0;
                }
            }
            mGameMap[Model.DoorX, Model.DoorY] = 3;
        }

        internal void LoadRights()
        {
            this.UsersWithRights = new List<uint>();

            DataTable Data = new DataTable();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                Data = dbClient.ReadDataTable("SELECT room_rights.user_id FROM room_rights WHERE room_id = '" + Id + "'");
            }

            if (Data == null)
            {
                return;
            }

            foreach (DataRow Row in Data.Rows)
            {
                this.UsersWithRights.Add((uint)Row["user_id"]);
            }
        }

        internal List<RoomItem> RemoveAllFurniture(GameClient Session)
        {
            List<RoomItem> ReturnList = new List<RoomItem>();
            foreach (RoomItem Item in FloorItems.Values)
            {
                Item.Interactor.OnRemove(Session, Item);
                ServerMessage Message = new ServerMessage(94);
                Message.AppendRawUInt(Item.Id);
                Message.AppendStringWithBreak("");
                Message.AppendBoolean(false);
                SendMessage(Message);

                //mFloorItems.Remove(Item.Id);

                ReturnList.Add(Item);
            }

            foreach (RoomItem Item in WallItems.Values)
            {
                Item.Interactor.OnRemove(Session, Item);
                ServerMessage Message = new ServerMessage(84);
                Message.AppendRawUInt(Item.Id);
                Message.AppendStringWithBreak("");
                Message.AppendBoolean(false);
                SendMessage(Message);
                //mWallItems.Remove(Item.Id);

                ReturnList.Add(Item);
            }

            mWallItems.Clear();
            mFloorItems.Clear();

            mRemovedItems.Clear();
            mMovedItems.Clear();
            mAddedItems.Clear();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("DELETE FROM room_items WHERE room_id = '" + RoomId + "'");
            }

            GenerateMaps();
            UpdateUserStatusses();

            return ReturnList;
        }

        internal void LoadFurniture()
        {
            //this.Items.Clear();
            this.mFloorItems.Clear();
            this.mWallItems.Clear();
            DataTable Data;
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                Data = dbClient.ReadDataTable("SELECT room_items.id, room_items.base_item, room_items.extra_data, room_items.x, room_items.y, room_items.z, room_items.rot, room_items.wall_pos FROM room_items WHERE room_id = '" + Id + "'");
            }

            if (Data == null)
            {
                return;
            }

            foreach (DataRow Row in Data.Rows)
            {
                RoomItem Item = new RoomItem((uint)Row["id"], RoomId, (uint)Row["base_item"], (string)Row["extra_data"],
                    (int)Row["x"], (int)Row["y"], (Double)Row["z"], (int)Row["rot"], (string)Row["wall_pos"], this);

                if (Item.IsRoller)
                    mGotRollers = true;

                if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.dimmer)
                {
                    if (MoodlightData == null)
                        MoodlightData = new MoodlightData(Item.Id);
                }
                if (mFloorItems.Contains(Item.Id))
                    mFloorItems.Remove(Item.Id);
                if (mWallItems.Contains(Item.Id))
                    mWallItems.Remove(Item.Id);

                if (Item.IsFloorItem)
                    mFloorItems.Add(Item.Id, Item);
                else
                    mWallItems.Add(Item.Id, Item);
            }
        }

        internal Boolean CheckRights(GameClient Session)
        {
            return CheckRights(Session, false);
        }

        internal Boolean CheckRights(GameClient Session, bool RequireOwnership)
        {
            try
            {
                if (Session.GetHabbo().Username == Owner)
                {
                    return true;
                }

                if (Session.GetHabbo().HasFuse("fuse_admin") || Session.GetHabbo().HasFuse("fuse_any_room_controller"))
                {
                    return true;
                }

                if (!RequireOwnership)
                {
                    if (Session.GetHabbo().HasFuse("fuse_any_room_rights"))
                        return true;
                    if (UsersWithRights.Contains(Session.GetHabbo().Id))
                        return true;
                    if (EveryoneGotRights)
                        return true;
                }
            }
            catch { }

            return false;
        }

        internal RoomItem GetItem(uint pId)
        {
            try
            {
                RoomItem FloorItem = mFloorItems[pId] as RoomItem;
                if (FloorItem != null)
                    return FloorItem;
                return mWallItems[pId] as RoomItem;

            }
            catch { }
            return null;
            //foreach (RoomItem Item in Items)
            //{
            //    if (Item.Id == Id)
            //    {
            ////        return Item;
            ////    }
            ////}

            //return null;
        }

        internal void RemoveFurniture(GameClient Session, uint pId)
        {
            RoomItem Item = GetItem(pId);

            if (Item == null)
            {
                return;
            }

            Item.Interactor.OnRemove(Session, Item);

            if (Item.IsWallItem)
            {
                ServerMessage Message = new ServerMessage(84);
                Message.AppendRawUInt(Item.Id);
                Message.AppendStringWithBreak("");
                Message.AppendBoolean(false);
                SendMessage(Message);
            }
            else if (Item.IsFloorItem)
            {
                ServerMessage Message = new ServerMessage(94);
                Message.AppendRawUInt(Item.Id);
                Message.AppendStringWithBreak("");
                Message.AppendBoolean(false);
                SendMessage(Message);
            }
            if (Item.IsWallItem)
                mWallItems.Remove(Item.Id);
            else
                mFloorItems.Remove(Item.Id);

            if (mAddedItems.Contains(Item.Id))
                mAddedItems.Remove(Item.Id);
            if (mMovedItems.Contains(Item.Id))
                mMovedItems.Remove(Item.Id);
            if (!mRemovedItems.Contains(Item.Id))
                mRemovedItems.Add(Item.Id, Item);
            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.ExecuteQuery("DELETE FROM room_items WHERE id = '" + Id + "' AND room_id = '" + RoomId + "' LIMIT 1");
            //}

            GenerateMaps();
            UpdateUserStatusses();
        }

        internal bool CanWalk(int X, int Y, bool Override)
        {
            if (AllowWalkthrough)
                return true;
            if (Override)
                return true;

            //if (Matrix[X, Y] == MatrixState.BLOCKED)
            //{
            //    return false;
            //}

            //if (X < 0 || X >= Model.MapSizeX || Y >= Model.MapSizeY || Y < 0)
            //{
            //    return false;
            //}

            if (GetUserForSquare(X, Y) != null)
                return false;

            //if (SquareHasUsers(X, Y, LastStep))
            //{
            //    return false;
            //}
            //else if (Matrix[X, Y] == MatrixState.WALKABLE_LASTSTEP && !LastStep)
            //{
            //    return false;
            //}

            return true;
        }

        //internal void FreeSqareForUsers(int pX, int pY)
        //{
        //    //UserMatrix[pX, pY] = false;
        //}

        internal void ProcessRoom()
        {
            try
            {
                //while (true)
                //{
                //DateTime TimeStart = DateTime.Now;
                if (mCycleEnded)
                    return;
                try
                {
                    CycleRollers();

                    int Usercount = 0;
                    #region cyclecode
                    int i = 0;

                    // Loop through all furni and process them if they want to be processed
                    if (mFloorItems != null)
                    {
                        foreach (RoomItem Item in FloorItems.Values)
                        {
                            if (Item.UpdateNeeded || Item.IsTrans)
                                Item.ProcessUpdates();
                        }
                    }

                    if (mWallItems != null)
                    {
                        foreach (RoomItem Item in WallItems.Values)
                        {
                            if (Item.UpdateNeeded)
                                Item.ProcessUpdates();
                        }
                    }

                    List<uint> ToRemove = new List<uint>();

                    int UserCount = 0;

                    if (UserList != null)
                    {
                        // Loop through all users and bots and process them
                        for (int j = 0; j < UserList.Length; j++)
                        {
                            RoomUser User = UserList[j];
                            if (User == null)
                                continue;

                            if (!User.IsBot)
                                Usercount++;
                            UserCount++;
                            User.IdleTime++;

                            if (!User.IsAsleep && User.IdleTime >= 600)
                            {
                                User.IsAsleep = true;

                                ServerMessage FallAsleep = new ServerMessage(486);
                                FallAsleep.AppendInt32(User.VirtualId);
                                FallAsleep.AppendBoolean(true);
                                SendMessage(FallAsleep);
                            }

                            if (User.GetClient() == null && !User.IsBot) //Possible fix for ghost bug
                            {

                                UserList[j] = null;
                                //UserMatrix[User.X, User.Y] = false;

                                mGameMap[User.X, User.Y] = User.SqState;
                                ServerMessage LeaveMessage = new ServerMessage(29);
                                LeaveMessage.AppendRawInt32(User.VirtualId);
                                SendMessage(LeaveMessage);

                                UpdateUserCount();
                            }



                            if (User.NeedsAutokick && !ToRemove.Contains(User.HabboId))
                            {
                                ToRemove.Add(User.HabboId);
                            }

                            if (User.CarryItemID > 0)
                            {
                                User.CarryTimer--;

                                if (User.CarryTimer <= 0)
                                {
                                    User.CarryItem(0);
                                }
                            }

                            if (User.SetStep)
                            {
                                if (CanWalk(User.SetX, User.SetY, User.AllowOverride))
                                {
                                    User.X = User.SetX;
                                    User.Y = User.SetY;
                                    User.Z = User.SetZ;

                                    if (User.X == Model.DoorX && User.Y == Model.DoorY && !ToRemove.Contains(User.HabboId) && !User.IsBot)
                                        ToRemove.Add(User.HabboId);

                                    UpdateUserStatus(User);
                                }

                                User.SetStep = false;
                            }



                            //if (User.Path == null)
                            //    continue;
                            //if (User.PathRecalcNeeded)
                            //{
                            //    //Pathfinder Pathfinder = new Pathfinder(this, User, this.AllowWalkthrough, ref this.BedMatrix, ref this.UserMatrix, ref this.Matrix);

                            //    //User.GoalX = User.PathRecalcX;
                            //    //User.GoalY = User.PathRecalcY;

                            //    //User.Path.Clear();
                            //    //User.Path = Pathfinder.FindPath();

                            //    //if (User.Path.Count > 1)
                            //    //{
                            //    //    User.PathStep = 1;
                            //    //    User.IsWalking = true;
                            //    //    User.PathRecalcNeeded = false;
                            //    //}
                            //    //else
                            //    //{
                            //    //    User.PathRecalcNeeded = false;
                            //    //    User.Path.Clear();
                            //    //}

                            //}

                            if (User.CurrentSwimType != SwimType.Walk && mPoolMap[User.X, User.Y] != 4)
                            {
                                User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().ApplyEffect(-1);
                                User.CurrentSwimType = SwimType.Walk;
                            }

                            if (User.IsWalking && !User.Freezed)
                            {
                                SquarePoint Point = DreamPathfinder.GetNextStep(User.X, User.Y, User.GoalX, User.GoalY, mGameMap, mItemHeightMap, mModel.MapSizeX, mModel.MapSizeY, User.AllowOverride, DiagonalEnabled);
                                if (Point.X == User.X && Point.Y == User.Y) //No path found, or reached goal (:
                                {
                                    //User.Path.Clear();
                                    User.IsWalking = false;
                                    User.RemoveStatus("mv");
                                    //User.PathRecalcNeeded = false;

                                    UpdateUserStatus(User);
                                }
                                else
                                {
                                    //User.UpdateNeeded = true;
                                    int nextX = Point.X;
                                    int nextY = Point.Y;

                                    User.RemoveStatus("mv");

                                    double nextZ = SqAbsoluteHeight(nextX, nextY, GetFurniObjects(nextX, nextY));

                                    User.Statusses.Remove("lay");
                                    User.Statusses.Remove("sit");
                                    User.AddStatus("mv", nextX + "," + nextY + "," + nextZ.ToString().Replace(',', '.'));

                                    int newRot = Rotation.Calculate(User.X, User.Y, nextX, nextY);

                                    User.RotBody = newRot;
                                    User.RotHead = newRot;

                                    User.SetStep = true;
                                    //User.SetX = BedMatrix[nextX, nextY].X;
                                    //User.SetY = BedMatrix[nextX, nextY].Y;
                                    User.SetX = nextX;
                                    User.SetY = nextY;
                                    User.SetZ = nextZ;
                                    //User.X = Point.X;
                                    //User.Y = Point.Y;
                                    if (User.CurrentSwimType == SwimType.Walk && mPoolMap[nextX, nextY] == 4)
                                    {
                                        //Todo: Determine if its a deep or normal water path
                                        User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().ApplyEffect(28);
                                        User.CurrentSwimType = SwimType.Normal;
                                    }
                                    else if (User.CurrentSwimType != SwimType.Walk && mPoolMap[nextX, nextY] != 4)
                                    {
                                        User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().ApplyEffect(-1);
                                        User.CurrentSwimType = SwimType.Walk;
                                    }


                                    mGameMap[User.X, User.Y] = User.SqState; // REstore the old one
                                    User.SqState = mGameMap[User.SetX, User.SetY];//Backup the new one

                                    if (!AllowWalkthrough)
                                        mGameMap[nextX, nextY] = 0;
                                }
                                //if (invalidSetStep || User.PathStep >= User.Path.Count || User.GoalX == User.X && User.Y == User.GoalY)
                                //{
                                //    User.Path.Clear();
                                //    User.IsWalking = false;
                                //    User.RemoveStatus("mv");
                                //    User.PathRecalcNeeded = false;

                                //    if (User.X == Model.DoorX && User.Y == Model.DoorY && !ToRemove.Contains(User.HabboId) && !User.IsBot)
                                //    {
                                //        ToRemove.Add(User.HabboId);
                                //    }

                                //    UpdateUserStatus(User);
                                //}
                                //else
                                //{
                                //    int k = (User.Path.Count - User.PathStep) - 1;
                                //    Coord NextStep = User.Path[k];
                                //    User.PathStep++;

                                //    int nextX = NextStep.X;
                                //    int nextY = NextStep.Y;

                                //    User.RemoveStatus("mv");

                                //    bool LastStep = false;

                                //    if (nextX == User.GoalX && nextY == User.GoalY)
                                //        LastStep = true;

                                //    if (CanWalk(nextX, nextY, 0, LastStep) || User.AllowOverride)
                                //    {
                                //        double nextZ = SqAbsoluteHeight(nextX, nextY, GetFurniObjects(nextX, nextY));

                                //        User.Statusses.Remove("lay");
                                //        User.Statusses.Remove("sit");
                                //        User.AddStatus("mv", nextX + "," + nextY + "," + nextZ.ToString().Replace(',', '.'));

                                //        int newRot = Rotation.Calculate(User.X, User.Y, nextX, nextY);

                                //        User.RotBody = newRot;
                                //        User.RotHead = newRot;

                                //        User.SetStep = true;
                                //        User.SetX = BedMatrix[nextX, nextY].X;
                                //        User.SetY = BedMatrix[nextX, nextY].Y;
                                //        User.SetZ = nextZ;
                                //    }
                                //    else
                                //    {
                                //        User.IsWalking = false;
                                //    }
                                //}

                                User.UpdateNeeded = true;
                            }
                            else
                            {
                                if (User.Statusses.ContainsKey("mv"))
                                {
                                    User.RemoveStatus("mv");
                                    User.UpdateNeeded = true;
                                }
                            }

                            if (User.IsBot)
                                User.BotAI.OnTimerTick();
                            else
                                i++; // we do not count bots. we do not take kindly to their kind 'round 'ere.

                        }
                    }

                    foreach (uint toRemove in ToRemove)
                        RemoveUserFromRoom(ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(toRemove), true, false);

                    // Update idle time
                    if (i >= 1)
                        this.IdleTime = 0;
                    else
                        this.IdleTime++;

                    if (!mCycleEnded)
                    {
                        // If room has been idle for a while
                        if (this.IdleTime >= 60)
                        {
                            mIsIdle = true;
                        }
                        //if (UserCount == 0)
                        //    UberEnvironment.GetGame().GetRoomManager().UnloadRoom(Id);
                        else
                        {
                            ServerMessage Updates = SerializeStatusUpdates(false);

                            if (Updates != null)
                                SendMessage(Updates);
                        }
                    }

                    #endregion

                    mRoomData.UsersNow = Usercount;
                }
                catch (ThreadAbortException) { }
                catch (Exception e)
                {
                    Logging.LogThreadException(e.ToString(), "Room cycle task");
                }

                //TimeSpan TimeUsed = DateTime.Now - TimeStart;
                //Thread.Sleep(500 - TimeUsed.Milliseconds);
                // }
            }
            catch (ThreadAbortException)
            { }
        }

        internal bool mIsIdle = false;
        private int mRoolerCycle;

        private void CycleRollers() // Credit goes to Thor for the roller code
        {
            if (mGotRollers)
            {
                if (mRoolerCycle >= mRollerSpeed || mRollerSpeed == 0)
                {
                    Hashtable ClonedItems = mFloorItems.Clone() as Hashtable;

                    List<uint> ItemsMoved = new List<uint>();
                    List<uint> UsersMoved = new List<uint>();

                    foreach (RoomItem Item in ClonedItems.Values)
                    {
                        if (Item.IsRoller)
                        {
                            Coord NextCoord = Item.SquareInFront;

                            List<RoomItem> ItemsOnRoller = GetRoomItemForSquare(Item.GetX, Item.GetY);
                            RoomUser UserOnRoller = GetUserForSquare(Item.GetX, Item.GetY);

                            if (ItemsOnRoller.Count > 0 || UserOnRoller != null)
                            {
                                List<RoomItem> ItemsOnNext = GetRoomItemForSquare(NextCoord.X, NextCoord.Y);
                                Double NextZ = 0;
                                int ItemCount = 0;
                                int RollesOnNext = 0;


                                foreach (RoomItem tItem in ItemsOnNext)
                                {
                                    if (tItem.TotalHeight > NextZ)
                                        NextZ = tItem.TotalHeight;
                                    if (!tItem.IsRoller)
                                        ItemCount++;
                                    else
                                        RollesOnNext++;
                                }

                                bool rItemsOnNext = (ItemCount > 0);
                                if (GetUserForSquare(NextCoord.X, NextCoord.Y) != null)
                                    rItemsOnNext = true;

                                bool RollersOnNext = (RollesOnNext > 0);

                                //MatrixState StateForNextSquare = MatrixState.BLOCKED;
                                //try
                                //{
                                //    StateForNextSquare = Matrix[NextCoord.X, NextCoord.Y];
                                //}
                                //catch { }


                                foreach (RoomItem tItem in ItemsOnRoller)
                                {
                                    if (!tItem.IsRoller && !ItemsMoved.Contains(tItem.Id) && CanRollItemHere(NextCoord.X, NextCoord.Y)
                                        && !(rItemsOnNext && RollersOnNext) && Item.GetZ < tItem.GetZ && GetUserForSquare(NextCoord.X, NextCoord.Y) == null)
                                    {
                                        Double sNextZ;
                                        if (RollersOnNext)
                                            sNextZ = tItem.GetZ;
                                        else
                                            sNextZ = tItem.GetZ - Item.TotalHeight;

                                        UpdateItemOnRoller(tItem, NextCoord, Item.Id, sNextZ);
                                        ItemsMoved.Add(tItem.Id);
                                    }
                                }

                                if (UserOnRoller != null && !(rItemsOnNext && RollersOnNext) && !rItemsOnNext && CanRollItemHere(NextCoord.X, NextCoord.Y))
                                {
                                    if (!UsersMoved.Contains(UserOnRoller.HabboId))
                                    {
                                        UpdateUserOnRoller(UserOnRoller, NextCoord, Item.Id, NextZ);
                                        UsersMoved.Add(UserOnRoller.HabboId);
                                    }
                                }
                            }
                        }
                    }

                    ClonedItems.Clear();
                    //ClonedItems = null;

                    ItemsMoved.Clear();
                    UsersMoved.Clear();

                    mRoolerCycle = 0;
                }
                else
                    mRoolerCycle++;
            }
        }

        private bool CanRollItemHere(int x, int y)
        {
            if (!ValidTile(x, y))
                return false;
            if (Model.SqState[x, y] == SquareState.BLOCKED)
                return false;

            return true;
        }

        //internal void CloseSqare(int x, int y)
        //{
        //    mGameMap[x, y] = 0;
        //}

        //internal void OpenSqare(int x, int y)
        //{
        //    mGameMap[x, y] = 0;
        //}

        internal bool SquareIsOpen(int x, int y, bool pOverride)
        {
            if (mModel.MapSizeX < x || mModel.MapSizeY < y)
                return false;
            return CanWalk(mGameMap[x, y], pOverride);
        }

        internal bool CanWalk(byte pState, bool pOverride)
        {

            if (!pOverride)
            {
                if (pState == 3)
                    return true;
                if (pState == 1)
                    return true;
            }
            return true;
        }

        private void UpdateItemOnRoller(RoomItem pItem, Coord NextCoord, uint pRolledID, Double NextZ)
        {
            ServerMessage mMessage = new ServerMessage();
            mMessage.Init(230); // Cf
            mMessage.AppendInt32(pItem.GetX);
            mMessage.AppendInt32(pItem.GetY);

            mMessage.AppendInt32(NextCoord.X);
            mMessage.AppendInt32(NextCoord.Y);

            mMessage.AppendInt32(1);

            mMessage.AppendUInt(pItem.Id);

            mMessage.AppendStringWithBreak(pItem.GetZ.ToString().Replace(',', '.'));
            mMessage.AppendStringWithBreak(NextZ.ToString().Replace(',', '.'));

            mMessage.AppendUInt(pRolledID);

            SendMessage(mMessage);

            //SetFloorItem(null, pItem, NextCoord.X, NextCoord.Y, pItem.Rot, false, true);
            SetFloorItem(pItem, NextCoord.X, NextCoord.Y, NextZ);
        }

        private void UpdateUserOnRoller(RoomUser pUser, Coord pNextCoord, uint pRollerID, Double NextZ)
        {
            ServerMessage mMessage = new ServerMessage();
            mMessage.Init(230); // Cf
            mMessage.AppendInt32(pUser.X);
            mMessage.AppendInt32(pUser.Y);

            mMessage.AppendInt32(pNextCoord.X);
            mMessage.AppendInt32(pNextCoord.Y);

            mMessage.AppendInt32(0);

            mMessage.AppendUInt(pRollerID);

            mMessage.AppendString("J");

            mMessage.AppendInt32(pUser.VirtualId);

            mMessage.AppendStringWithBreak(pUser.Z.ToString().Replace(',', '.'));
            //mMessage.AppendChar(2);
            mMessage.AppendStringWithBreak(NextZ.ToString().Replace(',', '.'));
            //mMessage.appendChar(2);

            //mRollerBroadcast.Append(mMessage.ToString());

            SendMessage(mMessage);


            //UserMatrix[pUser.X, pUser.Y] = false;

            mGameMap[pUser.X, pUser.Y] = 1;
            pUser.X = pNextCoord.X;
            pUser.Y = pNextCoord.Y;
            pUser.Z = NextZ;
            mGameMap[pUser.X, pUser.Y] = 0;

            //UserMatrix[pUser.X, pUser.Y] = true;

        }

        internal RoomUser GetUserForSquare(int x, int y)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;
                if (User.X == x && User.Y == y)
                    return User;
            }

            return null;
        }

        private List<RoomItem> GetRoomItemForSquare(int pX, int pY)
        {
            List<RoomItem> Items = new List<RoomItem>();

            foreach (RoomItem Item in FloorItems.Values)
            {
                if (Item.GetX == pX && Item.GetY == pY)
                    Items.Add(Item);
            }

            return Items;
        }

        //private bool SquareHasUsersOverride(int X, int Y)
        //{
        //    //Coord Coord = BedMatrix[X, Y];
        //    //return UserMatrix[Coord.X, Coord.Y];
        //    return mGameMap
        //}

        //private Boolean IsSquareOpen(int x, int y, RoomUser User)
        //{
        //    if (ValidTile(x, y) && User.AllowOverride)
        //    {
        //        return true;
        //    }

        //    if (User.X == x && User.Y == y)
        //    {
        //        return true;
        //    }

        //    bool isLastStep = false;

        //    if (User.GoalX == x && User.GoalY == y)
        //    {
        //        isLastStep = true;
        //    }

        //    if (!CanWalk(x, y, 0, isLastStep))
        //    {
        //        return false;
        //    }

        //    return true;
        //}

        #region User handling (General)
        internal void AddUserToRoom(GameClient Session, bool Spectator)
        {
            RoomUser User = new RoomUser(Session.GetHabbo().Id, RoomId, UserCounter++);

            if (User == null || User.GetClient() == null || User.GetClient().GetHabbo() == null)
                return;

            if (!CycleActive)
            {
                CycleActive = true;
                //mCycleThread = new Thread(new ThreadStart(ProcessRoom));
                //mCycleThread.Start();
            }

            if (Spectator)
            {
                User.IsSpectator = true;
            }
            else
            {
                User.SetPos(Model.DoorX, Model.DoorY, Model.DoorZ);
                User.SetRot(Model.DoorOrientation);

                if (CheckRights(Session, true))
                {
                    User.AddStatus("flatcrtl", "useradmin");
                }
                else if (CheckRights(Session))
                {
                    User.AddStatus("flatcrtl", "");
                }

                if (!User.IsBot && User.GetClient().GetHabbo().IsTeleporting)
                {
                    RoomItem Item = GetItem(User.GetClient().GetHabbo().TeleporterId);

                    if (Item != null)
                    {
                        User.SetPos(Item.GetX, Item.GetY, Item.GetZ);
                        User.SetRot(Item.Rot);

                        Item.InteractingUser2 = Session.GetHabbo().Id;
                        Item.ExtraData = "2";
                        Item.UpdateState(false, true);
                    }
                }

                User.GetClient().GetHabbo().IsTeleporting = false;
                User.GetClient().GetHabbo().TeleporterId = 0;

                ServerMessage EnterMessage = new ServerMessage(28);
                EnterMessage.AppendInt32(1);
                User.Serialize(EnterMessage);
                SendMessage(EnterMessage);
            }

            //UserList.Add(User);
            int PersonalID = GetFreeID();
            User.InternalRoomID = PersonalID;
            UserList[PersonalID] = User;


            //using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.ExecuteQuery("INSERT INTO user_roomvisits (user_id,room_id,entry_timestamp,exit_timestamp,hour,minute) VALUES ('" + Id + "','" + RoomId + "','" + ButterflyEnvironment.GetUnixTimestamp() + "','0','" + DateTime.Now.Hour + "','" + DateTime.Now.Minute + "')");
            //    if (!Spectator)
            //        dbClient.ExecuteQuery("UPDATE rooms SET users_now = '" + UsersNow + "' WHERE id = '" + Id + "' LIMIT 1");
            //}

            Session.GetHabbo().CurrentRoomId = Id;
            if (Session.GetHabbo().GetMessenger() != null)
                Session.GetHabbo().GetMessenger().OnStatusChanged(false);
            

            if (!Spectator)
            {
                UpdateUserCountNoDB();

                for (int i = 0; i < UserList.Length; i++)
                {
                    RoomUser RoomUser = UserList[i];
                    if (RoomUser == null)
                        continue;

                    if (!RoomUser.IsBot)
                        continue;

                    RoomUser.BotAI.OnUserEnterRoom(User);
                }
            }
        }

        internal void RemoveUserFromRoom(GameClient Session, Boolean NotifyClient, Boolean NotifyKick)
        {
            if (mDisposed)
            {
                if (NotifyClient)
                {
                    if (NotifyKick)
                    {
                        Session.GetMessageHandler().GetResponse().Init(33);
                        Session.GetMessageHandler().GetResponse().AppendInt32(4008);
                        Session.GetMessageHandler().SendResponse();
                    }

                    Session.GetMessageHandler().GetResponse().Init(18);
                    Session.GetMessageHandler().SendResponse();
                }
            }
            else
            {
                try
                {
                    if (Session == null)
                    {
                        return;
                    }

                    if (Session.GetHabbo() == null)
                        return;

                    RoomUser User = GetRoomUserByHabbo(Session.GetHabbo().Id);

                    if (User != null)
                    {
                        UserList[User.InternalRoomID] = null;

                        User.InternalRoomID = -1;
                        mGameMap[User.X, User.Y] = User.SqState;
                    }

                    if (NotifyClient)
                    {
                        if (NotifyKick)
                        {
                            Session.GetMessageHandler().GetResponse().Init(33);
                            Session.GetMessageHandler().GetResponse().AppendInt32(4008);
                            Session.GetMessageHandler().SendResponse();
                        }

                        Session.GetMessageHandler().GetResponse().Init(18);
                        Session.GetMessageHandler().SendResponse();
                    }

                    List<RoomUser> PetsToRemove = new List<RoomUser>();

                    if (User != null)
                    {
                        if (Session.GetHabbo() != null)
                        {
                            if (!User.IsSpectator)
                            {
                                if (User.CurrentSwimType != SwimType.Walk)
                                {
                                    User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().CurrentEffect = -1;
                                }
                                //UserMatrix[User.X, User.Y] = false;

                                mGameMap[User.X, User.Y] = User.SqState;
                                ServerMessage LeaveMessage = new ServerMessage(29);
                                LeaveMessage.AppendRawInt32(User.VirtualId);
                                SendMessage(LeaveMessage);

                                if (Session.GetHabbo() != null)
                                {
                                    if (HasActiveTrade(Session.GetHabbo().Id))
                                        TryStopTrade(Session.GetHabbo().Id);

                                    if (Session.GetHabbo().Username == Owner)
                                    {
                                        if (HasOngoingEvent)
                                        {
                                            Event = new RoomEvent(0, null, null, 0, new List<string>());

                                            ServerMessage Message = new ServerMessage(370);
                                            Message.AppendStringWithBreak("-1");
                                            SendMessage(Message);
                                        }
                                    }
                                    Session.GetHabbo().CurrentRoomId = 0;

                                    if (Session.GetHabbo().GetMessenger() != null)
                                        Session.GetHabbo().GetMessenger().OnStatusChanged(false);
                                }

                                //DateTime Start = DateTime.Now;
                                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                                {
                                    //TimeSpan TimeUsed1 = DateTime.Now - Start;
                                    //Logging.LogThreadException("Time used on sys part 2: " + TimeUsed1.Seconds + "s, " + TimeUsed1.Milliseconds + "ms", "");

                                    //if (Session.GetHabbo() != null)
                                    //    dbClient.ExecuteQuery("UPDATE user_roomvisits SET exit_timestamp = '" + ButterflyEnvironment.GetUnixTimestamp() + "' WHERE room_id = '" + this.Id + "' AND user_id = '" + Id + "' ORDER BY exit_timestamp DESC LIMIT 1");
                                    dbClient.ExecuteQuery("UPDATE rooms SET users_now = '" + this.UsersNow + "' WHERE id = '" + Id + "'");
                                }

                                UpdateUserCountNoDB();

                                List<RoomUser> Bots = new List<RoomUser>();

                                for (int i = 0; i < UserList.Length; i++)
                                {
                                    RoomUser Usr = UserList[i];
                                    if (Usr == null)
                                        continue;
                                    if (Usr.IsBot)
                                        Bots.Add(Usr);
                                }

                                foreach (RoomUser Bot in Bots)
                                {
                                    Bot.BotAI.OnUserLeaveRoom(Session);

                                    if (Bot.IsPet && Bot.PetData.OwnerId == Session.GetHabbo().Id && !CheckRights(Session, true))
                                    {
                                        PetsToRemove.Add(Bot);
                                    }
                                }
                            }
                        }
                    }

                    foreach (RoomUser toRemove in PetsToRemove)
                    {
                        Session.GetHabbo().GetInventoryComponent().AddPet(toRemove.PetData);
                        RemoveBot(toRemove.VirtualId, false);
                    }

                }
                catch (Exception e) { Logging.LogCriticalException("Error during removing user from room:" + e.ToString()); }
            }
        }

        internal RoomUser GetPet(uint PetId)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User.IsBot && User.IsPet && User.PetData != null && User.PetData.PetId == PetId)
                    return User;
            }

            return null;
        }

        //internal bool RoomContainsPet(uint PetId)
        //{
        //    return (GetPet(PetId) != null);
        //}

        internal void UpdateUserCount()
        {
            this.UsersNow = UserCount;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE rooms SET users_now = '" + this.UsersNow + "' WHERE id = '" + Id + "' LIMIT 1");
            }
        }

        internal void UpdateUserCountNoDB()
        {
            this.UsersNow = UserCount;


        }

        internal RoomUser GetRoomUserByVirtualId(int VirtualId)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;

                if (User.VirtualId == VirtualId)
                {
                    return User;
                }

            }

            return null;
        }

        internal RoomUser GetRoomUserByHabbo(uint pId)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;

                if (User.IsBot)
                {
                    continue;
                }

                if (User.HabboId == pId)
                {
                    return User;
                }
            }

            return null;
        }

        internal RoomUser GetRoomUserByHabbo(string pName)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;
                if (User.IsBot || User.GetClient().GetHabbo() == null)
                    continue;

                if (User.GetClient().GetHabbo().Username.ToLower() == pName.ToLower())
                    return User;
            }

            return null;
        }
        #endregion

        #region Communication
        internal void SendMessage(ServerMessage Message)
        {
            try
            {
                if (UserList == null)
                    return;
                byte[] PacketData = Message.GetBytes();

                for (int i = 0; i < UserList.Length; i++)
                {
                    RoomUser User = UserList[i];
                    if (User == null)
                        continue;

                    if (User.IsBot)
                        continue;

                    GameClient UsersClient = User.GetClient();
                    if (UsersClient == null)
                        continue;

                    try
                    {
                        UsersClient.GetConnection().SendData(PacketData);
                    }
                    catch { }
                    
                    //User.GetClient().SendMessage(Message);
                }
            }
            catch (InvalidOperationException) { }
        }

        //internal void SendMessageToUsersWithRank(ServerMessage Message, int MinRank)
        //{
        //    try
        //    {
        //        byte[] PacketData = Message.GetBytes();

        //        for (int i = 0; i < UserList.Length; i++)
        //        {
        //            RoomUser User = UserList[i];
        //            if (User == null)
        //                continue;

        //            if (User.IsBot)
        //                continue;

        //            GameClient UsersClient = User.GetClient();
        //            if (UsersClient == null)
        //                continue;
        //            if (UsersClient.GetHabbo() == null)
        //                continue;
        //            if (UsersClient.GetHabbo().Rank < MinRank)
        //                continue;
        //            try
        //            {
        //                UsersClient.GetConnection().SendData(PacketData);
        //            }
        //            catch { }
        //        }
        //    }
        //    catch (InvalidOperationException) { }
        //}

        internal void SendMessageToUsersWithRights(ServerMessage Message)
        {
            try
            {
                byte[] PacketData = Message.GetBytes();

                for (int i = 0; i < UserList.Length; i++)
                {
                    RoomUser User = UserList[i];
                    if (User == null)
                        continue;

                    if (User.IsBot)
                        continue;

                    GameClient UsersClient = User.GetClient();
                    if (UsersClient == null)
                        continue;

                    if (!CheckRights(UsersClient))
                        continue;

                    try
                    {
                        UsersClient.GetConnection().SendData(PacketData);
                    }
                    catch { }
                    //User.GetClient().SendMessage(Message);
                }
            }
            catch (InvalidOperationException) { }
        }
        #endregion
        internal void Destroy()
        {
            // Send kick message. No users should be left in this room, but if they are for whatever reason, they should leave.
            SendMessage(new ServerMessage(18));

            Dispose();
            // Clear user list and dispose of the engine thread
            //this.IdleTime = 0;
            //this.KeepAlive = false;
            ////UserList.Clear();
            //Array.Clear(UserList, 0, UserList.Length);

            //CycleActive = false;
            //if (mCycleThread != null)
            //    if (mCycleThread.IsCompleted)
            //        mCycleThread.Dispose();
            //mCycleThread = null;
        }

        private bool mDisposed;

        internal byte[,] GetGameMap
        {
            get
            {
                return mGameMap;
            }
        }
        #region IDisposable members
        internal void Dispose()
        {
            Dispose(true);
            //GC.SuppressFinalize(this);
        }

        //internal bool GotPets
        //{
        //    get
        //    {
        //        return GetPetList().Count > 0;
        //    }
        //}

        //internal void SaveFurniture()
        //{
        //    try
        //    {
        //        if (mAddedItems.Count > 0 || mRemovedItems.Count > 0 || mMovedItems.Count > 0)
        //        {
        //            StringBuilder Queries = new StringBuilder();
        //            using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //            {
        //                foreach (RoomItem Item in mRemovedItems.Values)
        //                {
        //                    Queries.Append("DELETE FROM room_items WHERE id = '" + Item.Id + "' AND room_id = '" + RoomId + "'; ");
        //                }
        //                mRemovedItems.Clear();

        //                if (mAddedItems.Count > 0)
        //                {
        //                    int WallitemCount = 0;
        //                    int FlooritemCount = 0;
        //                    foreach (RoomItem Item in mAddedItems.Values)
        //                    {
        //                        if (Item.IsFloorItem)
        //                            FlooritemCount++;
        //                        else
        //                            WallitemCount++;
        //                    }

        //                    foreach (RoomItem Item in mAddedItems.Values)
        //                    {
        //                        Queries.Append("DELETE FROM room_items WHERE id = '" + Item.Id + "'; ");
        //                    }

        //                    if (FlooritemCount > 0)
        //                    {
        //                        Queries.Append("INSERT INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ");
        //                        foreach (RoomItem Item in mAddedItems.Values)
        //                        {
        //                            if (Item.IsFloorItem)
        //                            {
        //                                dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
        //                                Queries.Append("('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data" + Item.Id + ",'" + Item.GetX + "','" + Item.GetY + "','" + Item.GetZ + "','" + Item.Rot + "',''),");
        //                            }
        //                        }

        //                        Queries.Remove(Queries.Length - 1, 1);
        //                        Queries.Append("; ");
        //                    }

        //                    if (WallitemCount > 0)
        //                    {
        //                        Queries.Append("INSERT INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ");
        //                        foreach (RoomItem Item in mAddedItems.Values)
        //                        {
        //                            if (Item.IsWallItem)
        //                            {
        //                                dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
        //                                dbClient.AddParamWithValue("pos" + Item.Id, Item.WallPos);
        //                                Queries.Append("('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data" + Item.Id + ",'0','0','0','0',@pos" + Item.Id + "),");
        //                            }
        //                        }

        //                        Queries.Remove(Queries.Length - 1, 1);
        //                        Queries.Append("; ");
        //                    }

        //                    mAddedItems.Clear();
        //                }

        //                foreach (RoomItem Item in mMovedItems.Values)
        //                {
        //                    Queries.Append("UPDATE room_items SET x = '" + Item.GetX + "', y = '" + Item.GetY + "', z = '" + Item.GetZ + "', rot = '" + Item.Rot + "', wall_pos = '' WHERE id = '" + Item.Id + "'; ");
        //                }
        //                mMovedItems.Clear();

        //                if (Queries.Length > 0)
        //                    dbClient.ExecuteQuery(Queries.ToString());
        //            }
        //        }
        //    }
        //    catch (Exception e)
        //    {
        //        Logging.LogCriticalException("Error during saving furniture for room " + RoomId + ". Stack: " + e.ToString());
        //    }
        //}

        internal void SaveFurniture(DatabaseClient dbClient)
        {
            //DateTime Start = DateTime.Now;
            try
            {
                if (mAddedItems.Count > 0 || mRemovedItems.Count > 0 || mMovedItems.Count > 0)
                {
                    StringBuilder Queries = new StringBuilder();
                    foreach (RoomItem Item in mRemovedItems.Values)
                    {
                        Queries.Append("DELETE FROM room_items WHERE id = '" + Item.Id + "' AND room_id = '" + RoomId + "'; ");
                    }
                    mRemovedItems.Clear();

                    if (mAddedItems.Count > 0)
                    {
                        //foreach (RoomItem Item in mAddedItems.Values)
                        //{
                        //    Queries.Append("DELETE FROM room_items WHERE id = '" + Item.Id + "'; ");
                        //}

                        int WallitemCount = 0;
                        int FlooritemCount = 0;
                        foreach (RoomItem Item in mAddedItems.Values)
                        {
                            if (Item.IsFloorItem)
                                FlooritemCount++;
                            else
                                WallitemCount++;
                        }

                        if (FlooritemCount > 0)
                        {
                            Queries.Append("REPLACE INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ");
                            foreach (RoomItem Item in mAddedItems.Values)
                            {
                                if (Item.IsFloorItem)
                                {
                                    dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
                                    Queries.Append("('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data" + Item.Id + ",'" + Item.GetX + "','" + Item.GetY + "','" + Item.GetZ + "','" + Item.Rot + "',''),");
                                }
                            }

                            Queries.Remove(Queries.Length - 1, 1);
                            Queries.Append("; ");
                        }

                        if (WallitemCount > 0)
                        {
                            Queries.Append("REPLACE INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ");
                            foreach (RoomItem Item in mAddedItems.Values)
                            {
                                if (Item.IsWallItem)
                                {
                                    dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
                                    dbClient.AddParamWithValue("pos" + Item.Id, Item.WallPos);
                                    Queries.Append("('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data" + +Item.Id + ",'0','0','0','0',@pos" + Item.Id + "),");
                                }
                            }

                            Queries.Remove(Queries.Length - 1, 1);
                            Queries.Append("; ");
                        }

                        mAddedItems.Clear();
                    }

                    foreach (RoomItem Item in mMovedItems.Values)
                    {
                        Queries.Append("UPDATE room_items SET x = '" + Item.GetX + "', y = '" + Item.GetY + "', z = '" + Item.GetZ + "', rot = '" + Item.Rot + "', wall_pos = '' WHERE id = '" + Item.Id + "'; ");
                    }
                    mMovedItems.Clear();

                    if (Queries.Length > 0)
                        dbClient.ExecuteQuery(Queries.ToString());
                }
            }
            catch (Exception e)
            {
                Logging.LogCriticalException("Error during saving furniture for room " + RoomId + ". Stack: " + e.ToString());
            }

            //TimeSpan TimeUsed = DateTime.Now - Start;
            //Logging.LogMessage("Time it took to save room furniture: " + TimeUsed.Seconds + " s, " + TimeUsed.Milliseconds + " ms");
        }
        private void Dispose(bool Disposing)
        {
            if (!this.mDisposed)
            {
                mDisposed = true;
                if (Disposing)
                {
                    mGotRollers = false;
                    if (mCycleThread != null)
                    {
                        mCycleEnded = true;
                        //iveThreads--;
                        try
                        {
                            if (mCycleThread.IsAlive)
                                mCycleThread.Abort();
                        }
                        catch { }
                        //try
                        //{
                        //    mCycleThread.
                        //}
                        //catch (Exception e) { }
                    }

                    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                    {
                        SaveFurniture(dbClient);
                    }

                    //mCycleThread = null;
                    //KeepAlive = false;

                    if (Tags != null)
                        Tags.Clear();
                    //Tags = null;

                    if (UserList != null)
                        Array.Clear(UserList, 0, UserList.Length);
                    //UserList = null;

                    //myIcon = null;
                    if (UsersWithRights != null)
                        UsersWithRights.Clear();
                    //myIcon = null;

                    if (Bans != null)
                        Bans.Clear();
                    //Bans = null;

                    //Wallpaper = null;
                    //Floor = null;
                    //Landscape = null;

                    if (mFloorItems != null)
                        mFloorItems.Clear();
                    //mFloorItems = null;

                    if (mWallItems != null)
                        mWallItems.Clear();
                    //mWallItems = null;

                    //MoodlightData = null;

                    if (ActiveTrades != null)
                        ActiveTrades.Clear();
                    //ActiveTrades = null;

                    //if (Matrix != null)
                    //    Array.Clear(Matrix, 0, Matrix.Length);
                    //Matrix = null;

                    //if (BedMatrix != null)
                    //    Array.Clear(BedMatrix, 0, BedMatrix.Length);
                    //BedMatrix = null;

                    //if (HeightMatrix != null)
                    //    Array.Clear(HeightMatrix, 0, HeightMatrix.Length);
                    //HeightMatrix = null;
                }
            }
        }
        #endregion

        internal ServerMessage SerializeStatusUpdates(Boolean All)
        {
            List<RoomUser> Users = new List<RoomUser>();

            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;

                if (!All)
                {
                    if (!User.UpdateNeeded)
                    {
                        continue;
                    }

                    User.UpdateNeeded = false;
                }

                Users.Add(User);
            }

            if (Users.Count == 0)
            {
                return null;
            }

            ServerMessage Message = new ServerMessage(34);
            Message.AppendInt32(Users.Count);

            foreach (RoomUser User in Users)
            {
                User.SerializeStatus(Message);
            }

            return Message;
        }

        #region Room Bans
        internal Boolean UserIsBanned(uint pId)
        {
            return Bans.ContainsKey(pId);
        }

        internal void RemoveBan(uint pId)
        {
            Bans.Remove(pId);
        }

        internal void AddBan(uint pId)
        {
            Bans.Add(pId, ButterflyEnvironment.GetUnixTimestamp());
        }

        internal Boolean HasBanExpired(uint pId)
        {
            if (!UserIsBanned(pId))
            {
                return true;
            }

            Double diff = ButterflyEnvironment.GetUnixTimestamp() - Bans[pId];

            if (diff > 900)
            {
                return true;
            }

            return false;
        }
        #endregion

        internal int ItemCountByType(String InteractionType)
        {
            int i = 0;

            foreach (RoomItem Item in WallItems.Values)
            {
                if (Item.GetBaseItem().InteractionType == InterractionTypes.GetTypeFromString(InteractionType))
                    i++;
            }

            foreach (RoomItem Item in FloorItems.Values)
            {
                if (Item.GetBaseItem().InteractionType == InterractionTypes.GetTypeFromString(InteractionType))
                    i++;
            }

            return i;
        }

        #region Trading
        internal bool HasActiveTrade(RoomUser User)
        {
            if (User.IsBot)
            {
                return false;
            }

            return HasActiveTrade(User.GetClient().GetHabbo().Id);
        }

        internal bool HasActiveTrade(uint UserId)
        {
            using (TimedLock.Lock(ActiveTrades))
            {
                foreach (Trade Trade in ActiveTrades)
                {
                    if (Trade.ContainsUser(UserId))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        //internal Trade GetUserTrade(RoomUser User)
        //{
        //    if (User.IsBot)
        //    {
        //        return null;
        //    }

        //    return GetUserTrade(User.GetClient().GetHabbo().Id);
        //}

        internal Trade GetUserTrade(uint UserId)
        {
            using (TimedLock.Lock(ActiveTrades))
            {
                foreach (Trade Trade in ActiveTrades)
                {
                    if (Trade.ContainsUser(UserId))
                    {
                        return Trade;
                    }
                }
            }

            return null;
        }

        internal void TryStartTrade(RoomUser UserOne, RoomUser UserTwo)
        {
            if (UserOne == null || UserTwo == null || UserOne.IsBot || UserTwo.IsBot || UserOne.IsTrading || UserTwo.IsTrading || HasActiveTrade(UserOne) || HasActiveTrade(UserTwo))
            {
                return;
            }

            ActiveTrades.Add(new Trade(UserOne.GetClient().GetHabbo().Id, UserTwo.GetClient().GetHabbo().Id, RoomId));
        }

        internal void TryStopTrade(uint UserId)
        {
            Trade Trade = GetUserTrade(UserId);

            if (Trade == null)
            {
                return;
            }

            Trade.CloseTrade(UserId);
            ActiveTrades.Remove(Trade);
        }
        #endregion

        #region Furni handling and stacking
        internal bool SetFloorItem(GameClient Session, RoomItem Item, int newX, int newY, int newRot, bool newItem, bool OnRoller)
        {
            // Find affected tiles
            Dictionary<int, AffectedTile> AffectedTiles = GetAffectedTiles(Item.GetBaseItem().Length, Item.GetBaseItem().Width, newX, newY, newRot);

            // Verify tiles are valid
                if (!ValidTile(newX, newY))
                {
                    return false;
                }

                foreach (AffectedTile Tile in AffectedTiles.Values)
                {
                    if (!ValidTile(Tile.X, Tile.Y))
                    {
                        return false;
                    }
                }

            // Start calculating new Z coordinate
            Double newZ = Model.SqFloorHeight[newX, newY];

            if (!OnRoller)
            {
                // Is the item trying to stack on itself!?
                if (Item.Rot == newRot && Item.GetX == newX && Item.GetY == newY && Item.GetZ != newZ)
                {
                    return false;
                }

                // Make sure this tile is open and there are no users here
                if (Model.SqState[newX, newY] != SquareState.OPEN)
                {
                    return false;
                }

                foreach (AffectedTile Tile in AffectedTiles.Values)
                {
                    if (Model.SqState[Tile.X, Tile.Y] != SquareState.OPEN)
                    {
                        return false;
                    }
                }

                // And that we have no users
                if (!Item.GetBaseItem().IsSeat && !Item.IsRoller)
                {
                    if (SquareHasUsers(newX, newY))
                    {
                        return false;
                    }

                    foreach (AffectedTile Tile in AffectedTiles.Values)
                    {
                        if (SquareHasUsers(Tile.X, Tile.Y))
                        {
                            return false;
                        }
                    }
                }
            }

            // Find affected objects
            List<RoomItem> ItemsOnTile = GetFurniObjects(newX, newY);
            List<RoomItem> ItemsAffected = new List<RoomItem>();
            List<RoomItem> ItemsComplete = new List<RoomItem>();

            foreach (AffectedTile Tile in AffectedTiles.Values)
            {
                List<RoomItem> Temp = GetFurniObjects(Tile.X, Tile.Y);

                if (Temp != null)
                {
                    ItemsAffected.AddRange(Temp);
                }
            }

            if (ItemsOnTile == null) ItemsOnTile = new List<RoomItem>();

            ItemsComplete.AddRange(ItemsOnTile);
            ItemsComplete.AddRange(ItemsAffected);

            if (!OnRoller)
            {
                // Check for items in the stack that do not allow stacking on top of them
                foreach (RoomItem I in ItemsComplete)
                {
                    if (I == null)
                        continue;

                    if (I.Id == Item.Id)
                    {
                        continue;
                    }

                    if (I.GetBaseItem() == null)
                        continue;

                    if (!I.GetBaseItem().Stackable)
                    {
                        return false;
                    }
                }
            }

            if (!Item.IsRoller)
            {
                // If this is a rotating action, maintain item at current height
                if (Item.Rot != newRot && Item.GetX == newX && Item.GetY == newY)
                {
                    newZ = Item.GetZ;
                }

                // Are there any higher objects in the stack!?
                foreach (RoomItem I in ItemsComplete)
                {
                    if (I.Id == Item.Id)
                    {
                        continue; // cannot stack on self
                    }

                    if (I.TotalHeight > newZ)
                    {
                        newZ = I.TotalHeight;
                    }
                }
            }

            // Verify the rotation is correct
            if (newRot != 0 && newRot != 2 && newRot != 4 && newRot != 6 && newRot != 8)
            {
                newRot = 0;
            }

            //Item.GetX = newX;
            //Item.GetY = newY;
            //Item.GetZ = newZ;
            Item.SetState(newX, newY, newZ);

            Item.Rot = newRot;

            if (!OnRoller)
                Item.Interactor.OnPlace(Session, Item);
            

            if (newItem)
            {
                //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                //{
                //    dbClient.AddParamWithValue("extra_data", Item.ExtraData);
                //    dbClient.ExecuteQuery("INSERT INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data,'" + Item.GetX + "','" + Item.GetY + "','" + Item.GetZ + "','" + Item.Rot + "','')");
                //}
                if (mRemovedItems.Contains(Item.Id))
                    mRemovedItems.Remove(Item.Id);
                if (mAddedItems.Contains(Item.Id))
                    return false;

                mAddedItems.Add(Item.Id, Item);
                
                if (Item.IsFloorItem)
                {
                    if (mFloorItems.Contains(Item.Id))
                        mFloorItems.Remove(Item.Id);
                    mFloorItems.Add(Item.Id, Item);
                }
                else
                {
                    if (mWallItems.Contains(Item.Id))
                        mWallItems.Remove(Item.Id);
                    mWallItems.Add(Item.Id, Item);
                }

                ServerMessage Message = new ServerMessage(93);
                Item.Serialize(Message);
                SendMessage(Message);
            }
            else
            {
                //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                //{
                //    dbClient.ExecuteQuery("UPDATE room_items SET x = '" + Item.GetX + "', y = '" + Item.GetY + "', z = '" + Item.GetZ + "', rot = '" + Item.Rot + "', wall_pos = '' WHERE id = '" + Item.Id + "' LIMIT 1");
                //}
                if (!mMovedItems.Contains(Item.Id) && !mAddedItems.ContainsKey(Item.Id))
                {
                    mMovedItems.Add(Item.Id, Item);
                }

                if (!OnRoller)
                {
                    ServerMessage Message = new ServerMessage(95);
                    Item.Serialize(Message);
                    SendMessage(Message);
                }
            }

            GenerateMaps();
            UpdateUserStatusses();

            return true;
        }

        internal void AddChangedItem(RoomItem Item)
        {
            if (!mMovedItems.Contains(Item.Id) && !mAddedItems.ContainsKey(Item.Id))
            {
                mMovedItems.Add(Item.Id, Item);
            }
        }


        internal bool SetFloorItem(RoomItem Item, int newX, int newY, Double newZ)
        {
            //Item.GetX = newX;
            //Item.GetY = newY;
            //Item.GetZ = newZ;
            Item.SetState(newX, newY, newZ);

            if (!mMovedItems.Contains(Item.Id))
                mMovedItems.Add(Item.Id, Item);

            GenerateMaps();
            UpdateUserStatusses();

            return true;
        }

        internal bool SetWallItem(GameClient Session, RoomItem Item)
        {
            Item.Interactor.OnPlace(Session, Item);

            if (Item.GetBaseItem().InteractionType == InteractionType.dimmer)
            {
                if (MoodlightData == null)
                {
                    MoodlightData = new MoodlightData(Item.Id);
                    Item.ExtraData = MoodlightData.GenerateExtraData();
                }
            }

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.AddParamWithValue("extra_data", Item.ExtraData);
            //    dbClient.AddParamWithValue("pos", Item.WallPos);
            //    dbClient.ExecuteQuery("INSERT INTO room_items (id,room_id,base_item,extra_data,x,y,z,rot,wall_pos) VALUES ('" + Item.Id + "','" + RoomId + "','" + Item.BaseItem + "',@extra_data,'0','0','0','0',@pos)");
            //}

            mAddedItems.Add(Item.Id, Item);

            if (Item.IsFloorItem)
                mFloorItems.Add(Item.Id, Item);
            else if (!mWallItems.Contains(Item.Id))
                mWallItems.Add(Item.Id, Item);

            ServerMessage Message = new ServerMessage(83);
            Item.Serialize(Message);
            SendMessage(Message);

            return true;
        }
        #endregion

        internal void UpdateUserStatusses()
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;
                UpdateUserStatus(User);
            }
        }

        private double SqAbsoluteHeight(int X, int Y, List<RoomItem> ItemsOnSquare)
        {
            try
            {
                //List<RoomItem> ItemsOnSquare = GetFurniObjects(X, Y);
                double HighestStack = 0;

                bool deduct = false;
                double deductable = 0.0;


                foreach (RoomItem Item in ItemsOnSquare)
                {
                    if (Item.TotalHeight > HighestStack)
                    {
                        if (Item.GetBaseItem().IsSeat || Item.GetBaseItem().InteractionType == InteractionType.bed)
                        {
                            deduct = true;
                            deductable = Item.GetBaseItem().Height;
                        }
                        else
                        {
                            deduct = false;
                        }

                        HighestStack = Item.TotalHeight;
                    }
                }

                double floorHeight = Model.SqFloorHeight[X, Y];
                double stackHeight = HighestStack - Model.SqFloorHeight[X, Y];

                if (deduct)
                {
                    stackHeight -= deductable;
                }

                if (stackHeight < 0)
                {
                    stackHeight = 0;
                }

                return (floorHeight + stackHeight);
            }
            catch { return 0; }
        }

        internal void UpdateUserStatus(RoomUser User)
        {
            if (User.Statusses.ContainsKey("lay") || User.Statusses.ContainsKey("sit"))
            {
                User.Statusses.Remove("lay");
                User.Statusses.Remove("sit");
                User.UpdateNeeded = true;
            }
            List<RoomItem> ItemsOnSquare = GetFurniObjects(User.X, User.Y);

            double newZ = SqAbsoluteHeight(User.X, User.Y, ItemsOnSquare);

            if (newZ != User.Z)
            {
                User.Z = newZ;
                User.UpdateNeeded = true;
            }

            if (Model.SqState[User.X, User.Y] == SquareState.SEAT)
            {
                if (!User.Statusses.ContainsKey("sit"))
                {
                    User.Statusses.Add("sit", "1.0");
                }

                User.Z = Model.SqFloorHeight[User.X, User.Y];
                User.RotHead = Model.SqSeatRot[User.X, User.Y];
                User.RotBody = Model.SqSeatRot[User.X, User.Y];

                User.UpdateNeeded = true;
            }

            foreach (RoomItem Item in ItemsOnSquare)
            {
                if (Item.GetBaseItem().IsSeat)
                {
                    if (!User.Statusses.ContainsKey("sit"))
                    {
                        User.Statusses.Add("sit", Item.GetBaseItem().Height.ToString().Replace(',', '.'));
                    }

                    User.Z = Item.GetZ;
                    User.RotHead = Item.Rot;
                    User.RotBody = Item.Rot;

                    User.UpdateNeeded = true;
                }

                if (Item.GetBaseItem().InteractionType == InteractionType.bed)
                {
                    if (!User.Statusses.ContainsKey("lay"))
                    {
                        User.Statusses.Add("lay", Item.GetBaseItem().Height.ToString().Replace(',', '.') + " null");
                    }

                    User.Z = Item.GetZ;
                    User.RotHead = Item.Rot;
                    User.RotBody = Item.Rot;

                    User.UpdateNeeded = true;
                }
            }
        }

        internal bool ValidTile(int X, int Y)
        {
            if (X < 0 || Y < 0 || X >= Model.MapSizeX || Y >= Model.MapSizeY)
            {
                return false;
            }

            return true;
        }

        internal void TurnHeads(int X, int Y, uint SenderId)
        {
            for (int i = 0; i < UserList.Length; i++)
            {
                RoomUser User = UserList[i];
                if (User == null)
                    continue;

                if (User.HabboId == SenderId)
                {
                    continue;
                }

                User.SetRot(Rotation.Calculate(User.X, User.Y, X, Y), true);
            }
        }

        internal List<RoomItem> GetFurniObjects(int X, int Y)
        {
            List<RoomItem> Results = new List<RoomItem>();

            foreach (RoomItem Item in FloorItems.Values)
            {
                if (Item.GetX == X && Item.GetY == Y)
                {
                    Results.Add(Item);
                }

                Dictionary<int, AffectedTile> PointList = GetAffectedTiles(Item.GetBaseItem().Length, Item.GetBaseItem().Width, Item.GetX, Item.GetY, Item.Rot);

                foreach (AffectedTile Tile in PointList.Values)
                {
                    if (Tile.X == X && Tile.Y == Y)
                    {
                        Results.Add(Item);
                    }
                }
            }
            return Results;
        }

        //internal RoomItem FindItem(uint Id)
        //{
        //    using (TimedLock.Lock(Items)
        //    {
        //        foreach (RoomItem Item in Items)
        //        {
        //            if (Item.Id == Id)
        //            {
        //                return Item;
        //            }
        //        }
        //    }

        //    return null;
        //}

        internal static Dictionary<int, AffectedTile> GetAffectedTiles(int Length, int Width, int PosX, int PosY, int Rotation)
        {
            int x = 0;

            Dictionary<int, AffectedTile> PointList = new Dictionary<int, AffectedTile>();

            if (Length > 1)
            {
                if (Rotation == 0 || Rotation == 4)
                {
                    for (int i = 1; i < Length; i++)
                    {
                        PointList.Add(x++, new AffectedTile(PosX, PosY + i, i));

                        for (int j = 1; j < Width; j++)
                        {
                            PointList.Add(x++, new AffectedTile(PosX + j, PosY + i, (i < j) ? j : i));
                        }
                    }
                }
                else if (Rotation == 2 || Rotation == 6)
                {
                    for (int i = 1; i < Length; i++)
                    {
                        PointList.Add(x++, new AffectedTile(PosX + i, PosY, i));

                        for (int j = 1; j < Width; j++)
                        {
                            PointList.Add(x++, new AffectedTile(PosX + i, PosY + j, (i < j) ? j : i));
                        }
                    }
                }
            }

            if (Width > 1)
            {
                if (Rotation == 0 || Rotation == 4)
                {
                    for (int i = 1; i < Width; i++)
                    {
                        PointList.Add(x++, new AffectedTile(PosX + i, PosY, i));

                        for (int j = 1; j < Length; j++)
                        {
                            PointList.Add(x++, new AffectedTile(PosX + i, PosY + j, (i < j) ? j : i));
                        }
                    }
                }
                else if (Rotation == 2 || Rotation == 6)
                {
                    for (int i = 1; i < Width; i++)
                    {
                        PointList.Add(x++, new AffectedTile(PosX, PosY + i, i));

                        for (int j = 1; j < Length; j++)
                        {
                            PointList.Add(x++, new AffectedTile(PosX + j, PosY + i, (i < j) ? j : i));
                        }
                    }
                }
            }

            return PointList;
        }

        //internal bool SquareHasUsers(int X, int Y, bool LastStep)
        //{
        //    if (AllowWalkthrough)
        //        return false;

        //    return SquareHasUsers(X, Y);
        //}

        internal bool SquareHasUsers(int X, int Y)
        {
            //Coord Coord = BedMatrix[X, Y];
            //return UserMatrix[Coord.X, Coord.Y];
            return (GetUserForSquare(X, Y) != null);
        }

        //This function is based on the one from "Holograph Emulator" (Wich sucks ASS!)
        internal static string WallPositionCheck(string wallPosition)
        {
            //:w=3,2 l=9,63 l
            try
            {
                if (wallPosition.Contains(Convert.ToChar(13)))
                { return null; }
                if (wallPosition.Contains(Convert.ToChar(9)))
                { return null; }

                string[] posD = wallPosition.Split(' ');
                if (posD[2] != "l" && posD[2] != "r")
                    return null;

                string[] widD = posD[0].Substring(3).Split(',');
                int widthX = int.Parse(widD[0]);
                int widthY = int.Parse(widD[1]);
                if (widthX < 0 || widthY < 0 || widthX > 200 || widthY > 200)
                    return null;

                string[] lenD = posD[1].Substring(2).Split(',');
                int lengthX = int.Parse(lenD[0]);
                int lengthY = int.Parse(lenD[1]);
                if (lengthX < 0 || lengthY < 0 || lengthX > 200 || lengthY > 200)
                    return null;

                return ":w=" + widthX + "," + widthY + " " + "l=" + lengthX + "," + lengthY + " " + posD[2];
            }
            catch
            {
                return null;
            }
        }

        internal static bool TilesTouching(int X1, int Y1, int X2, int Y2)
        {
            if (!(Math.Abs(X1 - X2) > 1 || Math.Abs(Y1 - Y2) > 1)) return true;
            if (X1 == X2 && Y1 == Y2) return true;
            return false;
        }

        internal static int TileDistance(int X1, int Y1, int X2, int Y2)
        {
            return Math.Abs(X1 - X2) + Math.Abs(Y1 - Y2);
        }

        //internal void ResetUsers()
        //{
        //    for (int i = 0; i < UserList.Length; i++)
        //    {
        //        RoomUser User = UserList[i];
        //        if (User == null)
        //            continue;
        //        User.GoalX = User.X;
        //        User.GoalY = User.Y;
        //        User.ResetStatus();
        //        User.ClearMovement(false);
        //    }
        //}

        internal void SetSpeed(int p)
        {
            this.mRollerSpeed = p;
        }
    }

    internal class AffectedTile
    {
        int mX;
        int mY;
        int mI;

        internal AffectedTile(int x, int y, int i)
        {
            mX = x;
            mY = y;
            mI = i;
        }

        internal int X
        {
            get
            {
                return mX;
            }
        }

        internal int Y
        {
            get
            {
                return mY;
            }
        }

        //internal int I
        //{
        //    get
        //    {
        //        return mI;
        //    }
        //}
    }
}
