﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Threading;
using System.Threading.Tasks;
using Butterfly.Collections;
using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Users;
using Butterfly.Messages;
using Butterfly.Storage;


namespace Butterfly.HabboHotel.Rooms
{
    class RoomManager
    {
        internal readonly int MAX_PETS_PER_ROOM = 15;

        //private Dictionary<uint, Room> Rooms;
        private Sandbox Rooms;
        private Dictionary<string, RoomModel> Models;
        private Hashtable mLoadedRoomData;
        private List<TeleUserData> TeleActions;
        private Task EngineThread;
        //private bool CycleStarted = false;
        //private DateTime LastCycle;
        //private List<uint> RoomsToUnload;
        internal List<RoomData> mDryCache;
        private Task mCycleTask;
        private Task mRemoverTask;

        //private Task mChatlogLogger;
        //private Hashtable mChatloggs;
        //private int mChatlogCount;

        //private byte[] mNavigatorCache;
        //private Task mNavigatorCacheWorker;

        //private void CycleNavi()
        //{
        //    while (true)
        //    {

        //        Thread.Sleep(5000);
        //    }
        //}



        internal int LoadedRoomsCount
        {
            get
            {
                return this.Rooms.Count;
            }
        }

        internal static int CreatedRoomsCount
        {
            get
            {
                int Count = 0;
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    Count = int.Parse(dbClient.ReadString("SELECT COUNT(*) FROM rooms"));
                }
                return Count;
            }
        }

        //internal int GetHanditemCount
        //{
        //    get
        //    {
        //        int Count = 0;
        //        using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //        {
        //            Count = int.Parse(dbClient.ReadString("SELECT COUNT(*) FROM user_items"));
        //        }
        //        return Count;
        //    }
        //}

        //internal int GetRoomItemCount
        //{
        //    get
        //    {
        //        int Count = 0;
        //        using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //        {
        //            Count = int.Parse(dbClient.ReadString("SELECT COUNT(*) FROM room_items"));
        //        }
        //        return Count;
        //    }
        //}

        internal void LoadDryCache()
        {
            mDryCache.Clear();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                for (int i = 0; i < 12; i++)
                {
                    //SQL Query should be rewritten
                    DataTable RoomTable = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags, allow_rightsoverride FROM rooms WHERE category = '" + i + "' AND roomtype = 'private' ORDER BY users_now DESC LIMIT 40");

                    foreach (DataRow dRow in RoomTable.Rows)
                    {
                        mDryCache.Add(FetchDryRoomData((uint)dRow["id"], dRow));
                    }
                }
            }
        }

        private bool CheckState(uint RoomID)
        {
            foreach (RoomData Data in mDryCache)
            {
                if (Data.Id == RoomID)
                    return true;
            }
            return false;
        }

        internal void CheckForDryCacheConflicts(uint RoomID)
        {
            if (CheckState(RoomID))
                LoadDryCache();
        }

        internal RoomManager()
        {
            this.Rooms = new Sandbox();
            this.Models = new Dictionary<string, RoomModel>();
            this.TeleActions = new List<TeleUserData>();
            //this.RoomsToUnload = new List<uint>();
            this.mLoadedRoomData = new Hashtable();
            mDryCache = new List<RoomData>();

            this.EngineThread = new Task(ProcessEngine);
            //this.EngineThread. = "Room Engine";
            //this.EngineThread.Priority = ThreadPriority.BelowNormal;
            this.EngineThread.Start();

            //mChatlogLogger = new Task(ChatlogCycle);
            //mChatlogLogger.Start();

            //this.EngineWatch = new Thread(new ThreadStart(WatchCycle));
            //EngineWatch.Start();
            //LoadDryCache();
            mToRemove = new List<Room>();

            mCycleTask = new Task(RoomCycleTask);
            mCycleTask.Start();

            mRemoverTask = new Task(RemoveTask);
            mRemoverTask.Start();
        }



        private void RoomCycleTask()
        {
            List<Room> ToRemove = new List<Room>();
            while (true)
            {
                DateTime CycleStart = DateTime.Now;

                using (ClonedTable Table = Rooms.GetThreadSafeTable)
                {
                    foreach (Room Room in Table.Values)
                    {
                        if (Room.mIsIdle)
                            ToRemove.Add(Room);
                        else
                            Room.ProcessRoom();
                    }

                    foreach (Room removeThis in ToRemove)
                    {
                        Rooms.Remove(removeThis);
                        mToRemove.Add(removeThis);
                    }
                }

                ToRemove.Clear();
                TimeSpan TimeSpent = DateTime.Now - CycleStart;
                int Sleeptime = 500 - TimeSpent.Milliseconds;
                if (Sleeptime > 0)
                    Thread.Sleep(Sleeptime);
            }

        }
        private List<Room> mToRemove;

        private void RemoveTask()
        {
            while (true)
            {
                List<Room> Remove = new List<Room>(mToRemove);
                mToRemove.Clear();
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    foreach (Room room in Remove)
                    {
                        Logging.WriteLine("[RoomMgr] Requesting unload of idle room - ID#: " + room.RoomId);

                        room.SaveFurniture(dbClient);
                        UnloadRoom(room);
                    }
                }
                Remove.Clear();

                Thread.Sleep(5000);
            }
        }

        //private void ChatlogCycle()
        //{
        //    mChatloggs = new Hashtable();
        //    mChatlogCount = 0;

        //    while (true)
        //    {
        //        Hashtable Chatloggs;
        //        using (TimedLock.Lock(mChatloggs))
        //        {
        //            Chatloggs = mChatloggs.Clone() as Hashtable;
        //            mChatloggs.Clear();
        //        }
        //        if (Chatloggs.Count > 0)
        //        {
        //            StringBuilder Query = new StringBuilder();
        //            using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //            {
        //                Query.Append("INSERT INTO chatlogs (user_id,room_id,hour,minute,timestamp,message,user_name,full_date) VALUES ");
        //                foreach (ChatMessage Message in Chatloggs.Values)
        //                {
        //                    dbClient.AddParamWithValue("message" + Message.ID, Message.Message);
        //                    dbClient.AddParamWithValue("usrname" + Message.ID, Message.Username);
        //                    Query.Append("('" + Message.ID + "','" + Message.RoomID + "','" + Message.Hour + "','" + Message.Minute + "','" + Message.UnixTimeStamp + "',@message" + Message.ID + ",@usrname" + Message.ID + ",'" + Message.FullDate + "'),");
        //                }
        //                Query.Remove(Query.Length - 1, 1);
        //                Query.Append(";");

        //                dbClient.ExecuteQuery(Query.ToString());
        //            }
        //            Query.Clear();
        //        }
        //        Chatloggs.Clear();

        //        Thread.Sleep(30000);
        //    }
        //}

        //internal void AddChatmessage(string pUsername, uint pHabboID, uint pRoomID, string pMessage)
        //{
        //    //int MessageID = mChatlogCount++;
        //    //mChatloggs.Add(MessageID, new ChatMessage(MessageID, pMessage, pRoomID, pHabboID, DateTime.Now.Hour, DateTime.Now.Minute, UberEnvironment.GetUnixTimestamp(), DateTime.Now.ToLongDateString(), pUsername));
        //}

        internal void RemoveAllRooms()
        {
            using (ClonedTable Table = Rooms.GetThreadSafeTable)
            {
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    foreach (Room Room in Table.Values)
                    {
                        Room.SaveFurniture(dbClient);
                    }
                }
                Console.WriteLine("Done with furniture saving, disposing rooms");
                foreach (Room Room in Table.Values)
                {
                    try
                    {
                        Room.Destroy();
                    }
                    catch { }
                }
                Console.WriteLine("Done disposing rooms!");
            }
        }

        //private void WatchCycle()
        //{
        //    while (true)
        //    {
        //        LastExecuted1 = DateTime.Now;
        //        if (CycleStarted)
        //        {
        //            TimeSpan SinceLastCycle = DateTime.Now - LastCycle;
        //            if (SinceLastCycle.Seconds > 10)
        //                ResetCycle();
        //        }

        //        Thread.Sleep(10000);
        //    }
        //}

        //internal void ResetCycle()
        //{
        //    Logging.WriteLine("<---- STATUS MESSAGE: CYCLE THREAD RESTARTING DUE HANGUP! ---->");

        //    if (EngineThread.IsAlive)
        //        EngineThread.Abort();

        //    Logging.WriteLine("<---- STATUS MESSAGE: CYCLE THREAD ABORTED ---->");

        //    using (ClonedTable Table = Rooms.GetThreadSafeTable)
        //    {
        //        foreach (Room Room in Table.Values)
        //        {
        //            Logging.WriteLine("<---- STATUS MESSAGE: RESETTING USERS IN ROOM [" + Room.RoomId + "] ---->");
        //            Room.ResetUsers();
        //        }
        //    }

        //    Logging.WriteLine("<---- STATUS MESSAGE: CYCLE THREAD STARTING ---->");

        //    EngineThread = new Thread(new ThreadStart(ProcessEngine));
        //    EngineThread.Start();

        //    Logging.WriteLine("<---- STATUS MESSAGE: CYCLE THREAD HAVE BEEN RESTARTED DUE HANGUP! ---->");
        //}

        internal void AddTeleAction(TeleUserData Act)
        {
            this.TeleActions.Add(Act);
        }

        internal List<Room> GetEventRoomsForCategory(int Category)
        {
            List<Room> EventRooms = new List<Room>();

            try
            {
                using (ClonedTable Table = Rooms.GetThreadSafeTable)
                {
                    foreach (Room Room in Table.Values)
                    {
                        if (Room.Event == null)
                            continue;

                        if (Category > 0 && Room.Event.Category != Category)
                            continue;

                        EventRooms.Add(Room);
                    }
                }
            }
            catch { }

            return EventRooms;
        }

        //internal static DateTime LastExecuted;

        private void ProcessEngine()
        {
            //Thread.Sleep(5000); Because?
            while (true)
            {
                try
                {
                    if (TeleActions.Count > 0)
                    {
                        DateTime ExecutionStart = DateTime.Now;

                        try
                        {
                            //LastCycle = DateTime.Now;
                            List<TeleUserData> OldTeleActions = new List<TeleUserData>();
                            using (TimedLock.Lock(this.TeleActions))
                            {
                                OldTeleActions.AddRange(this.TeleActions);
                                TeleActions.Clear();
                            }

                            foreach (TeleUserData TeleAction in OldTeleActions)
                            {
                                if (TeleAction == null)
                                    continue;
                                TeleAction.Execute();
                            }
                        }
                        catch (Exception e)
                        {
                            //Logging.WriteLine("InvalidOpException in Room Manager..", Core.LogLevel.Error);
                            Logging.HandleException(e, "tele code error");

                        }

                        finally
                        {
                            DateTime ExecutionComplete = DateTime.Now;
                            TimeSpan Diff = ExecutionComplete - ExecutionStart;

                            double sleepTime = 500 - Diff.TotalMilliseconds;

                            if (sleepTime < 0)
                                sleepTime = 0;

                            if (sleepTime > 500)
                                sleepTime = 500;

                            Thread.Sleep((int)Math.Floor(sleepTime));
                        }
                    }
                    else
                        Thread.Sleep(500);
                }
                catch (Exception e)
                {
                    Logging.LogThreadException(e.ToString(), "Room manager task (Process engine)");

                    try
                    {
                        if (TeleActions != null)
                            TeleActions.Clear();
                    }
                    catch { }

                    Thread.Sleep(500);
                }
            }
        }

        internal void LoadModels(DatabaseClient dbClient)
        {
            Models.Clear();

            DataTable Data = dbClient.ReadDataTable("SELECT id,door_x,door_y,door_z,door_dir,heightmap,public_items,club_only FROM room_models");
            DataTable PublicRoomItems = dbClient.ReadDataTable("SELECT model, x, y, name, seat, rotation FROM room_model_static");

            List<PublicRoomSquare> PublicRoomDataSqList = new List<PublicRoomSquare>();

            foreach (DataRow dRow in PublicRoomItems.Rows)
            {
                PublicRoomDataSqList.Add(new PublicRoomSquare(dRow));
            }

            if (Data == null)
                return;

            foreach (DataRow Row in Data.Rows)
            {
                string Modelname = (string)Row["id"];

                //var TmpSquares = from k in PublicRoomDataSqList
                //                   where k.RoomModelName == Modelname
                //                   select k;
                //LINQ saved the day :D (Aggain...)

                Models.Add(Modelname, new RoomModel((int)Row["door_x"],
                    (int)Row["door_y"], (Double)Row["door_z"], (int)Row["door_dir"], (string)Row["heightmap"],
                    (string)Row["public_items"], ButterflyEnvironment.EnumToBool(Row["club_only"].ToString())));
            }
        }

        private static RoomModel GetCustomData(UInt32 roomID)
        {
            DataRow RoomData;
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                RoomData = dbClient.ReadDataRow("SELECT doorx,doory,height,modeldata FROM room_models_customs WHERE roomid = '" + roomID + "'");
            }

            return new RoomModel((int)RoomData["doorx"], (int)RoomData["doory"], (Double)RoomData["height"], 2, (string)RoomData["modeldata"], "", false);
        }

        internal RoomModel GetModel(string Model, UInt32 RoomID)
        {
            if (Model == "custom")
                return GetCustomData(RoomID);

            if (Models.ContainsKey(Model))
                return Models[Model];

            return null;
        }

        internal RoomData GenerateNullableRoomData(UInt32 RoomId)
        {
            if (GenerateRoomData(RoomId) != null)
            {
                return GenerateRoomData(RoomId);
            }

            RoomData Data = new RoomData();
            Data.FillNull(RoomId);
            return Data;
        }

        internal RoomData GenerateRoomData(UInt32 RoomId)
        {
            RoomData Data = new RoomData();
            if (mLoadedRoomData.ContainsKey(RoomId))
                return mLoadedRoomData[RoomId] as RoomData;

            if (IsRoomLoaded(RoomId))
            {
                return GetRoom(RoomId).GetRoomData;
            }
            else
            {
                DataRow Row = null;

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    Row = dbClient.ReadDataRow("SELECT rooms.id, rooms.caption, rooms.description, rooms.roomtype, rooms.owner, rooms.state, rooms.category, rooms.users_now, rooms.users_max, rooms.model_name, rooms.public_ccts, rooms.score, rooms.allow_pets, rooms.allow_pets_eat, rooms.allow_walkthrough, rooms.allow_hidewall, rooms.password, rooms.wallpaper, rooms.floor, rooms.landscape, rooms.icon_items, rooms.icon_bg, rooms.icon_fg, rooms.tags, rooms.allow_rightsoverride FROM rooms WHERE id = '" + RoomId + "' LIMIT 1");
                }

                if (Row == null)
                {
                    return null;
                }

                Data.Fill(Row);
                mLoadedRoomData.Add(RoomId, Data);
            }

            return Data;
        }

        internal Boolean IsRoomLoaded(UInt32 RoomId)
        {
            return Rooms.ContainsKey(RoomId);
        }

        internal Room LoadRoom(UInt32 Id)
        {
            if (IsRoomLoaded(Id))
            {
                return GetRoom(Id);
            }

            RoomData Data = GenerateRoomData(Id);

            if (Data == null)
                return null;

            Room Room = new Room(Data.Id, Data.Name, Data.Description, Data.Type, Data.Owner, Data.Category, Data.State,
                Data.UsersMax, Data.ModelName, Data.CCTs, Data.Score, Data.Tags, Data.AllowPets, Data.AllowPetsEating,
                Data.AllowWalkthrough, Data.Hidewall, Data.Icon, Data.Password, Data.Wallpaper, Data.Floor, Data.Landscape, Data, Data.AllowRightsOverride);

            if (!Rooms.ContainsKey(Id))
                Rooms.Add(Room.RoomId, Room);
            else
                return GetRoom(Id);
            

            Room.InitBots();
            Room.InitPets();

            //Logging.WriteLine("[RoomMgr] Loaded room: \"" + Room.Name + "\" (ID: " + Id + ")");

            return Room;
        }

        //internal void RequestRoomUnload(UInt32 Id)
        //{
        //    UnloadRoom(Id);
        //}

        internal void UnloadRoom(Room Room)
        {
            if (Room == null)
            {
                return;
            }

            Rooms.Remove(Room.RoomId);
            Room.Destroy();

            //Logging.WriteLine("[RoomMgr] Unloaded room: \"" + Room.Name + "\" (ID: " + Room.RoomId + ")");
        }

        internal RoomData FetchRoomData(UInt32 RoomId, DataRow dRow)
        {
            if (mLoadedRoomData.ContainsKey(RoomId))
                return mLoadedRoomData[RoomId] as RoomData;

            RoomData Data = new RoomData();

            if (IsRoomLoaded(RoomId))
                Data = GetRoom(RoomId).GetRoomData;
            else
                Data.Fill(dRow);

            mLoadedRoomData.Add(RoomId, Data);
            return Data;
        }

        internal RoomData FetchDryRoomData(UInt32 RoomId, DataRow dRow)
        {
            if (mLoadedRoomData.ContainsKey(RoomId))
                return mLoadedRoomData[RoomId] as RoomData;

            RoomData Data = new RoomData();

            if (IsRoomLoaded(RoomId))
                Data = GetRoom(RoomId).GetRoomData;
            else
                Data.DeadFill(dRow);

            mLoadedRoomData.Add(RoomId, Data);
            return Data;
        }

        internal Room GetRoom(UInt32 RoomId)
        {
            try
            {
                return Rooms[RoomId] as Room;
            }
            catch { }

            return null;
            //throw new ObjectNotFoundException("No room for roomID " + RoomId + " found, script aborted.");
        }

        internal RoomData CreateRoom(GameClient Session, string Name, string Model)
        {
            Name = ButterflyEnvironment.FilterInjectionChars(Name);

            if (!Models.ContainsKey(Model))
            {
                Session.SendNotif("Sorry, this room model has not been added yet. Try again later.");
                return null;
            }

            if (Models[Model].ClubOnly && !Session.GetHabbo().HasFuse("fuse_use_special_room_layouts"))
            {
                Session.SendNotif("You must be an Uber Club member to use that room layout.");
                return null;
            }

            if (Name.Length < 3)
            {
                Session.SendNotif("Room name is too short for room creation!");
                return null;
            }

            UInt32 RoomId = 0;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("caption", Name);
                dbClient.AddParamWithValue("model", Model);
                dbClient.AddParamWithValue("username", Session.GetHabbo().Username);
                dbClient.ExecuteQuery("INSERT INTO rooms (roomtype,caption,owner,model_name) VALUES ('private',@caption,@username,@model)");
                //Session.GetHabbo().GetUserData.GetUsersRooms = dbClient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags FROM rooms WHERE owner = @username ORDER BY id ASC");

                RoomId = (UInt32)dbClient.ReadDataRow("SELECT id FROM rooms WHERE owner = @username AND caption = @caption ORDER BY id DESC")[0];

                Session.GetHabbo().UpdateRooms(dbClient);
            }
            
            return GenerateRoomData(RoomId);
        }

        internal Dictionary<Room, int> GetActiveRoomList()
        {
            Dictionary<Room, int> Toreturn = new Dictionary<Room, int>();
            using (ClonedTable ActiveRooms = Rooms.GetThreadSafeTable)
                foreach (Room ActiveRoom in ActiveRooms.Values)
                    if (ActiveRoom.UserCount > 0)
                        Toreturn.Add(ActiveRoom, ActiveRoom.UserCount);
                    
            return Toreturn;
        }
    }

    class TeleUserData
    {
        private UInt32 RoomId;
        private UInt32 TeleId;

        private GameClientMessageHandler mHandler;
        private Habbo mUserRefference;

        internal TeleUserData(GameClientMessageHandler pHandler, Habbo pUserRefference, UInt32 RoomId, UInt32 TeleId)
        {
            //this.User = User;
            this.mHandler = pHandler;
            this.mUserRefference = pUserRefference;
            this.RoomId = RoomId;
            this.TeleId = TeleId;
        }

        internal void Execute()
        {
            if (mHandler == null || mUserRefference == null)
            {
                return;
            }

            mUserRefference.IsTeleporting = true;
            mUserRefference.TeleporterId = TeleId;
            mHandler.PrepareRoomForUser(RoomId, "");
        }
    }
}
