﻿using System;
using System.Collections.Generic;
using System.Data;

using Butterfly.HabboHotel.GameClients;
using Butterfly.Storage;

namespace Butterfly.HabboHotel.Support
{
    class ModerationBanManager
    {
        internal List<ModerationBan> Bans;

        internal ModerationBanManager()
        {
            Bans = new List<ModerationBan>();
        }

        internal void LoadBans(DatabaseClient dbClient)
        {
            Bans.Clear();

            DataTable BanData = dbClient.ReadDataTable("SELECT bantype,value,reason,expire FROM bans WHERE expire > '" + ButterflyEnvironment.GetUnixTimestamp() + "'");

            if (BanData == null)
            {
                return;
            }

            foreach (DataRow Row in BanData.Rows)
            {
                ModerationBanType Type = ModerationBanType.IP;

                if ((string)Row["bantype"] == "user")
                {
                    Type = ModerationBanType.USERNAME;
                }

                Bans.Add(new ModerationBan(Type, (string)Row["value"], (string)Row["reason"], (Double)Row["expire"]));
            }
        }

        internal void CheckForBanConflicts(GameClient Client)
        {
            using (TimedLock.Lock(Bans))
            {
                foreach (ModerationBan Ban in Bans)
                {
                    if (Ban.Expired)
                    {
                        continue;
                    }

                    if (Ban.Type == ModerationBanType.IP && Client.GetConnection().ipAddress == Ban.Variable)
                    {
                        throw new ModerationBanException(Ban.ReasonMessage);
                    }

                    if (Client.GetHabbo() != null)
                    {
                        if (Ban.Type == ModerationBanType.USERNAME && Client.GetHabbo().Username.ToLower() == Ban.Variable.ToLower())
                        {
                            throw new ModerationBanException(Ban.ReasonMessage);
                        }
                    }
                }
            }
        }

        // PENDING REWRITE
        internal void BanUser(GameClient Client, string Moderator, Double LengthSeconds, string Reason, Boolean IpBan)
        {
            ModerationBanType Type = ModerationBanType.USERNAME;
            string Var = Client.GetHabbo().Username;
            string RawVar = "user";
            Double Expire = ButterflyEnvironment.GetUnixTimestamp() + LengthSeconds;

            if (IpBan)
            {
                Type = ModerationBanType.IP;
                Var = Client.GetConnection().ipAddress;
                RawVar = "ip";
            }

            Bans.Add(new ModerationBan(Type, Var, Reason, Expire));

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("rawvar", RawVar);
                dbClient.AddParamWithValue("var", Var);
                dbClient.AddParamWithValue("reason", Reason);
                dbClient.AddParamWithValue("mod", Moderator);

                dbClient.ExecuteQuery("INSERT INTO bans (bantype,value,reason,expire,added_by,added_date) VALUES (@rawvar,@var,@reason,'" + Expire + "',@mod,'" + DateTime.Now.ToLongDateString() + "')");
            }

            if (IpBan)
            {
                DataTable UsersAffected = null;

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.AddParamWithValue("var", Var);
                    UsersAffected = dbClient.ReadDataTable("SELECT id FROM users WHERE ip_last = @var");
                }

                if (UsersAffected != null)
                {
                    foreach (DataRow Row in UsersAffected.Rows)
                    {
                        using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            dbClient.ExecuteQuery("UPDATE user_info SET bans = bans + 1 WHERE user_id = '" + (uint)Row["id"] + "' LIMIT 1");
                        }
                    }
                }
            }
            else
            {
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.ExecuteQuery("UPDATE user_info SET bans = bans + 1 WHERE user_id = '" + Client.GetHabbo().Id + "' LIMIT 1");
                }
            }

            Client.SendBanMessage("You have been banned: " + Reason);
            Client.Disconnect();
        }
    }
}
