﻿using System;
using System.Collections.Generic;
using System.Data;
using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Users.Badges;
using Butterfly.HabboHotel.Users.Inventory;
using Butterfly.HabboHotel.Users.Messenger;
using Butterfly.HabboHotel.Users.Subscriptions;
using Butterfly.Storage;
using Butterfly_Emulator.ServerManager;

namespace Butterfly.HabboHotel.Users
{
    class Habbo
    {
        internal UInt32 Id;

        internal string Username;
        internal string RealName;

        //internal string AuthTicket;

        internal uint Rank;

        internal string Motto;
        internal string Look;
        internal string Gender;

        internal Int32 Credits;
        internal Int32 ActivityPoints;
        internal Double LastActivityPointsUpdate;

        internal bool Muted;
        //internal bool WarningGiven = false;
        //internal DateTime MuteStarted;

        internal int Respect;
        internal int DailyRespectPoints;
        internal int DailyPetRespectPoints;

        internal uint LoadingRoom;
        internal Boolean LoadingChecksPassed;
        internal uint CurrentRoomId;
        internal uint HomeRoom;

        internal bool IsTeleporting;
        internal UInt32 TeleporterId;

        internal List<UInt32> FavoriteRooms;
        internal List<UInt32> MutedUsers;
        internal List<string> Tags;
        internal Dictionary<uint, int> Achievements;
        internal List<uint> RatedRooms;

        private SubscriptionManager SubscriptionManager;
        private HabboMessenger Messenger;
        private BadgeComponent BadgeComponent;
        private InventoryComponent InventoryComponent;
        private AvatarEffectsInventoryComponent AvatarEffectsInventoryComponent;

        private GameClient mClient;

        //internal int NewbieStatus;
        internal bool SpectatorMode;
        internal bool Disconnected;

        internal bool CalledGuideBot;
        internal bool MutantPenalty;

        //internal bool BlockNewFriends;

        private DataTable mFriendList;
        private DataTable mFriendRequests;

        internal List<RoomData> UsersRooms;

        internal Boolean InRoom
        {
            get
            {
                if (CurrentRoomId >= 1)
                {
                    return true;
                }

                return false;
            }
        }

        internal Room CurrentRoom
        {
            get
            {
                if (CurrentRoomId <= 0)
                {
                    return null;
                }

                return ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(CurrentRoomId);
            }
        }

        internal Habbo(UInt32 Id, string Username, string RealName,
            uint Rank, string Motto, string Look, string Gender, Int32 Credits,
            Int32 ActivityPoints, Double LastActivityPointsUpdate, bool Muted,
            UInt32 HomeRoom, Int32 Respect, Int32 DailyRespectPoints, Int32 DailyPetRespectPoints,
            bool MutantPenalty, GameClient pClient, HabboData UserData)
        {
            this.Id = Id;
            this.Username = Username;
            this.RealName = RealName;
            //this.AuthTicket = AuthTicket;
            this.Rank = Rank;
            this.Motto = Motto;
            this.Look = Look.ToLower();
            this.Gender = Gender.ToLower();
            this.Credits = Credits;
            this.ActivityPoints = ActivityPoints;
            this.LastActivityPointsUpdate = LastActivityPointsUpdate;
            this.Muted = Muted;
            this.LoadingRoom = 0;
            this.LoadingChecksPassed = false;
            this.CurrentRoomId = 0;
            this.HomeRoom = HomeRoom;
            this.FavoriteRooms = new List<uint>();
            this.MutedUsers = new List<uint>();
            this.Tags = new List<string>();
            this.Achievements = new Dictionary<uint, int>();
            this.RatedRooms = new List<uint>();
            this.Respect = Respect;
            this.DailyRespectPoints = DailyRespectPoints;
            this.DailyPetRespectPoints = DailyPetRespectPoints;
            //this.NewbieStatus = NewbieStatus;
            this.CalledGuideBot = false;
            this.MutantPenalty = MutantPenalty;
            //this.BlockNewFriends = BlockNewFriends;

            this.IsTeleporting = false;
            this.TeleporterId = 0;
            this.mClient = pClient;
            this.UsersRooms = new List<RoomData>();

            if (pClient != null)
            {
                SubscriptionManager = new SubscriptionManager(Id, UserData);
                BadgeComponent = new BadgeComponent(Id, UserData);
                InventoryComponent = new InventoryComponent(Id, pClient, UserData);
                AvatarEffectsInventoryComponent = new AvatarEffectsInventoryComponent(Id, pClient, UserData);

                this.SpectatorMode = false;
                this.Disconnected = false;

                foreach (DataRow Row in UserData.GetUsersRooms.Rows)
                    UsersRooms.Add(ButterflyEnvironment.GetGame().GetRoomManager().FetchRoomData((uint)Row["id"], Row));
                mFriendList = UserData.GetFriendList;
                mFriendRequests = UserData.GetFriendReqs;
                //Logging.WriteLine(Username + " -> Online!");
            }
            
        }

        //internal HabboData GetUserData
        //{
        //    get
        //    {
        //        return mUserData;
        //    }
        //}

        internal void UpdateRooms(DatabaseClient dbclient)
        {
            UsersRooms.Clear();

            dbclient.AddParamWithValue("name", Username);
            DataTable dbTable = dbclient.ReadDataTable("SELECT id, caption, description, roomtype, owner, state, category, users_now, users_max, model_name, public_ccts, score, allow_pets, allow_pets_eat, allow_walkthrough, allow_hidewall, password, wallpaper, floor, landscape, icon_items, icon_bg, icon_fg, tags, allow_rightsoverride FROM rooms WHERE owner = @name ORDER BY id ASC");

            foreach (DataRow Row in dbTable.Rows)
                UsersRooms.Add(ButterflyEnvironment.GetGame().GetRoomManager().FetchRoomData((uint)Row["id"], Row));
        }

        internal void LoadData(HabboData UserData)
        {
            //SubscriptionManager.LoadSubscriptions();
            //BadgeComponent.LoadBadges();
            //InventoryComponent.LoadInventory();
            //AvatarEffectsInventoryComponent.LoadEffects();

            LoadAchievements(UserData);
            LoadFavorites(UserData);
            LoadMutedUsers(UserData);
            LoadTags(UserData);

        }

        internal bool HasFuse(string Fuse)
        {
            if (ButterflyEnvironment.GetGame().GetRoleManager().RankHasRight(Rank, Fuse))
            {
                return true;
            }

            foreach (string SubscriptionId in GetSubscriptionManager().SubList)
            {
                if (ButterflyEnvironment.GetGame().GetRoleManager().SubHasRight(SubscriptionId, Fuse))
                {
                    return true;
                }
            }

            return false;
        }

        internal void LoadFavorites(HabboData UserData)
        {
            this.FavoriteRooms.Clear();

            DataTable Data = UserData.GetUserFavouriteRooms;

            foreach (DataRow Row in Data.Rows)
            {
                FavoriteRooms.Add((uint)Row["room_id"]);
            }
        }

        internal void LoadMutedUsers(HabboData UserData)
        {
            DataTable Data = UserData.GetUserIgnores;

            foreach (DataRow Row in Data.Rows)
            {
                MutedUsers.Add((uint)Row["ignore_id"]);
            }
        }

        internal void LoadTags(HabboData UserData)
        {
            this.Tags.Clear();

            DataTable Data = UserData.GetUserTags;

            foreach (DataRow Row in Data.Rows)
            {
                Tags.Add((string)Row["tag"]);
            }

            if (Tags.Count >= 5)
            {
                ButterflyEnvironment.GetGame().GetAchievementManager().UnlockAchievement(GetClient(), 7, 1);
            }
        }

        internal void LoadAchievements(HabboData UserData)
        {
            DataTable Data = UserData.GetAchievementData;

            if (Data == null)
                return;

            foreach (DataRow Row in Data.Rows)
            {
                Achievements.Add((uint)Row["achievement_id"], (int)Row["achievement_level"]);
            }
        }

        private bool HabboinfoSaved = false;

        internal string GetQueryString
        {
            get
            {
                HabboinfoSaved = true;
                return "UPDATE users SET users.last_online = '" + DateTime.Now.ToString() + "', users.online = '0', activity_points = '" + ActivityPoints + "', activity_points_lastupdate = '" + LastActivityPointsUpdate + "', credits = '" + Credits + "' WHERE id = '" + Id + "'; ";
            }
        }

        internal void OnDisconnect()
        {
            
            if (this.Disconnected)
                return;

            this.Disconnected = true;

            SessionManagement.IncreaseDisconnection();
            Logging.WriteLine(Username + " has logged out.");

            
            if (!HabboinfoSaved)
            {
                //DateTime Start = DateTime.Now;
                HabboinfoSaved = true;
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    //TimeSpan TimeUsed1 = DateTime.Now - Start;
                    //Logging.LogThreadException("Time used on sys part 1: " + TimeUsed1.Seconds + "s, " + TimeUsed1.Milliseconds + "ms", "");

                    dbClient.ExecuteQuery("UPDATE users SET users.last_online = '" + DateTime.Now.ToString() + "', users.online = '0', activity_points = '" + ActivityPoints + "', activity_points_lastupdate = '" + LastActivityPointsUpdate + "', credits = '" + Credits + "' WHERE id = '" + Id + "' LIMIT 1;");
                }
            }
            
            //DateTime Start2 = DateTime.Now;
            if (InRoom && CurrentRoom != null)
                CurrentRoom.RemoveUserFromRoom(mClient, false, false);
            //TimeSpan TimeUsed2 = DateTime.Now - Start2;
            //Logging.LogThreadException("Time used on OnDisconnect part 2: " + TimeUsed2.Seconds + "s, " + TimeUsed2.Milliseconds + "ms", "");

            if (Messenger != null)
            {
                Messenger.AppearOffline = true;
                Messenger.OnStatusChanged(true);
                //Messenger = null;
            }

            if (SubscriptionManager != null)
            {
                SubscriptionManager.Clear();
                //SubscriptionManager = null;
            }

            InventoryComponent.RunDBUpdate();
            //mUserData.Clear();
            // todo: drop events, kick bots, etc


            
        }

        //internal void OnEnterRoom(UInt32 RoomId)
        //{

        //}

        //internal void OnLeaveRoom()
        //{

        //}

        internal void InitMessenger()
        {
            if (GetMessenger() != null)
            {
                return;
            }
            Messenger = new HabboMessenger(Id);
            Messenger.LoadBuddies(mFriendList);
            Messenger.LoadRequests(mFriendRequests);

            GameClient Client = GetClient();
            if (Client == null)
                return;

            Client.SendMessage(Messenger.SerializeFriends());
            Client.SendMessage(Messenger.SerializeRequests());

            Messenger.OnStatusChanged(true);
        }

        internal void UpdateCreditsBalance()
        {
            mClient.GetMessageHandler().GetResponse().Init(6);
            mClient.GetMessageHandler().GetResponse().AppendStringWithBreak(Credits + ".0");
            mClient.GetMessageHandler().SendResponse();

            //if (InDatabase)
            //{
            //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //    {
            //        dbClient.ExecuteQuery("UPDATE users SET credits = '" + Credits + "' WHERE id = '" + Id + "' LIMIT 1;");
            //    }
            //}
        }

        internal void UpdateActivityPointsBalance(Boolean InDatabase)
        {
            UpdateActivityPointsBalance(0);
        }

        internal void UpdateActivityPointsBalance(int NotifAmount)
        {
            mClient.GetMessageHandler().GetResponse().Init(438);
            mClient.GetMessageHandler().GetResponse().AppendInt32(ActivityPoints);
            mClient.GetMessageHandler().GetResponse().AppendInt32(NotifAmount);
            mClient.GetMessageHandler().SendResponse();

            //if (InDatabase)
            //{
            //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //    {
            //        dbClient.ExecuteQuery("UPDATE users SET activity_points = '" + ActivityPoints + "', activity_points_lastupdate = '" + LastActivityPointsUpdate + "' WHERE id = '" + Id + "' LIMIT 1;");
            //    }
            //}
        }

        internal void Mute()
        {
            if (!this.Muted)
            {
                GetClient().SendNotif("You have been muted by an moderator.");
                this.Muted = true;
            }
        }

        internal void Unmute()
        {
            if (this.Muted)
            {
                GetClient().SendNotif("You have been unmuted by an moderator.");
                this.Muted = false;
            }
        }

        private GameClient GetClient()
        {
            return ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(Id);
        }

        internal SubscriptionManager GetSubscriptionManager()
        {
            return SubscriptionManager;
        }

        internal HabboMessenger GetMessenger()
        {
            return Messenger;
        }

        internal BadgeComponent GetBadgeComponent()
        {
            return BadgeComponent;
        }

        internal InventoryComponent GetInventoryComponent()
        {
            return InventoryComponent;
        }

        internal AvatarEffectsInventoryComponent GetAvatarEffectsInventoryComponent()
        {
            return AvatarEffectsInventoryComponent;
        }
    }
}
