﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Butterfly.Core;
using Butterfly.HabboHotel.Catalogs;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pets;
using Butterfly.Messages;
using Butterfly.Storage;

namespace Butterfly.HabboHotel.Users.Inventory
{
    class InventoryComponent
    {
        private List<UserItem> InventoryItems;
        private Hashtable InventoryPets;
        private Hashtable mAddedItems;
        private List<UInt32> mRemovedItems;
        private GameClient mClient;

        internal uint UserId;

        internal int ItemCount
        {
            get
            {
                return InventoryItems.Count;
            }
        }

        //internal int PetCount
        //{
        //    get
        //    {
        //        return InventoryPets.Count;
        //    }
        //}

        internal InventoryComponent(uint UserId, GameClient Client, HabboData UserData)
        {
            this.mClient = Client;
            this.UserId = UserId;
            this.InventoryItems = new List<UserItem>();
            this.InventoryPets = new Hashtable();
            this.mAddedItems = new Hashtable();
            this.mRemovedItems = new List<UInt32>();

            this.InventoryItems.Clear();

            DataTable Data = UserData.GetUserInventory;
            //Data = dbClient.ReadDataTable("SELECT user_items.id,user_items.base_item,user_items.extra_data FROM user_items WHERE user_id = " + UserId);

            foreach (DataRow Row in Data.Rows)
                InventoryItems.Add(new UserItem((uint)Row["id"], (uint)Row["base_item"], (string)Row["extra_data"]));

            this.InventoryPets.Clear();

            DataTable Data2 = UserData.GetUserPets;
            
            foreach (DataRow Row in Data2.Rows)
            {
                Pet PetToAdd = Catalog.GeneratePetFromRow(Row);
                InventoryPets.Add(PetToAdd.PetId, PetToAdd);
            }
        }

        internal void ClearItems()
        {
            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                dbClient.ExecuteQuery("DELETE FROM user_items WHERE user_id = '" + UserId + "';");

            mAddedItems.Clear();
            mRemovedItems.Clear();
            InventoryItems.Clear();
            InventoryPets.Clear();

            GetClient().GetMessageHandler().GetResponse().Init(101);
            GetClient().GetMessageHandler().SendResponse();
        }

        #region PETS
        //internal void ClearPets()
        //{
        //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
        //        dbClient.ExecuteQuery("DELETE FROM user_pets WHERE user_id = " + UserId + " AND room_id = 0;");

        //    UpdatePets(true);
        //}

        //internal void UpdatePets(bool FromDatabase)
        //{
        //    if (FromDatabase)
        //    {
        //        LoadInventory();
        //    }

        //    GetClient().SendMessage(SerializePetInventory());
        //}

        internal Pet GetPet(uint Id)
        {
            return InventoryPets[Id] as Pet;

            //using (TimedLock.Lock(this.InventoryPets))
            //{
            //    List<Pet>.Enumerator Pets = this.InventoryPets.GetEnumerator();

            //    while (Pets.MoveNext())
            //    {
            //        Pet Pet = Pets.Current;

            //        if (Pet.PetId == Id)
            //        {
            //            return Pet;
            //        }
            //    }
            //}

            //return null;
        }

        internal bool RemovePet(uint PetId)
        {

            ServerMessage RemoveMessage = new ServerMessage(604);
            RemoveMessage.AppendUInt(PetId);
            GetClient().SendMessage(RemoveMessage);

            InventoryPets.Remove(PetId);
            return true;
        }

        internal void MovePetToRoom(UInt32 PetId)
        {
            RemovePet(PetId);
            //if (RemovePet(PetId))
            //{
            //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //    {
            //        dbClient.AddParamWithValue("roomid", RoomId);
            //        dbClient.AddParamWithValue("petid", PetId);
            //        dbClient.ExecuteQuery("UPDATE user_pets SET room_id = @roomid, x = 0, y = 0, z = 0 WHERE id = @petid LIMIT 1");
            //    }
            //}
        }

        internal void AddPet(Pet Pet)
        {
            if (Pet == null)
                return;

            Pet.PlacedInRoom = false;

            InventoryPets.Add(Pet.PetId, Pet);

            //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.AddParamWithValue("botid", Pet.PetId);
            //    dbClient.ExecuteQuery("UPDATE user_pets SET room_id = 0, x = 0, y = 0, z = 0 WHERE id = @botid LIMIT 1");
            //}

            ServerMessage AddMessage = new ServerMessage(603);
            Pet.SerializeInventory(AddMessage);
            GetClient().SendMessage(AddMessage);
            //UpdatePets(false);
        }
        #endregion

        internal void LoadInventory()
        {
            using (TimedLock.Lock(this.InventoryItems))
            {
                this.InventoryItems.Clear();

                DataTable Data;
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                    Data = dbClient.ReadDataTable("SELECT user_items.id,user_items.base_item,user_items.extra_data FROM user_items WHERE user_id = " + UserId);

                if (Data != null)
                {
                    foreach (DataRow Row in Data.Rows)
                    {
                        InventoryItems.Add(new UserItem((uint)Row["id"], (uint)Row["base_item"], (string)Row["extra_data"]));
                    }
                }


                using (TimedLock.Lock(this.InventoryPets))
                {
                    this.InventoryPets.Clear();
                    DataTable Data2;
                    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                    {
                        dbClient.AddParamWithValue("userid", UserId);
                        Data2 = dbClient.ReadDataTable("SELECT id, user_id, room_id, name, type, race, color, expirience, energy, nutrition, respect, createstamp, x, y, z FROM user_pets WHERE user_id = @userid AND room_id = 0");
                    }

                    if (Data2 != null)
                    {
                        foreach (DataRow Row in Data2.Rows)
                        {
                            Pet newPet = Catalog.GeneratePetFromRow(Row);
                            InventoryPets.Add(newPet.PetId, newPet);
                        }
                    }

                }
            }
        }

        internal void UpdateItems(bool FromDatabase)
        {
            if (FromDatabase)
            {
                LoadInventory();
                RunDBUpdate();
            }

            if (mClient.GetMessageHandler() == null)
                return;
            if (mClient.GetMessageHandler().GetResponse() == null)
                return;

            GetClient().GetMessageHandler().GetResponse().Init(101);
            GetClient().GetMessageHandler().SendResponse();
        }
        

        internal UserItem GetItem(uint Id)
        {
            using (TimedLock.Lock(this.InventoryItems))
            {
                List<UserItem>.Enumerator Items = this.InventoryItems.GetEnumerator();

                while (Items.MoveNext())
                {
                    UserItem Item = Items.Current;

                    if (Item.Id == Id)
                    {
                        return Item;
                    }
                }
            }

            return null;
        }

        internal void AddItem(UInt32 Id, UInt32 BaseItem, string ExtraData)
        {
            using (TimedLock.Lock(this.InventoryItems))
            {
                UserItem ItemToAdd = new UserItem(Id, BaseItem, ExtraData);

                InventoryItems.Add(ItemToAdd);

                if (mRemovedItems.Contains(Id))
                    mRemovedItems.Remove(Id);
                if (mAddedItems.ContainsKey(Id))
                    return;

                mAddedItems.Add(Id, ItemToAdd);
                //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                //{
                //    dbClient.AddParamWithValue("extra_data", ExtraData);
                //    dbClient.ExecuteQuery("INSERT INTO user_items (id,user_id,base_item,extra_data) VALUES ('" + Id + "','" + UserId + "','" + BaseItem + "',@extra_data)");
                //}
                //Console.WriteLine("Item added: " + BaseItem);
            }
        }
        

        internal void RemoveItem(UInt32 Id, bool PlacedInroom)
        {
            GetClient().GetMessageHandler().GetResponse().Init(99);
            GetClient().GetMessageHandler().GetResponse().AppendUInt(Id);
            GetClient().GetMessageHandler().SendResponse();

            if (mAddedItems.ContainsKey(Id))
                mAddedItems.Remove(Id);
            if (mRemovedItems.Contains(Id))
                return;

            InventoryItems.Remove(GetItem(Id));
            if (PlacedInroom)
                mRemovedItems.Add(Id);
            

            //using (TimedLock.Lock(this.InventoryItems))
            //{
            //    InventoryItems.Remove(GetItem(Id));

            //    using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
            //    {
            //        dbClient.ExecuteQuery("DELETE FROM user_items WHERE id = '" + Id + "' LIMIT 1");
            //    }
            //}
        }

        internal ServerMessage SerializeItemInventory(int type)
        {
            ServerMessage Message = new ServerMessage(140);
            List<UserItem> FloorItems = new List<UserItem>();
            List<UserItem> WallItems = new List<UserItem>();
            using (TimedLock.Lock(this.InventoryItems))
            {
                List<UserItem>.Enumerator eItems = this.InventoryItems.GetEnumerator();

                while (eItems.MoveNext())
                {
                    if (eItems.Current.GetBaseItem().Type.ToString().ToLower() == "s")
                        FloorItems.Add(eItems.Current);
                    else
                        WallItems.Add(eItems.Current);
                }
            }
            if (type == 0)
            {
                Message.AppendStringWithBreak("S");
                Message.AppendInt32(1);
                Message.AppendInt32(1);
                Message.AppendInt32(FloorItems.Count);
                foreach (UserItem Item in FloorItems)
                {
                    Item.Serialize(Message);
                }
            }
            else
            {
                Message.AppendStringWithBreak("I");
                Message.AppendInt32(1);
                Message.AppendInt32(1);
                Message.AppendInt32(WallItems.Count);
                foreach (UserItem Item in WallItems)
                {
                    Item.Serialize(Message);
                }
            }
            return Message;
        }

        internal ServerMessage SerializePetInventory()
        {
            ServerMessage Message = new ServerMessage(600);
            Message.AppendInt32(InventoryPets.Count);

            foreach (Pet Pet in InventoryPets.Values)
            {
                Pet.SerializeInventory(Message);
            }
            //Console.WriteLine(InventoryPets.Count);
            return Message;
        }

        private GameClient GetClient()
        {
            return ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(UserId);
        }

        internal void AddItemArray(List<RoomItem> RoomItemList)
        {
            foreach (RoomItem Item in RoomItemList)
            {
                AddItem(Item.Id, Item.BaseItem, Item.ExtraData);
            }
        }

        internal void RunDBUpdate()
        {
            //DateTime Start = DateTime.Now;
            try
            {
                using (TimedLock.Lock(this))
                {
                    if (mRemovedItems.Count > 0 || mAddedItems.Count > 0)
                    {
                        using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                        {
                            StringBuilder QueryBuilder = new StringBuilder();

                            if (mAddedItems.Count > 0)
                            {
                                foreach (UserItem Item in mAddedItems.Values)
                                {
                                    dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
                                    QueryBuilder.Append("REPLACE INTO user_items VALUES ('" + Item.Id + "','" + UserId + "','" + Item.BaseItem + "',@extra_data" + Item.Id + "); ");
                                }

                                mAddedItems.Clear();
                            }
                            if (mRemovedItems.Count > 0)
                            {
                                foreach (UInt32 ItemID in mRemovedItems)
                                {
                                    QueryBuilder.Append("DELETE FROM user_items WHERE id = '" + ItemID + "' AND user_id = '" + UserId + "'; ");
                                }

                                mRemovedItems.Clear();
                            }

                            dbClient.ExecuteQuery(QueryBuilder.ToString());
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Logging.LogCacheError("FATAL ERROR DURING USER INVENTORY DB UPDATE: " + e.ToString());
            }
            //TimeSpan TimeUsed = DateTime.Now - Start;
            //Logging.LogMessage("Time it took to save users hand: " + TimeUsed.Seconds + " s, " + TimeUsed.Milliseconds + " ms");
        }

        //internal void RunDBUpdate(DatabaseClient dbClient)
        //{
        //    try
        //    {
        //        using (TimedLock.Lock(this))
        //        {
        //            if (mRemovedItems.Count > 0 || mAddedItems.Count > 0)
        //            {
        //                StringBuilder QueryBuilder = new StringBuilder();

        //                if (mAddedItems.Count > 0)
        //                {
        //                    foreach (UserItem Item in mAddedItems.Values)
        //                    {
        //                        dbClient.AddParamWithValue("extra_data" + Item.Id, Item.ExtraData);
        //                        QueryBuilder.Append("DELETE FROM user_items WHERE id = '" + Item.Id + "'; INSERT INTO user_items VALUES ('" + Item.Id + "','" + UserId + "','" + Item.BaseItem + "',@extra_data" + Item.Id + "); ");
        //                    }

        //                    mAddedItems.Clear();
        //                }
        //                if (mRemovedItems.Count > 0)
        //                {
        //                    foreach (UInt32 ItemID in mRemovedItems)
        //                    {
        //                        QueryBuilder.Append("DELETE FROM user_items WHERE id = '" + ItemID + "' AND user_id = '" + UserId + "'; ");
        //                    }

        //                    mRemovedItems.Clear();
        //                }

        //                dbClient.ExecuteQuery(QueryBuilder.ToString());
        //            }
        //        }
        //        Console.WriteLine("Inventory for user: " + GetClient().GetHabbo().Username + " saved.");
        //    }
        //    catch (Exception e)
        //    {
        //        Logging.LogCacheError("FATAL ERROR DURING DB UPDATE: " + e.ToString());
        //    }
        //}
    }
}
