﻿using System;
using System.Collections.Generic;
using System.Data;

using Butterfly.Storage;

namespace Butterfly.HabboHotel.Users.Subscriptions
{
    class SubscriptionManager
    {
        private uint UserId;
        private Dictionary<string, Subscription> Subscriptions;

        internal List<string> SubList
        {
            get
            {
                List<string> List = new List<string>();

                using (TimedLock.Lock(this.Subscriptions.Values))
                {
                    foreach (Subscription Subscription in Subscriptions.Values)
                    {
                        List.Add(Subscription.SubscriptionId);
                    }
                }

                return List;
            }
        }

        internal SubscriptionManager(uint UserId, HabboData UserData)
        {
            this.UserId = UserId;

            Subscriptions = new Dictionary<string, Subscription>();

            DataTable SubscriptionData = UserData.GetSupscriptionData;

            if (SubscriptionData != null)
            {
                foreach (DataRow Row in SubscriptionData.Rows)
                {
                    Subscriptions.Add((string)Row["subscription_id"], new Subscription((string)Row["subscription_id"], (int)Row["timestamp_expire"], (int)Row["timestamp_activated"], (int)Row["usedgifts"], (int)Row["useddays"]));
                }
            }
        }

        //internal void LoadSubscriptions()
        //{

        //}

        internal void Clear()
        {
            Subscriptions.Clear();
        }

        internal Subscription GetSubscription(string SubscriptionId)
        {
            if (Subscriptions.ContainsKey(SubscriptionId))
            {
                return Subscriptions[SubscriptionId];
            }

            return null;
        }

        internal Boolean HasSubscription(string SubscriptionId)
        {
            if (!Subscriptions.ContainsKey(SubscriptionId))
            {
                return false;
            }

            Subscription Sub = Subscriptions[SubscriptionId];

            if (Sub.IsValid())
            {
                return true;
            }

            return false;
        }

        internal void AddOrExtendSubscription(string SubscriptionId, int DurationSeconds)
        {
            SubscriptionId = SubscriptionId.ToLower();

            if (Subscriptions.ContainsKey(SubscriptionId))
            {
                Subscription Sub = Subscriptions[SubscriptionId];
                Sub.ExtendSubscription(DurationSeconds);

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.AddParamWithValue("subcrbr", SubscriptionId);
                    dbClient.ExecuteQuery("UPDATE user_subscriptions SET timestamp_expire = '" + Sub.ExpireTime + "' WHERE user_id = '" + UserId + "' AND subscription_id = @subcrbr LIMIT 1");
                }

                return;
            }

            int TimeCreated = (int)ButterflyEnvironment.GetUnixTimestamp();
            int TimeExpire = ((int)ButterflyEnvironment.GetUnixTimestamp() + DurationSeconds);

            Subscription NewSub = new Subscription(SubscriptionId, TimeExpire, TimeCreated, 0, 1);

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("subcrbr", SubscriptionId);
                dbClient.ExecuteQuery("INSERT INTO user_subscriptions (user_id,subscription_id,timestamp_activated,timestamp_expire) VALUES ('" + UserId + "',@subcrbr,'" + TimeCreated + "','" + TimeExpire + "')");
            }

            Subscriptions.Add(NewSub.SubscriptionId.ToLower(), NewSub);
        }
    }
}
