﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Butterfly.Core;
using Butterfly.HabboHotel.Advertisements;
using Butterfly.HabboHotel.Catalogs;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Navigators;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Users.Badges;
using Butterfly.Storage;

namespace Butterfly.Messages
{
    partial class GameClientMessageHandler
    {
        private void GetAdvertisement()
        {
            RoomAdvertisement Ad = ButterflyEnvironment.GetGame().GetAdvertisementManager().GetRandomRoomAdvertisement();

            Response.Init(258);

            if (Ad == null)
            {
                Response.AppendStringWithBreak("");
                Response.AppendStringWithBreak("");
            }
            else
            {
                Response.AppendStringWithBreak(Ad.AdImage);
                Response.AppendStringWithBreak(Ad.AdLink);

                Ad.OnView();
            }

            SendResponse();
        }

        private void GetPub()
        {
            uint Id = Request.PopWiredUInt();

            RoomData Data = ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Id);

            if (Data == null || Data.Type != "public")
            {
                return;
            }

            Response.Init(453);
            Response.AppendUInt(Data.Id);
            Response.AppendStringWithBreak(Data.CCTs);
            Response.AppendUInt(Data.Id);
            SendResponse();
        }

        private void OpenFlat()
        {
            uint Id = Request.PopWiredUInt();
            string Password = Request.PopFixedString();
            int Junk = Request.PopWiredInt32();

            Logging.WriteLine("Requesting room: [" + Id + "]");

            RoomData Data = ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Id);

            if (Data == null || Data.Type != "private")
            {
                return;
            }

            PrepareRoomForUser(Id, Password);
        }

        private void OpenPub()
        {
            int Junk = Request.PopWiredInt32();
            uint Id = Request.PopWiredUInt();
            int Junk2 = Request.PopWiredInt32();

            RoomData Data = ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Id);

            if (Data == null || Data.Type != "public")
            {
                return;
            }

            PrepareRoomForUser(Data.Id, "");
        }

        private void GetGroupBadges()
        {
            Response.Init(309);
            Response.AppendStringWithBreak("IcIrDs43103s19014d5a1dc291574a508bc80a64663e61a00");
            SendResponse();
        }

        private void GetInventory()
        {
            Session.SendMessage(Session.GetHabbo().GetInventoryComponent().SerializeItemInventory(0)); // Floor Items
            Session.SendMessage(Session.GetHabbo().GetInventoryComponent().SerializeItemInventory(1)); // Wall Items
        }

        private void GetRoomData1()
        {
            if (Session.GetHabbo().LoadingRoom <= 0)
            {
                return;
            }

            Response.Init(297);
            Response.AppendInt32(0);
            SendResponse();
        }

        private void GetRoomData2()
        {
            if (Session.GetHabbo().LoadingRoom <= 0)
            {
                return;
            }

            RoomData Data = ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Session.GetHabbo().LoadingRoom);

            if (Data == null)
            {
                return;
            }

            if (Data.Model == null)
            {
                Session.SendNotif("Sorry, model data is missing from this room and therefore cannot be loaded.");
                Session.SendMessage(new ServerMessage(18));
                ClearRoomLoading();
                return;
            }

            Session.SendMessage(Data.Model.SerializeHeightmap());
            Session.SendMessage(Data.Model.SerializeRelativeHeightmap());
        }

        private void GetRoomData3()
        {
            if (Session.GetHabbo().LoadingRoom <= 0 || !Session.GetHabbo().LoadingChecksPassed)
            {
                return;
            }

            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().LoadingRoom);

            if (Room == null)
            {
                return;
            }

            ClearRoomLoading();

            Response.Init(30);

            if (!string.IsNullOrEmpty(Room.Model.StaticFurniMap))
            {
                Response.AppendStringWithBreak(Room.Model.StaticFurniMap);
            }
            else
            {
                Response.AppendInt32(0);
            }

            SendResponse();

            if (Room.Type == "private")
            {
                Hashtable FloorItems = Room.FloorItems;
                Hashtable WallItems = Room.WallItems;

                Response.Init(32);
                Response.AppendInt32(FloorItems.Count);

                foreach (RoomItem Item in FloorItems.Values)
                {
                    Item.Serialize(Response);
                }

                SendResponse();

                Response.Init(45);
                Response.AppendInt32(WallItems.Count);

                foreach (RoomItem Item in WallItems.Values)
                {
                    Item.Serialize(Response);
                }

                SendResponse();
            }

            Room.AddUserToRoom(Session, Session.GetHabbo().SpectatorMode);

            List<RoomUser> UsersToDisplay = new List<RoomUser>();

            for (int i = 0; i < Room.UserList.Length; i++)
            {
                RoomUser User = Room.UserList[i];
                if (User == null)
                    continue;

                if (User.IsSpectator)
                    continue;

                UsersToDisplay.Add(User);
            }

            Response.Init(28);
            Response.AppendInt32(UsersToDisplay.Count);

            foreach (RoomUser User in UsersToDisplay)
            {
                User.Serialize(Response);
            }

            SendResponse();

            //GXI
            GetResponse().Init(472);
            GetResponse().AppendBoolean(Room.Hidewall);
            SendResponse();

            if (Room.Type == "public")
            {
                Response.Init(471);
                Response.AppendBoolean(false);
                Response.AppendStringWithBreak(Room.ModelName);
                Response.AppendBoolean(false);
                SendResponse();
            }
            else if (Room.Type == "private")
            {
                Response.Init(471);
                Response.AppendBoolean(true);
                Response.AppendUInt(Room.RoomId);

                if (Room.CheckRights(Session, true))
                {
                    Response.AppendBoolean(true);
                }
                else
                {
                    Response.AppendBoolean(false);
                }

                SendResponse();

                // GQhntX]uberEmu PacketloggingDescriptionHQMSCQFJtag1tag2Ika^SMqurbIHH

                Response.Init(454);
                Response.AppendInt32(1);
                Response.AppendUInt(Room.RoomId);
                Response.AppendInt32(0);
                Response.AppendStringWithBreak(Room.Name);
                Response.AppendStringWithBreak(Room.Owner);
                Response.AppendInt32(Room.State);
                Response.AppendInt32(0);
                Response.AppendInt32(25);
                Response.AppendStringWithBreak(Room.Description);
                Response.AppendInt32(0);
                Response.AppendInt32(1);
                Response.AppendInt32(8228);
                Response.AppendInt32(Room.Category);
                Response.AppendStringWithBreak("");
                Response.AppendInt32(Room.TagCount);

                foreach (string Tag in Room.Tags)
                {
                    Response.AppendStringWithBreak(Tag);
                }

                Room.Icon.Serialize(Response);
                Response.AppendBoolean(false);
                SendResponse();
            }

            ServerMessage Updates = Room.SerializeStatusUpdates(true);

            if (Updates != null)
            {
                Session.SendMessage(Updates);
            }

            for (int i = 0; i < Room.UserList.Length; i++)
            {
                RoomUser User = Room.UserList[i];
                if (User == null)
                    continue;

                if (User.IsSpectator)
                {
                    continue;
                }

                if (User.IsDancing)
                {
                    Response.Init(480);
                    Response.AppendInt32(User.VirtualId);
                    Response.AppendInt32(User.DanceId);
                    SendResponse();
                }

                if (User.IsAsleep)
                {
                    Response.Init(486);
                    Response.AppendInt32(User.VirtualId);
                    Response.AppendBoolean(true);
                    SendResponse();
                }

                if (User.CarryItemID > 0 && User.CarryTimer > 0)
                {
                    Response.Init(482);
                    Response.AppendInt32(User.VirtualId);
                    Response.AppendInt32(User.CarryTimer);
                    SendResponse();
                }

                if (!User.IsBot)
                {
                    try
                    {
                        if (User.GetClient().GetHabbo() != null && User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent() != null && User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().CurrentEffect >= 1)
                        {
                            Response.Init(485);
                            Response.AppendInt32(User.VirtualId);
                            Response.AppendInt32(User.GetClient().GetHabbo().GetAvatarEffectsInventoryComponent().CurrentEffect);
                            SendResponse();
                        }
                    }
                    catch { }
                }
            }
        }

        internal void PrepareRoomForUser(uint Id, string Password)
        {
            ClearRoomLoading();

            if (ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(Id) == null)
            {
                return;
            }

            if (Session.GetHabbo().InRoom)
            {
                Room OldRoom = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

                if (OldRoom != null)
                {
                    OldRoom.RemoveUserFromRoom(Session, false, false);
                }
            }

            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().LoadRoom(Id);

            if (Room == null)
                return;

            Session.GetHabbo().LoadingRoom = Id;

            if (Room.UserIsBanned(Session.GetHabbo().Id))
            {
                if (Room.HasBanExpired(Session.GetHabbo().Id))
                {
                    Room.RemoveBan(Session.GetHabbo().Id);
                }
                else
                {
                    // C`PA
                    Response.Init(224);
                    Response.AppendInt32(4);
                    SendResponse();

                    Response.Init(18);
                    SendResponse();

                    return;
                }
            }

            if (Room.UsersNow >= Room.UsersMax)
            {
                if (!ButterflyEnvironment.GetGame().GetRoleManager().RankHasRight(Session.GetHabbo().Rank, "fuse_enter_full_rooms"))
                {
                    Response.Init(224);
                    Response.AppendInt32(1);
                    SendResponse();

                    Response.Init(18);
                    SendResponse();

                    return;
                }
            }

            if (Room.Type == "public")
            {
                if (Room.State > 0 && !Session.GetHabbo().HasFuse("fuse_mod"))
                {
                    Session.SendNotif("This internal room is accessible to Uber staff only.");

                    Response.Init(18);
                    SendResponse();

                    return;
                }

                Response.Init(166);
                Response.AppendStringWithBreak("/client/public/" + Room.ModelName + "/0");
                SendResponse();
            }
            else if (Room.Type == "private")
            {
                Response.Init(19);
                SendResponse();

                if (!Session.GetHabbo().HasFuse("fuse_enter_any_room") && !Room.CheckRights(Session, true) && !Session.GetHabbo().IsTeleporting)
                {
                    if (Room.State == 1)
                    {
                        if (Room.UserCount == 0)
                        {
                            Response.Init(131);
                            SendResponse();
                        }
                        else
                        {
                            Response.Init(91);
                            Response.AppendStringWithBreak("");
                            SendResponse();

                            ServerMessage RingMessage = new ServerMessage(91);
                            RingMessage.AppendStringWithBreak(Session.GetHabbo().Username);
                            Room.SendMessageToUsersWithRights(RingMessage);
                        }

                        return; 
                    }
                    else if (Room.State == 2)
                    {
                        if (Password.ToLower() != Room.Password.ToLower())
                        {
                            Response.Init(33);
                            Response.AppendInt32(-100002);
                            SendResponse();

                            Response.Init(18);
                            SendResponse();

                            return;
                        }
                    }
                }

                Response.Init(166);
                Response.AppendStringWithBreak("/client/private/" + Room.RoomId + "/id");
                SendResponse();
            }

            Session.GetHabbo().LoadingChecksPassed = true;

            LoadRoomForUser();
        }

        private void ReqLoadRoomForUser()
        {
            LoadRoomForUser();
        }

        internal void LoadRoomForUser()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().LoadingRoom);

            if (Room == null || !Session.GetHabbo().LoadingChecksPassed)
            {
                return;
            }

            // todo: Room.SerializeGroupBadges()
            Response.Init(309);
            Response.AppendStringWithBreak("IcIrDs43103s19014d5a1dc291574a508bc80a64663e61a00");
            SendResponse();

            Response.Init(69);
            Response.AppendStringWithBreak(Room.ModelName);
            Response.AppendUInt(Room.RoomId);
            SendResponse();

            if (Session.GetHabbo().SpectatorMode)
            {
                Response.Init(254);
                SendResponse();
            }

            if (Room.Type == "private")
            {
                if (Room.Wallpaper != "0.0")
                {
                    Response.Init(46);
                    Response.AppendStringWithBreak("wallpaper");
                    Response.AppendStringWithBreak(Room.Wallpaper);
                    SendResponse();
                }

                if (Room.Floor != "0.0")
                {
                    Response.Init(46);
                    Response.AppendStringWithBreak("floor");
                    Response.AppendStringWithBreak(Room.Floor);
                    SendResponse();
                }

                Response.Init(46);
                Response.AppendStringWithBreak("landscape");
                Response.AppendStringWithBreak(Room.Landscape.Replace(',','.'));
                SendResponse();

                if (Room.CheckRights(Session, true))
                {
                    Response.Init(42);
                    SendResponse();

                    Response.Init(47);
                    SendResponse();
                }
                else if (Room.CheckRights(Session))
                {
                    Response.Init(42);
                    SendResponse();
                }

                Response.Init(345);

                if (Session.GetHabbo().RatedRooms.Contains(Room.RoomId) || Room.CheckRights(Session, true))
                {
                    Response.AppendInt32(Room.Score);
                }
                else
                {
                    Response.AppendInt32(-1);
                }

                SendResponse();

                if (Room.HasOngoingEvent)
                {
                    Session.SendMessage(Room.Event.Serialize(Session));
                }
                else
                {
                    Response.Init(370);
                    Response.AppendStringWithBreak("-1");
                    SendResponse();
                }
            }
        }

        internal void ClearRoomLoading()
        {
            Session.GetHabbo().LoadingRoom = 0;
            Session.GetHabbo().LoadingChecksPassed = false;
        }

        private void Talk()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            User.Chat(Session, ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString()), false);
        }

        private void Shout()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            User.Chat(Session, ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString()), true);
        }

        private void Whisper()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            if (Session.GetHabbo().Muted)
            {
                Session.SendNotif("You are muted.");
                return;
            }

            if (Room.RoomMuted)
                return;

            string Params = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            string ToUser = Params.Split(' ')[0];
            string Message = Params.Substring(ToUser.Length + 1);

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);
            RoomUser User2 = Room.GetRoomUserByHabbo(ToUser);

            ServerMessage TellMsg = new ServerMessage();
            TellMsg.Init(25);
            TellMsg.AppendInt32(User.VirtualId);
            TellMsg.AppendStringWithBreak(Message);
            TellMsg.AppendBoolean(false);

            if (User != null && !User.IsBot)
            {
                User.GetClient().SendMessage(TellMsg);
            }

            User.Unidle();

            if (User2 != null && !User2.IsBot)
            {
                if (!User2.GetClient().GetHabbo().MutedUsers.Contains(Session.GetHabbo().Id))
                {
                    User2.GetClient().SendMessage(TellMsg);
                }

                //using (DatabaseClient dbClient = UberEnvironment.GetDatabase().GetClient())
                //{
                //    dbClient.AddParamWithValue("message", "<Whisper to " + User2.GetClient().GetHabbo().Username + ">: " + Message);
                //    dbClient.AddParamWithValue("usrname", Session.GetHabbo().Username);
                //    dbClient.ExecuteQuery("INSERT INTO chatlogs (user_id,room_id,hour,minute,timestamp,message,user_name,full_date) VALUES ('" + Session.GetHabbo().Id + "','" + Room.RoomId + "','" + DateTime.Now.Hour + "','" + DateTime.Now.Minute + "','" + UberEnvironment.GetUnixTimestamp() + "',@message,@usrname,'" + DateTime.Now.ToLongDateString() + "')");
                //} Nanananaaa dunt phunk with mah haart
                //UberEnvironment.GetGame().GetRoomManager().AddChatmessage(Session.GetHabbo().Username, Session.GetHabbo().Id, Room.RoomId, "<Whisper to " + User2.GetClient().GetHabbo().Username + ">: " + Message);
            }
        }

        private void Move()
        {
            Room Room = Session.GetHabbo().CurrentRoom;

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null || !User.CanWalk)
            {
                return;
            }

            int MoveX = Request.PopWiredInt32();
            int MoveY = Request.PopWiredInt32();

            if (MoveX == User.X && MoveY == User.Y)
            {
                return;
            }

            User.MoveTo(MoveX, MoveY);
        }

        private void CanCreateRoom()
        {
            Response.Init(512);
            Response.AppendBoolean(false); // true = show error with number below
            Response.AppendInt32(99999);
            SendResponse();

            // todo: room limit
        }

        private void CreateRoom()
        {
            string RoomName = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            string ModelName = Request.PopFixedString();
            string RoomState = Request.PopFixedString(); // unused, room open by default on creation. may be added in later build of Habbo?

            RoomData NewRoom = ButterflyEnvironment.GetGame().GetRoomManager().CreateRoom(Session, RoomName, ModelName);

            if (NewRoom != null)
            {
                Response.Init(59);
                Response.AppendUInt(NewRoom.Id);
                Response.AppendStringWithBreak(NewRoom.Name);
                SendResponse();
            }
        }

        private void GetRoomEditData()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            GetResponse().Init(465);
            GetResponse().AppendUInt(Room.RoomId);
            GetResponse().AppendStringWithBreak(Room.Name);
            GetResponse().AppendStringWithBreak(Room.Description);
            GetResponse().AppendInt32(Room.State);
            GetResponse().AppendInt32(Room.Category);
            GetResponse().AppendInt32(Room.UsersMax);
            GetResponse().AppendInt32(25);
            GetResponse().AppendInt32(Room.TagCount);

            foreach (string Tag in Room.Tags)
            {
                GetResponse().AppendStringWithBreak(Tag);
            }

            GetResponse().AppendInt32(Room.UsersWithRights.Count); // users /w rights count

            foreach (uint UserId in Room.UsersWithRights)
            {
                GetResponse().AppendUInt(UserId);
                GetResponse().AppendStringWithBreak(ButterflyEnvironment.GetGame().GetClientManager().GetNameById(UserId));
            }

            GetResponse().AppendInt32(Room.UsersWithRights.Count); // users /w rights count

            GetResponse().AppendBoolean(Room.AllowPets); // allows pets in room - pet system lacking, so always off
            GetResponse().AppendBoolean(Room.AllowPetsEating); // allows pets to eat your food - pet system lacking, so always off
            GetResponse().AppendBoolean(Room.AllowWalkthrough);
            GetResponse().AppendBoolean(Room.Hidewall);

            SendResponse();
        }

        private void SaveRoomIcon()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            int Junk = Request.PopWiredInt32(); // always 3

            Dictionary<int, int> Items = new Dictionary<int, int>();

            int Background = Request.PopWiredInt32();
            int TopLayer = Request.PopWiredInt32();
            int AmountOfItems = Request.PopWiredInt32();

            for (int i = 0; i < AmountOfItems; i++)
            {
                int Pos = Request.PopWiredInt32();
                int Item = Request.PopWiredInt32();

                if (Pos < 0 || Pos > 10)
                {
                    return;
                }

                if (Item < 1 || Item > 27)
                {
                    return;
                }

                if (Items.ContainsKey(Pos))
                {
                    return;
                }

                Items.Add(Pos, Item);
            }

            if (Background < 1 || Background > 24)
            {
                return;
            }

            if (TopLayer < 0 || TopLayer > 11)
            {
                return;
            }

            StringBuilder FormattedItems = new StringBuilder();
            int j = 0;

            foreach (KeyValuePair<int, int> Item in Items)
            {
                if (j > 0)
                {
                    FormattedItems.Append("|");
                }

                FormattedItems.Append(Item.Key + "," + Item.Value);

                j++;
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE rooms SET icon_bg = '" + Background + "', icon_fg = '" + TopLayer + "', icon_items = '" + FormattedItems.ToString() + "' WHERE id = '" + Room.RoomId + "' LIMIT 1");
            }

            Room.Icon = new RoomIcon(Background, TopLayer, Items);

            Response.Init(457);
            Response.AppendUInt(Room.RoomId);
            Response.AppendBoolean(true);
            SendResponse();

            Response.Init(456);
            Response.AppendUInt(Room.RoomId);
            SendResponse();

            RoomData Data = Room.GetRoomData;

            Response.Init(454);
            Response.AppendBoolean(false);
            Data.Serialize(Response, false);
            SendResponse();
        }

        private void SaveRoomData()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            int Id = Request.PopWiredInt32();
            string Name = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            string Description = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            int State = Request.PopWiredInt32();
            string Password = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            int MaxUsers = Request.PopWiredInt32();
            int CategoryId = Request.PopWiredInt32();
            int TagCount = Request.PopWiredInt32();

            List<string> Tags = new List<string>();
            StringBuilder formattedTags = new StringBuilder();

            for (int i = 0; i < TagCount; i++)
            {
                if (i > 0)
                {
                    formattedTags.Append(",");
                }

                string tag = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString().ToLower());

                Tags.Add(tag);
                formattedTags.Append(tag);
            }

            int AllowPets = 0;
            int AllowPetsEat = 0;
            int AllowWalkthrough = 0;
            int Hidewall = 0;

            string _AllowPets = Request.PlainReadBytes(1)[0].ToString();
            Request.AdvancePointer(1);

            string _AllowPetsEat = Request.PlainReadBytes(1)[0].ToString();
            Request.AdvancePointer(1);

            string _AllowWalkthrough = Request.PlainReadBytes(1)[0].ToString();
            Request.AdvancePointer(1);

            string _Hidewall = Request.PlainReadBytes(1)[0].ToString();
            Request.AdvancePointer(1);

            if (Name.Length < 1)
            {
                return;
            }

            if (State < 0 || State > 2)
            {
                return;
            }

            if (MaxUsers != 10 && MaxUsers != 15 && MaxUsers != 20 && MaxUsers != 25)
            {
                return;
            }

            FlatCat FlatCat = ButterflyEnvironment.GetGame().GetNavigator().GetFlatCat(CategoryId);

            if (FlatCat == null)
            {
                return;
            }

            if (FlatCat.MinRank > Session.GetHabbo().Rank)
            {
                Session.SendNotif("You are not allowed to use this category. Your room has been moved to no category instead.");
                CategoryId = 0;
            }

            if (TagCount > 2)
            {
                return;
            }

            if (State < 0 || State > 2)
            {
                return;
            }

            if (_AllowPets == "65")
            {
                AllowPets = 1;
                Room.AllowPets = true;
            }
            else
            {
                Room.AllowPets = false;
            }

            if (_AllowPetsEat == "65")
            {
                AllowPetsEat = 1;
                Room.AllowPetsEating = true;
            }
            else
            {
                Room.AllowPetsEating = false;
            }

            if (_AllowWalkthrough == "65")
            {
                AllowWalkthrough = 1;
                Room.AllowWalkthrough = true;
            }
            else
            {
                Room.AllowWalkthrough = false;
            }
            Room.GenerateMaps();

            if (_Hidewall == "65")
            {
                Hidewall = 1;
                Room.Hidewall = true;
            }
            else
            {
                Room.Hidewall = false;
            }

            Room.Name = Name;
            Room.State = State;
            Room.Description = Description;
            Room.Category = CategoryId;
            Room.Password = Password;
            Room.Tags = Tags;
            Room.UsersMax = MaxUsers;

            string formattedState = "open";

            if (Room.State == 1)
            {
                formattedState = "locked";
            }
            else if (Room.State > 1)
            {
                formattedState = "password";
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("caption", Room.Name);
                dbClient.AddParamWithValue("description", Room.Description);
                dbClient.AddParamWithValue("password", Room.Password);
                dbClient.AddParamWithValue("tags", formattedTags.ToString());
                dbClient.ExecuteQuery("UPDATE rooms SET caption = @caption, description = @description, password = @password, category = '" + CategoryId + "', state = '" + formattedState + "', tags = @tags, users_max = '" + MaxUsers + "', allow_pets = '" + AllowPets + "', allow_pets_eat = '" + AllowPetsEat + "', allow_walkthrough = '" + AllowWalkthrough + "', allow_hidewall = '" + Hidewall + "' WHERE id = '" + Room.RoomId + "' LIMIT 1;");
            }

            GetResponse().Init(467);
            GetResponse().AppendUInt(Room.RoomId);
            SendResponse();

            GetResponse().Init(456);
            GetResponse().AppendUInt(Room.RoomId);
            SendResponse();

            GetResponse().Init(472);
            GetResponse().AppendBoolean(Room.Hidewall);
            ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId).SendMessage(Response);

            RoomData Data = Room.GetRoomData;

            GetResponse().Init(454);
            GetResponse().AppendBoolean(false);
            Data.Serialize(GetResponse(), false);

            SendResponse();
        }

        private void GiveRights()
        {
            uint UserId = Request.PopWiredUInt();

            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);
            RoomUser RoomUser = Room.GetRoomUserByHabbo(UserId);

            if (Room == null || !Room.CheckRights(Session, true) || RoomUser == null || RoomUser.IsBot)
            {
                return;
            }

            if (Room.UsersWithRights.Contains(UserId))
            {
                Session.SendNotif("User already has rights! (There appears to be a bug with the rights button, we are looking into it - for now rely on 'Advanced settings')");
                return;
            }

            Room.UsersWithRights.Add(UserId);

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("INSERT INTO room_rights (room_id,user_id) VALUES ('" + Room.RoomId + "','" + UserId + "')");
            }

            Response.Init(510);
            Response.AppendUInt(Room.RoomId);
            Response.AppendUInt(UserId);
            Response.AppendStringWithBreak(RoomUser.GetClient().GetHabbo().Username);
            SendResponse();

            RoomUser.AddStatus("flatcrtl", "");
            RoomUser.UpdateNeeded = true;

            RoomUser.GetClient().SendMessage(new ServerMessage(42));

            // G~hntX]h_u@UMeth0d9
        }

        private void TakeRights()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            StringBuilder DeleteParams = new StringBuilder();

            int Amount = Request.PopWiredInt32();

            for (int i = 0; i < Amount; i++)
            {
                if (i > 0)
                {
                    DeleteParams.Append(" OR ");
                }

                uint UserId = Request.PopWiredUInt();
                Room.UsersWithRights.Remove(UserId);
                DeleteParams.Append("room_id = '" + Room.RoomId + "' AND user_id = '" + UserId + "'");

                RoomUser User = Room.GetRoomUserByHabbo(UserId);

                if (User != null && !User.IsBot)
                {
                    User.GetClient().SendMessage(new ServerMessage(43));
                }

                // GhntX]hqu@U
                Response.Init(511);
                Response.AppendUInt(Room.RoomId);
                Response.AppendUInt(UserId);
                SendResponse();
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("DELETE FROM room_rights WHERE " + DeleteParams.ToString());
            }
        }

        private void TakeAllRights()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            foreach (uint UserId in Room.UsersWithRights)
            {
                RoomUser User = Room.GetRoomUserByHabbo(UserId);

                if (User != null && !User.IsBot)
                {
                    User.GetClient().SendMessage(new ServerMessage(43));
                }

                Response.Init(511);
                Response.AppendUInt(Room.RoomId);
                Response.AppendUInt(UserId);
                SendResponse();
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("DELETE FROM room_rights WHERE room_id = '" + Room.RoomId + "'");
            }

            Room.UsersWithRights.Clear();
        }

        private void KickUser()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            if (!Room.CheckRights(Session))
            {
                return; // insufficient permissions
            }

            uint UserId = Request.PopWiredUInt();
            RoomUser User = Room.GetRoomUserByHabbo(UserId);

            if (User == null || User.IsBot)
            {
                return;
            }

            if (Room.CheckRights(User.GetClient(), true) || User.GetClient().GetHabbo().HasFuse("fuse_mod"))
            {
                return; // can't kick room owner or mods!
            }

            Room.RemoveUserFromRoom(User.GetClient(), true, true);
        }

        private void BanUser()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return; // insufficient permissions
            }

            uint UserId = Request.PopWiredUInt();
            RoomUser User = Room.GetRoomUserByHabbo(UserId);

            if (User == null || User.IsBot)
            {
                return;
            }

            if (User.GetClient().GetHabbo().HasFuse("fuse_mod"))
            {
                return;
            }

            Room.AddBan(UserId);
            Room.RemoveUserFromRoom(User.GetClient(), true, true);
        }

        private void SetHomeRoom()
        {
            uint RoomId = Request.PopWiredUInt();
            RoomData Data = ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId);

            if (RoomId != 0)
            {
                if (Data == null || Data.Owner.ToLower() != Session.GetHabbo().Username.ToLower())
                {
                    return;
                }
            }

            Session.GetHabbo().HomeRoom = RoomId;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE users SET home_room = '" + RoomId + "' WHERE id = '" + Session.GetHabbo().Id + "' LIMIT 1;");
            }

            Response.Init(455);
            Response.AppendUInt(RoomId);
            SendResponse();
        }

        private void DeleteRoom()
        {
            uint RoomId = Request.PopWiredUInt();

            
            //TargetRoom = Session.GetHabbo().CurrentRoom; ;
            Room TargetRoom = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(RoomId);
            if (TargetRoom == null)
                return;
            else if (TargetRoom.Owner != Session.GetHabbo().Username || Session.GetHabbo().Rank != 7)
                return;
            else
            {
                ButterflyEnvironment.GetGame().GetRoomManager().UnloadRoom(TargetRoom);

                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    dbClient.ExecuteQuery("DELETE FROM rooms WHERE id = '" + RoomId + "' LIMIT 1");
                    dbClient.ExecuteQuery("DELETE FROM user_favorites WHERE room_id = '" + RoomId + "'");
                    dbClient.ExecuteQuery("DELETE FROM room_items WHERE room_id = '" + RoomId + "'");
                    dbClient.ExecuteQuery("DELETE FROM room_rights WHERE room_id = '" + RoomId + "'");
                    dbClient.ExecuteQuery("UPDATE users SET home_room = '0' WHERE home_room = '" + RoomId + "'");
                    dbClient.ExecuteQuery("DELETE FROM room_items WHERE room_id = '" + RoomId + "'");

                    Session.GetHabbo().UpdateRooms(dbClient);
                }

                ButterflyEnvironment.GetGame().GetRoomManager().CheckForDryCacheConflicts(RoomId);

                //RoomData Data = UberEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId);

                //if (Data == null || Data.Owner.ToLower() != Session.GetHabbo().Username.ToLower())
                //{
                //    return;
                //}



                //Room Room = UberEnvironment.GetGame().GetRoomManager().GetRoom(Data.Id);

                //if (Room != null)
                //{

                //    for (int i = 0; i < Room.UserList.Length; i++)
                //    {
                //        RoomUser User = Room.UserList[i];
                //        if (User == null)
                //            continue;

                //        if (User.IsBot)
                //        {
                //            continue;
                //        }

                //        User.GetClient().SendMessage(new ServerMessage(18));
                //        User.GetClient().GetHabbo().OnLeaveRoom();
                //    }

                //    UberEnvironment.GetGame().GetRoomManager().UnloadRoom(Room);
                //}

                //Response.Init(101);
                //SendResponse();

                //Session.SendMessage(UberEnvironment.GetGame().GetNavigator().GetDynamicNavigatorPacket(Session, -3));
            }
        }

        private void LookAt()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            User.Unidle();

            int X = Request.PopWiredInt32();
            int Y = Request.PopWiredInt32();

            if (X == User.X && Y == User.Y)
            {
                return;
            }

            int Rot = Rotation.Calculate(User.X, User.Y, X, Y);

            User.SetRot(Rot);
        }

        private void StartTyping()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            ServerMessage Message = new ServerMessage(361);
            Message.AppendInt32(User.VirtualId);
            Message.AppendBoolean(true);
            Room.SendMessage(Message);
        }

        private void StopTyping()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            ServerMessage Message = new ServerMessage(361);
            Message.AppendInt32(User.VirtualId);
            Message.AppendBoolean(false);
            Room.SendMessage(Message);
        }

        private void IgnoreUser()
        {
            /*Room Room = UberEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoom);

            if (Room == null)
            {
                return;
            }

            uint Id = Request.PopWiredUInt();

            if (Session.GetHabbo().MutedUsers.Contains(Id))
            {
                return;
            }

            Session.GetHabbo().MutedUsers.Add(Id);

            Response.Init(419);
            Response.AppendInt32(1);
            SendResponse();*/
        }

        private void UnignoreUser()
        {
            /*Room Room = UberEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoom);

            if (Room == null)
            {
                return;
            }

            uint Id = Request.PopWiredUInt();

            if (!Session.GetHabbo().MutedUsers.Contains(Id))
            {
                return;
            }

            Session.GetHabbo().MutedUsers.Remove(Id);

            Response.Init(419);
            Response.AppendInt32(3);
            SendResponse();*/
        }

        private void CanCreateRoomEvent()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            Boolean Allow = true;
            int ErrorCode = 0;

            if (Room.State != 0)
            {
                Allow = false;
                ErrorCode = 3;
            }

            Response.Init(367);
            Response.AppendBoolean(Allow);
            Response.AppendInt32(ErrorCode);
            SendResponse();
        }

        private void StartEvent()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.Event != null || Room.State != 0)
            {
                return;
            }

            int category = Request.PopWiredInt32();
            string name = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            string descr = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            int tagCount = Request.PopWiredInt32();

            Room.Event = new RoomEvent(Room.RoomId, name, descr, category, null);
            Room.Event.Tags = new List<string>();

            for (int i = 0; i < tagCount; i++)
            {
                Room.Event.Tags.Add(ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString()));
            }

            Room.SendMessage(Room.Event.Serialize(Session));
        }

        private void StopEvent()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.Event == null)
            {
                return;
            }

            //Room.Event = null;

            ServerMessage Message = new ServerMessage(370);
            Message.AppendStringWithBreak("-1");
            Room.SendMessage(Message);
        }

        private void EditEvent()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.Event == null)
            {
                return;
            }

            int category = Request.PopWiredInt32();
            string name = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            string descr = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());
            int tagCount = Request.PopWiredInt32();

            Room.Event.Category = category;
            Room.Event.Name = name;
            Room.Event.Description = descr;
            Room.Event.Tags = new List<string>();

            for (int i = 0; i < tagCount; i++)
            {
                Room.Event.Tags.Add(ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString()));
            }

            Room.SendMessage(Room.Event.Serialize(Session));
        }

        private void Wave()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            User.Unidle();

            User.DanceId = 0;

            ServerMessage Message = new ServerMessage(481);
            Message.AppendInt32(User.VirtualId);
            Room.SendMessage(Message);
        }

        private void GetUserTags()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Request.PopWiredUInt());

            if (User == null || User.IsBot)
            {
                return;
            }

            Response.Init(350);
            Response.AppendUInt(User.GetClient().GetHabbo().Id);
            Response.AppendInt32(User.GetClient().GetHabbo().Tags.Count);

            using (TimedLock.Lock(User.GetClient().GetHabbo().Tags))
            {
                foreach (string Tag in User.GetClient().GetHabbo().Tags)
                {
                    Response.AppendStringWithBreak(Tag);
                }
            }

            SendResponse();
        }

        private void GetUserBadges()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Request.PopWiredUInt());

            if (User == null || User.IsBot)
                return;
            if (User.GetClient() == null)
                return;

            // CdjUzYZJIACH_RespectEarned1JACH_EmailVerification1E^jUzYZH

            Response.Init(228);
            Response.AppendUInt(User.GetClient().GetHabbo().Id);
            Response.AppendInt32(User.GetClient().GetHabbo().GetBadgeComponent().EquippedCount);

            using (TimedLock.Lock(User.GetClient().GetHabbo().GetBadgeComponent().BadgeList))
            {
                foreach (Badge Badge in User.GetClient().GetHabbo().GetBadgeComponent().BadgeList)
                {
                    if (Badge.Slot <= 0)
                    {
                        continue;
                    }

                    Response.AppendInt32(Badge.Slot);
                    Response.AppendStringWithBreak(Badge.Code);
                }
            }

            SendResponse();
        }

        private void RateRoom()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || Session.GetHabbo().RatedRooms.Contains(Room.RoomId) || Room.CheckRights(Session, true))
            {
                return;
            }

            int Rating = Request.PopWiredInt32();

            switch (Rating)
            {
                case -1:

                    Room.Score--;
                    break;

                case 1:

                    Room.Score++;
                    break;

                default:

                    return;
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE rooms SET score = '" + Room.Score + "' WHERE id = '" + Room.RoomId + "' LIMIT 1");
            }

            Session.GetHabbo().RatedRooms.Add(Room.RoomId);

            Response.Init(345);
            Response.AppendInt32(Room.Score);
            SendResponse();
        }

        private void Dance()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            User.Unidle();

            int DanceId = Request.PopWiredInt32();

            if (DanceId < 0 || DanceId > 4 || (!Session.GetHabbo().HasFuse("fuse_use_club_dance") && DanceId > 1))
            {
                DanceId = 0;
            }

            if (DanceId > 0 && User.CarryItemID > 0)
            {
                User.CarryItem(0);
            }

            User.DanceId = DanceId;

            ServerMessage DanceMessage = new ServerMessage(480);
            DanceMessage.AppendInt32(User.VirtualId);
            DanceMessage.AppendInt32(DanceId);
            Room.SendMessage(DanceMessage);
        }

        private void AnswerDoorbell()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session))
            {
                return;
            }

            string Name = Request.PopFixedString();
            byte[] Result = Request.ReadBytes(1);

            GameClient Client = ButterflyEnvironment.GetGame().GetClientManager().GetClientByHabbo(Name);

            if (Client == null)
            {
                return;
            }

            if (Result[0] == Convert.ToByte(65))
            {
                Client.GetHabbo().LoadingChecksPassed = true;

                Client.GetMessageHandler().Response.Init(41);
                Client.GetMessageHandler().SendResponse();
            }
            else
            {
                Client.GetMessageHandler().Response.Init(131);
                Client.GetMessageHandler().SendResponse();
            }
        }

        private void ApplyRoomEffect()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            UserItem Item = Session.GetHabbo().GetInventoryComponent().GetItem(Request.PopWiredUInt());

            if (Item == null)
            {
                return;
            }

            string type = "floor";

            if (Item.GetBaseItem().Name.ToLower().Contains("wallpaper"))
            {
                type = "wallpaper";
            }
            else if (Item.GetBaseItem().Name.ToLower().Contains("landscape"))
            {
                type = "landscape";
            }

            switch (type)
            {
                case "floor":

                    Room.Floor = Item.ExtraData;
                    break;

                case "wallpaper":

                    Room.Wallpaper = Item.ExtraData;
                    break;

                case "landscape":

                    Room.Landscape = Item.ExtraData;
                    break;
            }

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("extradata", Item.ExtraData);
                dbClient.ExecuteQuery("UPDATE rooms SET " + type + " = @extradata WHERE id = '" + Room.RoomId + "' LIMIT 1");
            }

            Session.GetHabbo().GetInventoryComponent().RemoveItem(Item.Id, false);

            ServerMessage Message = new ServerMessage(46);
            Message.AppendStringWithBreak(type);
            Message.AppendStringWithBreak(Item.ExtraData.Replace(',','.'));
            Room.SendMessage(Message);
        }

        private void PlaceItem()
        {
            // AZ@J16 10 10 0

            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session))
            {
                return;
            }

            string PlacementData = Request.PopFixedString();
            string[] DataBits = PlacementData.Split(' ');
            uint ItemId = uint.Parse(DataBits[0]);

            UserItem Item = Session.GetHabbo().GetInventoryComponent().GetItem(ItemId);

            if (Item == null)
            {
                return;
            }
            //bool UpdateNeeded = false;

            switch (Item.GetBaseItem().InteractionType)
            {
                case Butterfly_Emulator.HabboHotel.Items.InteractionType.dimmer:
                    {
                        if (Room.ItemCountByType("dimmer") >= 1)
                            Session.SendNotif("You can only have one moodlight in a room.");
                        break;
                    }
                //case "teleport":
                //    {
                //        UpdateNeeded = true;
                //        break;
                //    }

                //case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet0:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
                //case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet1:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
                //case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet3:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
                //case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet4:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
                //case case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet5:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
                //case case Butterfly_Emulator.HabboHotel.Items.InteractionType.pet6:
                //    {
                //        Pet ThePetWeWantToPlace = Session.GetHabbo().GetInventoryComponent().GetPet(ItemId);
                //        if (ThePetWeWantToPlace == null)
                //            return;

                //        List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
                //        List<BotResponse> BotResponse = new List<Uber.HabboHotel.RoomBots.BotResponse>();
                //        Room.DeployBot(new HabboHotel.RoomBots.RoomBot(ThePetWeWantToPlace.PetId, Room.RoomId, "pet", "freeroam", ThePetWeWantToPlace.Name, "", ThePetWeWantToPlace.Look, ThePetWeWantToPlace.X,
                //            ThePetWeWantToPlace.Y, (int)ThePetWeWantToPlace.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), ThePetWeWantToPlace);

                //        //Room.InitPets
                //        //DeployBot(new RoomBot(Pet.PetId, RoomId, "pet", "freeroam", Pet.Name, "", Pet.Look, Pet.X, Pet.Y, (int)Pet.Z, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);
                //        break;
                //    }
            }

            // Wall Item
            if (DataBits[1].StartsWith(":"))
            {
                string WallPos = Room.WallPositionCheck(":" + PlacementData.Split(':')[1]);

                if (WallPos == null)
                {
                    Response.Init(516);
                    Response.AppendInt32(11);
                    SendResponse();

                    return;
                }

                RoomItem RoomItem = new RoomItem(Item.Id, Room.RoomId, Item.BaseItem, Item.ExtraData, 0, 0, 0.0, 0, WallPos, Room);

                if (Room.SetWallItem(Session, RoomItem))
                {
                    Session.GetHabbo().GetInventoryComponent().RemoveItem(ItemId, true);
                }
            }
            // Floor Item
            else
            {
                int X = int.Parse(DataBits[1]);
                int Y = int.Parse(DataBits[2]);
                int Rot = int.Parse(DataBits[3]);

                RoomItem RoomItem = new RoomItem(Item.Id, Room.RoomId, Item.BaseItem, Item.ExtraData, 0, 0, 0, 0, "", Room);

                if (Room.SetFloorItem(Session, RoomItem, X, Y, Rot, true, false))
                {
                    Session.GetHabbo().GetInventoryComponent().RemoveItem(ItemId, true);
                }
            }
            //if (UpdateNeeded)
            //    Room.SaveFurniture();
        }

        private void TakeItem()
        {
            int junk = Request.PopWiredInt32();

            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null)
            {
                return;
            }

            if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.postit)
                return;

            Room.RemoveFurniture(Session, Item.Id);
            Session.GetHabbo().GetInventoryComponent().AddItem(Item.Id, Item.BaseItem, Item.ExtraData);
            Session.GetHabbo().GetInventoryComponent().UpdateItems(false);
        }

        private void MoveItem()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session))
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null)
            {
                return;
            }

            int x = Request.PopWiredInt32();
            int y = Request.PopWiredInt32();
            int Rotation = Request.PopWiredInt32();
            int Junk = Request.PopWiredInt32();

            bool UpdateNeeded = false;

            if (Item.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.teleport)
                UpdateNeeded = true;

            Room.SetFloorItem(Session, Item, x, y, Rotation, false, false);

            if (UpdateNeeded)
            {
                using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
                {
                    Room.SaveFurniture(dbClient);
                }
            }
        }

        private void TriggerItem()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null)
            {
                return;
            }

            Boolean hasRights = false;

            if (Room.CheckRights(Session))
            {
                hasRights = true;
            }

            Item.Interactor.OnTrigger(Session, Item, Request.PopWiredInt32(), hasRights);
        }

        private void TriggerItemDiceSpecial()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null)
            {
                return;
            }

            Boolean hasRights = false;

            if (Room.CheckRights(Session))
            {
                hasRights = true;
            }

            Item.Interactor.OnTrigger(Session, Item, -1, hasRights);
        }

        private void OpenPostit()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null || Item.GetBaseItem().InteractionType != Butterfly_Emulator.HabboHotel.Items.InteractionType.postit)
            {
                return;
            }

            // @p181855059CFF9C stickynotemsg
            Response.Init(48);
            Response.AppendStringWithBreak(Item.Id.ToString());
            Response.AppendStringWithBreak(Item.ExtraData);
            SendResponse();
        }

        private void SavePostit()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null || Item.GetBaseItem().InteractionType != Butterfly_Emulator.HabboHotel.Items.InteractionType.postit)
            {
                return;
            }

            String Data = Request.PopFixedString();
            String Color = Data.Split(' ')[0];
            String Text = ButterflyEnvironment.FilterInjectionChars(Data.Substring(Color.Length + 1), true);

            if (!Room.CheckRights(Session))
            {
                if (!Data.StartsWith(Item.ExtraData))
                {
                    return; // we can only ADD stuff! older stuff changed, this is not allowed
                }
            }

            switch (Color)
            {
                case "FFFF33":
                case "FF9CFF":
                case "9CCEFF":
                case "9CFF9C":

                    break;

                default:

                    return; // invalid color
            }

            Item.ExtraData = Color + " " + Text;
            Item.UpdateState(true, true);
        }

        private void DeletePostit()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            RoomItem Item = Room.GetItem(Request.PopWiredUInt());

            if (Item == null || Item.GetBaseItem().InteractionType != Butterfly_Emulator.HabboHotel.Items.InteractionType.postit)
            {
                return;
            }

            Room.RemoveFurniture(Session, Item.Id);
        }

        private void OpenPresent()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            RoomItem Present = Room.GetItem(Request.PopWiredUInt());

            if (Present == null)
            {
                return;
            }

            DataRow Data = null;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                Data = dbClient.ReadDataRow("SELECT base_id,amount,extra_data FROM user_presents WHERE item_id = '" + Present.Id + "' LIMIT 1");
            }

            if (Data == null)
            {
                return;
            }

            Item BaseItem = ButterflyEnvironment.GetGame().GetItemManager().GetItem((uint)Data["base_id"]);

            if (BaseItem == null)
            {
                return;
            }

            Room.RemoveFurniture(Session, Present.Id);

            Response.Init(219);
            Response.AppendUInt(Present.Id);
            SendResponse();

            Response.Init(129);
            Response.AppendStringWithBreak(BaseItem.Type.ToString());
            Response.AppendInt32(BaseItem.SpriteId);
            Response.AppendStringWithBreak(BaseItem.Name);
            SendResponse();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("DELETE FROM user_presents WHERE item_id = '" + Present.Id + "' LIMIT 1");
            }

            ButterflyEnvironment.GetGame().GetCatalog().DeliverItems(Session, BaseItem, (int)Data["amount"], (String)Data["extra_data"]);
        }

        private void GetMoodlight()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.MoodlightData == null)
            {
                return;
            }

            Response.Init(365);
            Response.AppendInt32(Room.MoodlightData.Presets.Count);
            Response.AppendInt32(Room.MoodlightData.CurrentPreset);

                int i = 0;

                foreach (MoodlightPreset Preset in Room.MoodlightData.Presets)
                {
                    i++;

                    Response.AppendInt32(i);
                    Response.AppendInt32(int.Parse(ButterflyEnvironment.BoolToEnum(Preset.BackgroundOnly)) + 1);
                    Response.AppendStringWithBreak(Preset.ColorCode);
                    Response.AppendInt32(Preset.ColorIntensity);
                }
            

            SendResponse();
        }

        private void UpdateMoodlight()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.MoodlightData == null)
            {
                return;
            }

            RoomItem Item = null;

            foreach (RoomItem I in Room.WallItems.Values)
            {
                if (I.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.dimmer)
                {
                    Item = I;
                    break;
                }
            }

            if (Item == null)
            {
                return;
            }

            // EVIH@G#EA4532RbI

            int Preset = Request.PopWiredInt32();
            int BackgroundMode = Request.PopWiredInt32();
            string ColorCode = Request.PopFixedString();
            int Intensity = Request.PopWiredInt32();

            bool BackgroundOnly = false;

            if (BackgroundMode >= 2)
            {
                BackgroundOnly = true;
            }

            Room.MoodlightData.Enabled = true;
            Room.MoodlightData.CurrentPreset = Preset;
            Room.MoodlightData.UpdatePreset(Preset, ColorCode, Intensity, BackgroundOnly);

            Item.ExtraData = Room.MoodlightData.GenerateExtraData();
            Item.UpdateState();
        }

        private void SwitchMoodlightStatus()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true) || Room.MoodlightData == null)
            {
                return;
            }

            RoomItem Item = null;

            foreach (RoomItem I in Room.WallItems.Values)
            {
                if (I.GetBaseItem().InteractionType == Butterfly_Emulator.HabboHotel.Items.InteractionType.dimmer)
                {
                    Item = I;
                    break;
                }
            }

            if (Item == null)
            {
                return;
            }

            if (Room.MoodlightData.Enabled)
            {
                Room.MoodlightData.Disable();
            }
            else
            {
                Room.MoodlightData.Enable();
            }

            Item.ExtraData = Room.MoodlightData.GenerateExtraData();
            Item.UpdateState();
        }

        private void InitTrade()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Session.GetHabbo().Id);
            RoomUser User2 = Room.GetRoomUserByVirtualId(Request.PopWiredInt32());

            Room.TryStartTrade(User, User2);
        }

        private void OfferTradeItem()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Trade Trade = Room.GetUserTrade(Session.GetHabbo().Id);
            UserItem Item = Session.GetHabbo().GetInventoryComponent().GetItem(Request.PopWiredUInt());

            if (Trade == null || Item == null)
            {
                return;
            }

            Trade.OfferItem(Session.GetHabbo().Id, Item);
        }

        private void TakeBackTradeItem()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Trade Trade = Room.GetUserTrade(Session.GetHabbo().Id);
            UserItem Item = Session.GetHabbo().GetInventoryComponent().GetItem(Request.PopWiredUInt());

            if (Trade == null || Item == null)
            {
                return;
            }

            Trade.TakeBackItem(Session.GetHabbo().Id, Item);
        }

        private void StopTrade()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Room.TryStopTrade(Session.GetHabbo().Id);
        }

        private void AcceptTrade()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Trade Trade = Room.GetUserTrade(Session.GetHabbo().Id);

            if (Trade == null)
            {
                return;
            }

            Trade.Accept(Session.GetHabbo().Id);
        }

        private void UnacceptTrade()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Trade Trade = Room.GetUserTrade(Session.GetHabbo().Id);

            if (Trade == null)
            {
                return;
            }

            Trade.Unaccept(Session.GetHabbo().Id);
        }

        private void CompleteTrade()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CanTradeInRoom)
            {
                return;
            }

            Trade Trade = Room.GetUserTrade(Session.GetHabbo().Id);

            if (Trade == null)
            {
                return;
            }

            Trade.CompleteTrade(Session.GetHabbo().Id);
        }

        private void GiveRespect()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || Session.GetHabbo().DailyRespectPoints <= 0)
            {
                return;
            }

            RoomUser User = Room.GetRoomUserByHabbo(Request.PopWiredUInt());
            
            if (User == null || User.GetClient().GetHabbo().Id == Session.GetHabbo().Id || User.IsBot)
            {
                return;
            }

            Session.GetHabbo().DailyRespectPoints--;
            User.GetClient().GetHabbo().Respect++;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("UPDATE users SET respect = respect + 1 WHERE id = '" + User.GetClient().GetHabbo().Id + "' LIMIT 1");
                dbClient.ExecuteQuery("UPDATE users SET daily_respect_points = daily_respect_points - 1 WHERE id = '" + Session.GetHabbo().Id + "' LIMIT 1");
            }

            // FxkqUzYP_
            ServerMessage Message = new ServerMessage(440);
            Message.AppendUInt(User.GetClient().GetHabbo().Id);
            Message.AppendInt32(User.GetClient().GetHabbo().Respect);
            Room.SendMessage(Message);
        }

        private void ApplyEffect()
        {
            Session.GetHabbo().GetAvatarEffectsInventoryComponent().ApplyEffect(Request.PopWiredInt32());
        }

        private void EnableEffect()
        {
            Session.GetHabbo().GetAvatarEffectsInventoryComponent().EnableEffect(Request.PopWiredInt32());
        }

        private void RecycleItems()
        {
            if (!Session.GetHabbo().InRoom)
            {
                return;
            }

            int itemCount = Request.PopWiredInt32();

            if (itemCount != 5)
            {
                return;
            }

            for (int i = 0; i < itemCount; i++)
            {
                UserItem Item = Session.GetHabbo().GetInventoryComponent().GetItem(Request.PopWiredUInt());

                if (Item != null && Item.GetBaseItem().AllowRecycle)
                {
                    Session.GetHabbo().GetInventoryComponent().RemoveItem(Item.Id, false);
                }
                else
                {
                    return;
                }
            }

            uint newItemId = ButterflyEnvironment.GetGame().GetCatalog().GenerateItemId();
            EcotronReward Reward = ButterflyEnvironment.GetGame().GetCatalog().GetRandomEcotronReward();

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.ExecuteQuery("INSERT INTO user_items (id,user_id,base_item,extra_data) VALUES ('" + newItemId + "','" + Session.GetHabbo().Id + "','1478','" + DateTime.Now.ToLongDateString() + "')");
                dbClient.ExecuteQuery("INSERT INTO user_presents (item_id,base_id,amount,extra_data) VALUES ('" + newItemId + "','" + Reward.BaseId + "','1','')");
            }

            Session.GetHabbo().GetInventoryComponent().UpdateItems(true);

            Response.Init(508);
            Response.AppendBoolean(true);
            Response.AppendUInt(newItemId);
            SendResponse();
        }

        private void RedeemExchangeFurni()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            RoomItem Exchange = Room.GetItem(Request.PopWiredUInt());

            if (Exchange == null)
            {
                return;
            }

            if (!Exchange.GetBaseItem().Name.StartsWith("CF_") && !Exchange.GetBaseItem().Name.StartsWith("CFC_"))
            {
                return;
            }
            
            string[] Split = Exchange.GetBaseItem().Name.Split('_');
            int Value = int.Parse(Split[1]);

            if (Value > 0)
            {
                Session.GetHabbo().Credits += Value;
                Session.GetHabbo().UpdateCreditsBalance();
            }

            Room.RemoveFurniture(null, Exchange.Id);

            Response.Init(219);
            SendResponse();
        }

        private void EnterInfobus()
        {
            // AQThe Infobus is currently closed.
            Response.Init(81);
            Response.AppendStringWithBreak("The Infobus is not yet in use.");
            SendResponse();
        }

        private void KickBot()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session, true))
            {
                return;
            }

            RoomUser Bot = Room.GetRoomUserByVirtualId(Request.PopWiredInt32());

            if (Bot == null || !Bot.IsBot)
            {
                return;
            }

            Room.RemoveBot(Bot.VirtualId, true);
        }

        private void PlacePet()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || (!Room.AllowPets && !Room.CheckRights(Session, true)))
            {
                return;
            }

            uint PetId = Request.PopWiredUInt();

            Pet Pet = Session.GetHabbo().GetInventoryComponent().GetPet(PetId);

            if (Pet == null || Pet.PlacedInRoom)
            {
                return;
            }

            int X = Request.PopWiredInt32();
            int Y = Request.PopWiredInt32();

            if (!Room.CanWalk(X, Y, false))
            {
                return;
            }

            if (Room.PetCount >= ButterflyEnvironment.GetGame().GetRoomManager().MAX_PETS_PER_ROOM)
            {
                Session.SendNotif("There are too many pets in this room. A room may only contain up to " + ButterflyEnvironment.GetGame().GetRoomManager().MAX_PETS_PER_ROOM + " pets.");
                return;
            }

            Pet.PlacedInRoom = true;
            Pet.RoomId = Room.RoomId;

            List<RandomSpeech> RndSpeechList = new List<RandomSpeech>();
            List<BotResponse> BotResponse = new List<Butterfly.HabboHotel.RoomBots.BotResponse>();
            RoomUser PetUser = Room.DeployBot(new RoomBot(Pet.PetId, Pet.RoomId, AIType.Pet, "freeroam", Pet.Name, "", Pet.Look, X, Y, 0, 0, 0, 0, 0, 0, ref RndSpeechList, ref BotResponse), Pet);

            if (Room.CheckRights(Session, true))
            {
                Session.GetHabbo().GetInventoryComponent().MovePetToRoom(Pet.PetId);
            }
        }

        private void GetPetInfo()
        {
            uint PetId = Request.PopWiredUInt();

            DataRow Row = null;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("petid", PetId);
                Row = dbClient.ReadDataRow("SELECT id, user_id, room_id, name, type, race, color, expirience, energy, nutrition, respect, createstamp, x, y, z FROM user_pets WHERE id = @petid LIMIT 1");
            }

            if (Row == null)
            {
                return;
            }

            Session.SendMessage(Catalog.GeneratePetFromRow(Row).SerializeInfo());
        }

        private void PickUpPet()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || Room.IsPublic || (!Room.AllowPets && !Room.CheckRights(Session, true)))
            {
                return;
            }

            uint PetId = Request.PopWiredUInt();
            RoomUser PetUser = Room.GetPet(PetId);

            if (PetUser == null || PetUser.PetData == null || PetUser.PetData.OwnerId != Session.GetHabbo().Id)
            {
                return;
            }

            if (PetUser.PetData.DBState != DatabaseUpdateState.NeedsInsert)
                PetUser.PetData.DBState = DatabaseUpdateState.NeedsUpdate;

            Session.GetHabbo().GetInventoryComponent().AddPet(PetUser.PetData);
            Room.RemoveBot(PetUser.VirtualId, false);
        }

        private void RespectPet()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || Room.IsPublic || (!Room.AllowPets && !Room.CheckRights(Session, true)))
            {
                return;
            }

            uint PetId = Request.PopWiredUInt();
            RoomUser PetUser = Room.GetPet(PetId);

            if (PetUser == null || PetUser.PetData == null || PetUser.PetData.OwnerId != Session.GetHabbo().Id)
            {
                return;
            }

            PetUser.PetData.OnRespect();
            Session.GetHabbo().DailyPetRespectPoints--;

            using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            {
                dbClient.AddParamWithValue("userid", Session.GetHabbo().Id);
                dbClient.ExecuteQuery("UPDATE users SET daily_pet_respect_points = daily_pet_respect_points - 1 WHERE id = @userid LIMIT 1");
            }
        }

        private void SetLookTransfer()
        {
            Room Room = ButterflyEnvironment.GetGame().GetRoomManager().GetRoom(Session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(Session))
                return;

            uint ItemID = Request.PopWiredUInt();

            string Gender = Request.PopFixedString().ToUpper();
            string Look = ButterflyEnvironment.FilterInjectionChars(Request.PopFixedString());

            RoomItem RoomItemToSet = Room.FloorItems[ItemID] as RoomItem;

            RoomItemToSet.Figure = Look;
            RoomItemToSet.Gender = Gender;

        }

        internal void RegisterRooms()
        { 
            RequestHandlers.Add(391, new RequestHandler(OpenFlat));
            RequestHandlers.Add(182, new RequestHandler(GetAdvertisement));
            RequestHandlers.Add(388, new RequestHandler(GetPub));
            RequestHandlers.Add(2, new RequestHandler(OpenPub));
            RequestHandlers.Add(230, new RequestHandler(GetGroupBadges));
            RequestHandlers.Add(215, new RequestHandler(GetRoomData1));
            RequestHandlers.Add(390, new RequestHandler(GetRoomData2));
            RequestHandlers.Add(126, new RequestHandler(GetRoomData3));
            RequestHandlers.Add(52, new RequestHandler(Talk));
            RequestHandlers.Add(55, new RequestHandler(Shout));
            RequestHandlers.Add(56, new RequestHandler(Whisper));
            RequestHandlers.Add(75, new RequestHandler(Move));
            RequestHandlers.Add(387, new RequestHandler(CanCreateRoom));
            RequestHandlers.Add(29, new RequestHandler(CreateRoom));
            RequestHandlers.Add(400, new RequestHandler(GetRoomEditData));
            RequestHandlers.Add(386, new RequestHandler(SaveRoomIcon));
            RequestHandlers.Add(401, new RequestHandler(SaveRoomData));
            RequestHandlers.Add(96, new RequestHandler(GiveRights));
            RequestHandlers.Add(97, new RequestHandler(TakeRights));
            RequestHandlers.Add(155, new RequestHandler(TakeAllRights));
            RequestHandlers.Add(95, new RequestHandler(KickUser));
            RequestHandlers.Add(320, new RequestHandler(BanUser));
            RequestHandlers.Add(71, new RequestHandler(InitTrade));
            RequestHandlers.Add(384, new RequestHandler(SetHomeRoom));
            RequestHandlers.Add(23, new RequestHandler(DeleteRoom));
            RequestHandlers.Add(79, new RequestHandler(LookAt));
            RequestHandlers.Add(317, new RequestHandler(StartTyping));
            RequestHandlers.Add(318, new RequestHandler(StopTyping));
            RequestHandlers.Add(319, new RequestHandler(IgnoreUser));
            RequestHandlers.Add(322, new RequestHandler(UnignoreUser));
            RequestHandlers.Add(345, new RequestHandler(CanCreateRoomEvent));
            RequestHandlers.Add(346, new RequestHandler(StartEvent));
            RequestHandlers.Add(347, new RequestHandler(StopEvent));
            RequestHandlers.Add(348, new RequestHandler(EditEvent));
            RequestHandlers.Add(94, new RequestHandler(Wave));
            RequestHandlers.Add(263, new RequestHandler(GetUserTags));
            RequestHandlers.Add(159, new RequestHandler(GetUserBadges));
            RequestHandlers.Add(261, new RequestHandler(RateRoom));
            RequestHandlers.Add(93, new RequestHandler(Dance));
            RequestHandlers.Add(98, new RequestHandler(AnswerDoorbell));
            RequestHandlers.Add(59, new RequestHandler(ReqLoadRoomForUser));
            RequestHandlers.Add(66, new RequestHandler(ApplyRoomEffect));
            RequestHandlers.Add(90, new RequestHandler(PlaceItem));
            RequestHandlers.Add(67, new RequestHandler(TakeItem));
            RequestHandlers.Add(73, new RequestHandler(MoveItem));
            RequestHandlers.Add(392, new RequestHandler(TriggerItem)); // Generic trigger item
            RequestHandlers.Add(393, new RequestHandler(TriggerItem)); // Generic trigger item
            RequestHandlers.Add(83, new RequestHandler(OpenPostit));
            RequestHandlers.Add(84, new RequestHandler(SavePostit));
            RequestHandlers.Add(85, new RequestHandler(DeletePostit));
            RequestHandlers.Add(78, new RequestHandler(OpenPresent));
            RequestHandlers.Add(341, new RequestHandler(GetMoodlight));
            RequestHandlers.Add(342, new RequestHandler(UpdateMoodlight));
            RequestHandlers.Add(343, new RequestHandler(SwitchMoodlightStatus));
            RequestHandlers.Add(72, new RequestHandler(OfferTradeItem));
            RequestHandlers.Add(405, new RequestHandler(TakeBackTradeItem));
            RequestHandlers.Add(70, new RequestHandler(StopTrade));
            RequestHandlers.Add(403, new RequestHandler(StopTrade));
            RequestHandlers.Add(69, new RequestHandler(AcceptTrade));
            RequestHandlers.Add(68, new RequestHandler(UnacceptTrade));
            RequestHandlers.Add(402, new RequestHandler(CompleteTrade));
            RequestHandlers.Add(371, new RequestHandler(GiveRespect));
            RequestHandlers.Add(372, new RequestHandler(ApplyEffect));
            RequestHandlers.Add(373, new RequestHandler(EnableEffect));
            RequestHandlers.Add(232, new RequestHandler(TriggerItem)); // One way gates
            RequestHandlers.Add(314, new RequestHandler(TriggerItem)); // Love Shuffler
            RequestHandlers.Add(247, new RequestHandler(TriggerItem)); // Habbo Wheel
            RequestHandlers.Add(76, new RequestHandler(TriggerItem)); // Dice
            RequestHandlers.Add(77, new RequestHandler(TriggerItemDiceSpecial)); // Dice (special)
            RequestHandlers.Add(414, new RequestHandler(RecycleItems));
            RequestHandlers.Add(183, new RequestHandler(RedeemExchangeFurni));
            RequestHandlers.Add(113, new RequestHandler(EnterInfobus));
            RequestHandlers.Add(441, new RequestHandler(KickBot));
            RequestHandlers.Add(3002, new RequestHandler(PlacePet));
            RequestHandlers.Add(3001, new RequestHandler(GetPetInfo));
            RequestHandlers.Add(3003, new RequestHandler(PickUpPet));
            RequestHandlers.Add(3005, new RequestHandler(RespectPet));
            RequestHandlers.Add(480, new RequestHandler(SetLookTransfer));
        }
    }
}